@@ -25,7 +25,9 @@ public enum CoreMLFlags implements OrtFlags {
    * Create an MLProgram. By default it will create a NeuralNetwork model. Requires Core ML 5 or
    * later.
    */
-  CREATE_MLPROGRAM(16); // COREML_FLAG_CREATE_MLPROGRAM(0x010)
+  CREATE_MLPROGRAM(16), // COREML_FLAG_CREATE_MLPROGRAM(0x010)
+  /** exclude ANE */
+  CPU_AND_GPU(32); // COREML_FLAG_USE_CPU_AND_GPU(0x020)
 
   /** The native value of the enum. */
   public final int value;
