@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2021, 2024, Oracle and/or its affiliates. All rights reserved.
  * Licensed under the MIT License.
  */
 package ai.onnxruntime;
@@ -12,17 +12,20 @@ import ai.onnxruntime.platform.Fp16Conversions;
 import java.nio.ByteBuffer;
 import java.nio.ByteOrder;
 import java.nio.FloatBuffer;
+import java.nio.IntBuffer;
 import java.nio.ShortBuffer;
+import java.util.ArrayList;
 import java.util.Collections;
+import java.util.List;
 import java.util.SplittableRandom;
 import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.Test;
 
 public class OnnxTensorTest {
+  private static final OrtEnvironment env = TestHelpers.getOrtEnvironment();
 
   @Test
   public void testScalarCreation() throws OrtException {
-    OrtEnvironment env = OrtEnvironment.getEnvironment();
     String[] stringValues = new String[] {"true", "false"};
     for (String s : stringValues) {
       try (OnnxTensor t = OnnxTensor.createTensor(env, s)) {
@@ -93,30 +96,104 @@ public class OnnxTensorTest {
   }
 
   @Test
-  public void testBufferCreation() throws OrtException {
-    OrtEnvironment env = OrtEnvironment.getEnvironment();
-
-    // Test creating a value from an array
-    // Arrays result in tensors allocated by ORT, so they do not have a backing java.nio.Buffer
+  public void testArrayCreation() throws OrtException {
+    // Test creating a value from a single dimensional array
     float[] arrValues = new float[] {0, 1, 2, 3, 4};
     try (OnnxTensor t = OnnxTensor.createTensor(env, arrValues)) {
-      // array creation isn't backed by buffers
-      assertFalse(t.ownsBuffer());
-      assertFalse(t.getBufferRef().isPresent());
-      FloatBuffer buf = t.getFloatBuffer();
+      Assertions.assertTrue(t.ownsBuffer());
+      Assertions.assertTrue(t.getBufferRef().isPresent());
+      FloatBuffer buf = (FloatBuffer) t.getBufferRef().get();
       float[] output = new float[arrValues.length];
       buf.get(output);
       Assertions.assertArrayEquals(arrValues, output);
 
-      // Can't modify the tensor through this buffer.
+      // Can modify the tensor through this buffer.
       buf.put(0, 25);
-      Assertions.assertArrayEquals(arrValues, output);
+      Assertions.assertArrayEquals(new float[] {25, 1, 2, 3, 4}, (float[]) t.getValue());
     }
 
+    // Test creating a value from a multidimensional float array
+    float[][][] arr3dValues =
+        new float[][][] {
+          {{0, 1, 2}, {3, 4, 5}},
+          {{6, 7, 8}, {9, 10, 11}},
+          {{12, 13, 14}, {15, 16, 17}},
+          {{18, 19, 20}, {21, 22, 23}}
+        };
+    try (OnnxTensor t = OnnxTensor.createTensor(env, arr3dValues)) {
+      Assertions.assertArrayEquals(new long[] {4, 2, 3}, t.getInfo().getShape());
+      Assertions.assertTrue(t.ownsBuffer());
+      Assertions.assertTrue(t.getBufferRef().isPresent());
+      float[][][] output = (float[][][]) t.getValue();
+      Assertions.assertArrayEquals(arr3dValues, output);
+
+      // Can modify the tensor through the buffer.
+      FloatBuffer buf = (FloatBuffer) t.getBufferRef().get();
+      buf.put(0, 25);
+      buf.put(12, 32);
+      buf.put(13, 33);
+      buf.put(23, 35);
+      arr3dValues[0][0][0] = 25;
+      arr3dValues[2][0][0] = 32;
+      arr3dValues[2][0][1] = 33;
+      arr3dValues[3][1][2] = 35;
+      output = (float[][][]) t.getValue();
+      Assertions.assertArrayEquals(arr3dValues, output);
+    }
+
+    // Test creating a value from a multidimensional int array
+    int[][][] iArr3dValues =
+        new int[][][] {
+          {{0, 1, 2}, {3, 4, 5}},
+          {{6, 7, 8}, {9, 10, 11}},
+          {{12, 13, 14}, {15, 16, 17}},
+          {{18, 19, 20}, {21, 22, 23}}
+        };
+    try (OnnxTensor t = OnnxTensor.createTensor(env, iArr3dValues)) {
+      Assertions.assertArrayEquals(new long[] {4, 2, 3}, t.getInfo().getShape());
+      Assertions.assertTrue(t.ownsBuffer());
+      Assertions.assertTrue(t.getBufferRef().isPresent());
+      int[][][] output = (int[][][]) t.getValue();
+      Assertions.assertArrayEquals(iArr3dValues, output);
+
+      // Can modify the tensor through the buffer.
+      IntBuffer buf = (IntBuffer) t.getBufferRef().get();
+      buf.put(0, 25);
+      iArr3dValues[0][0][0] = 25;
+      output = (int[][][]) t.getValue();
+      Assertions.assertArrayEquals(iArr3dValues, output);
+    }
+
+    // Test creating a value from a ragged array throws
+    int[][][] ragged =
+        new int[][][] {
+          {{0, 1, 2}, {3, 4, 5}},
+          {{6, 7, 8}},
+          {{12, 13}, {15, 16, 17}},
+          {{18, 19, 20}, {21, 22, 23}}
+        };
+    try (OnnxTensor t = OnnxTensor.createTensor(env, ragged)) {
+      Assertions.fail("Can't create tensors from ragged arrays");
+    } catch (OrtException e) {
+      Assertions.assertTrue(e.getMessage().contains("ragged"));
+    }
+
+    // Test creating a value from a non-array, non-primitive type throws.
+    List<Integer> list = new ArrayList<>(5);
+    list.add(5);
+    try (OnnxTensor t = OnnxTensor.createTensor(env, list)) {
+      Assertions.fail("Can't create tensors from lists");
+    } catch (OrtException e) {
+      Assertions.assertTrue(e.getMessage().contains("Cannot convert"));
+    }
+  }
+
+  @Test
+  public void testBufferCreation() throws OrtException {
     // Test creating a value from a non-direct byte buffer
     // Non-direct byte buffers are allocated on the Java heap and must be copied into off-heap
-    // direct byte buffers
-    // which can be directly passed to ORT
+    // direct byte buffers which can be directly passed to ORT
+    float[] arrValues = new float[] {0, 1, 2, 3, 4};
     FloatBuffer nonDirectBuffer = FloatBuffer.allocate(5);
     nonDirectBuffer.put(arrValues);
     nonDirectBuffer.rewind();
@@ -179,7 +256,6 @@ public class OnnxTensorTest {
 
   @Test
   public void testStringCreation() throws OrtException {
-    OrtEnvironment env = OrtEnvironment.getEnvironment();
     String[] arrValues = new String[] {"this", "is", "a", "single", "dimensional", "string"};
     try (OnnxTensor t = OnnxTensor.createTensor(env, arrValues)) {
       Assertions.assertArrayEquals(new long[] {6}, t.getInfo().shape);
@@ -209,7 +285,6 @@ public class OnnxTensorTest {
 
   @Test
   public void testUint8Creation() throws OrtException {
-    OrtEnvironment env = OrtEnvironment.getEnvironment();
     byte[] buf = new byte[] {0, 1};
     ByteBuffer data = ByteBuffer.wrap(buf);
     long[] shape = new long[] {2};
@@ -218,9 +293,31 @@ public class OnnxTensorTest {
     }
   }
 
+  @Test
+  public void testByteBufferCreation() throws OrtException {
+    ByteBuffer byteBuf = ByteBuffer.allocateDirect(Float.BYTES * 5).order(ByteOrder.nativeOrder());
+    FloatBuffer floatBuf = byteBuf.asFloatBuffer();
+    floatBuf.put(1.0f);
+    floatBuf.put(2.0f);
+    floatBuf.put(3.0f);
+    floatBuf.put(4.0f);
+    floatBuf.put(5.0f);
+    floatBuf.position(1);
+    float[] expected = new float[floatBuf.remaining()];
+    floatBuf.get(expected);
+    floatBuf.position(1);
+    byteBuf.position(4);
+    try (OnnxTensor t =
+        OnnxTensor.createTensor(
+            env, byteBuf, new long[] {floatBuf.remaining()}, OnnxJavaType.FLOAT)) {
+      Assertions.assertNotNull(t);
+      float[] actual = (float[]) t.getValue();
+      Assertions.assertArrayEquals(expected, actual);
+    }
+  }
+
   @Test
   public void testEmptyTensor() throws OrtException {
-    OrtEnvironment env = OrtEnvironment.getEnvironment();
     FloatBuffer buf = FloatBuffer.allocate(0);
     long[] shape = new long[] {4, 0};
     try (OnnxTensor t = OnnxTensor.createTensor(env, buf, shape)) {
@@ -241,7 +338,6 @@ public class OnnxTensorTest {
 
   @Test
   public void testBf16ToFp32() throws OrtException {
-    OrtEnvironment env = OrtEnvironment.getEnvironment();
     String modelPath = TestHelpers.getResourcePath("/java-bf16-to-fp32.onnx").toString();
     SplittableRandom rng = new SplittableRandom(1);
 
@@ -274,7 +370,6 @@ public class OnnxTensorTest {
 
   @Test
   public void testFp16ToFp32() throws OrtException {
-    OrtEnvironment env = OrtEnvironment.getEnvironment();
     String modelPath = TestHelpers.getResourcePath("/java-fp16-to-fp32.onnx").toString();
     SplittableRandom rng = new SplittableRandom(1);
 
@@ -307,14 +402,15 @@ public class OnnxTensorTest {
 
   @Test
   public void testFp32ToFp16() throws OrtException {
-    OrtEnvironment env = OrtEnvironment.getEnvironment();
     String modelPath = TestHelpers.getResourcePath("/java-fp32-to-fp16.onnx").toString();
     SplittableRandom rng = new SplittableRandom(1);
 
-    float[][] input = new float[10][5];
+    int dim1 = 10, dim2 = 5;
+    float[][] input = new float[dim1][dim2];
+    float[][] expectedOutput = new float[dim1][dim2];
     FloatBuffer floatBuf =
-        ByteBuffer.allocateDirect(4 * 10 * 5).order(ByteOrder.nativeOrder()).asFloatBuffer();
-    ShortBuffer shortBuf = ShortBuffer.allocate(10 * 5);
+        ByteBuffer.allocateDirect(4 * dim1 * dim2).order(ByteOrder.nativeOrder()).asFloatBuffer();
+    ShortBuffer shortBuf = ShortBuffer.allocate(dim1 * dim2);
 
     // Generate data
     for (int i = 0; i < input.length; i++) {
@@ -323,6 +419,8 @@ public class OnnxTensorTest {
         input[i][j] = Float.intBitsToFloat(bits);
         floatBuf.put(input[i][j]);
         shortBuf.put(Fp16Conversions.floatToFp16(input[i][j]));
+        expectedOutput[i][j] =
+            Fp16Conversions.fp16ToFloat(Fp16Conversions.floatToFp16(input[i][j]));
       }
     }
     floatBuf.rewind();
@@ -330,38 +428,45 @@ public class OnnxTensorTest {
 
     try (OrtSession.SessionOptions opts = new OrtSession.SessionOptions();
         OrtSession session = env.createSession(modelPath, opts);
-        OnnxTensor tensor = OnnxTensor.createTensor(env, floatBuf, new long[] {10, 5});
+        OnnxTensor tensor = OnnxTensor.createTensor(env, floatBuf, new long[] {dim1, dim2});
         OrtSession.Result result = session.run(Collections.singletonMap("input", tensor))) {
       OnnxTensor output = (OnnxTensor) result.get(0);
 
       // Check outbound Java side cast to fp32 works
       FloatBuffer castOutput = output.getFloatBuffer();
-      float[] expectedFloatArr = new float[10 * 5];
+      float[] expectedFloatArr = new float[dim1 * dim2];
       Fp16Conversions.convertFp16BufferToFloatBuffer(shortBuf).get(expectedFloatArr);
-      float[] actualFloatArr = new float[10 * 5];
+      float[] actualFloatArr = new float[dim1 * dim2];
       castOutput.get(actualFloatArr);
       Assertions.assertArrayEquals(expectedFloatArr, actualFloatArr);
 
       // Check bits are correct
       ShortBuffer outputBuf = output.getShortBuffer();
-      short[] expectedShortArr = new short[10 * 5];
+      short[] expectedShortArr = new short[dim1 * dim2];
       shortBuf.get(expectedShortArr);
-      short[] actualShortArr = new short[10 * 5];
+      short[] actualShortArr = new short[dim1 * dim2];
       outputBuf.get(actualShortArr);
       Assertions.assertArrayEquals(expectedShortArr, actualShortArr);
+
+      // Check outbound fp16 -> float[] conversion
+      float[][] floats = (float[][]) output.getValue();
+      for (int i = 0; i < dim1; i++) {
+        Assertions.assertArrayEquals(expectedOutput[i], floats[i]);
+      }
     }
   }
 
   @Test
   public void testFp32ToBf16() throws OrtException {
-    OrtEnvironment env = OrtEnvironment.getEnvironment();
     String modelPath = TestHelpers.getResourcePath("/java-fp32-to-bf16.onnx").toString();
     SplittableRandom rng = new SplittableRandom(1);
 
-    float[][] input = new float[10][5];
+    int dim1 = 10, dim2 = 5;
+    float[][] input = new float[dim1][dim2];
+    float[][] expectedOutput = new float[dim1][dim2];
     FloatBuffer floatBuf =
-        ByteBuffer.allocateDirect(4 * 10 * 5).order(ByteOrder.nativeOrder()).asFloatBuffer();
-    ShortBuffer shortBuf = ShortBuffer.allocate(10 * 5);
+        ByteBuffer.allocateDirect(4 * dim1 * dim2).order(ByteOrder.nativeOrder()).asFloatBuffer();
+    ShortBuffer shortBuf = ShortBuffer.allocate(dim1 * dim2);
 
     // Generate data
     for (int i = 0; i < input.length; i++) {
@@ -370,6 +475,8 @@ public class OnnxTensorTest {
         input[i][j] = Float.intBitsToFloat(bits);
         floatBuf.put(input[i][j]);
         shortBuf.put(Fp16Conversions.floatToBf16(input[i][j]));
+        expectedOutput[i][j] =
+            Fp16Conversions.bf16ToFloat(Fp16Conversions.floatToBf16(input[i][j]));
       }
     }
     floatBuf.rewind();
@@ -377,25 +484,31 @@ public class OnnxTensorTest {
 
     try (OrtSession.SessionOptions opts = new OrtSession.SessionOptions();
         OrtSession session = env.createSession(modelPath, opts);
-        OnnxTensor tensor = OnnxTensor.createTensor(env, floatBuf, new long[] {10, 5});
+        OnnxTensor tensor = OnnxTensor.createTensor(env, floatBuf, new long[] {dim1, dim2});
         OrtSession.Result result = session.run(Collections.singletonMap("input", tensor))) {
       OnnxTensor output = (OnnxTensor) result.get(0);
 
       // Check outbound Java side cast to fp32 works
       FloatBuffer castOutput = output.getFloatBuffer();
-      float[] expectedFloatArr = new float[10 * 5];
+      float[] expectedFloatArr = new float[dim1 * dim2];
       Fp16Conversions.convertBf16BufferToFloatBuffer(shortBuf).get(expectedFloatArr);
-      float[] actualFloatArr = new float[10 * 5];
+      float[] actualFloatArr = new float[dim1 * dim2];
       castOutput.get(actualFloatArr);
       Assertions.assertArrayEquals(expectedFloatArr, actualFloatArr);
 
       // Check bits are correct
       ShortBuffer outputBuf = output.getShortBuffer();
-      short[] expectedShortArr = new short[10 * 5];
+      short[] expectedShortArr = new short[dim1 * dim2];
       shortBuf.get(expectedShortArr);
-      short[] actualShortArr = new short[10 * 5];
+      short[] actualShortArr = new short[dim1 * dim2];
       outputBuf.get(actualShortArr);
       Assertions.assertArrayEquals(expectedShortArr, actualShortArr);
+
+      // Check outbound bf16 -> float[] conversion
+      float[][] floats = (float[][]) output.getValue();
+      for (int i = 0; i < dim1; i++) {
+        Assertions.assertArrayEquals(expectedOutput[i], floats[i]);
+      }
     }
   }
 
@@ -435,7 +548,6 @@ public class OnnxTensorTest {
 
   @Test
   public void testClose() throws OrtException {
-    OrtEnvironment env = OrtEnvironment.getEnvironment();
     long[] input = new long[] {1, 2, 3, 4, 5};
     OnnxTensor value = OnnxTensor.createTensor(env, input);
     assertFalse(value.isClosed());
