@@ -1,12 +1,12 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License.
 
-import {resolveBackendAndExecutionProviders} from './backend-impl.js';
-import {InferenceSessionHandler} from './backend.js';
-import {InferenceSession as InferenceSessionInterface} from './inference-session.js';
-import {OnnxValue} from './onnx-value.js';
-import {Tensor} from './tensor.js';
-import {TRACE_FUNC_BEGIN, TRACE_FUNC_END} from './trace.js';
+import { resolveBackendAndExecutionProviders } from './backend-impl.js';
+import { InferenceSessionHandler } from './backend.js';
+import { InferenceSession as InferenceSessionInterface } from './inference-session.js';
+import { OnnxValue } from './onnx-value.js';
+import { Tensor } from './tensor.js';
+import { TRACE_FUNC_BEGIN, TRACE_FUNC_END } from './trace.js';
 
 type SessionOptions = InferenceSessionInterface.SessionOptions;
 type RunOptions = InferenceSessionInterface.RunOptions;
@@ -20,14 +20,15 @@ export class InferenceSession implements InferenceSessionInterface {
   }
   run(feeds: FeedsType, options?: RunOptions): Promise<ReturnType>;
   run(feeds: FeedsType, fetches: FetchesType, options?: RunOptions): Promise<ReturnType>;
-  async run(feeds: FeedsType, arg1?: FetchesType|RunOptions, arg2?: RunOptions): Promise<ReturnType> {
+  async run(feeds: FeedsType, arg1?: FetchesType | RunOptions, arg2?: RunOptions): Promise<ReturnType> {
     TRACE_FUNC_BEGIN();
-    const fetches: {[name: string]: OnnxValue|null} = {};
+    const fetches: { [name: string]: OnnxValue | null } = {};
     let options: RunOptions = {};
     // check inputs
     if (typeof feeds !== 'object' || feeds === null || feeds instanceof Tensor || Array.isArray(feeds)) {
       throw new TypeError(
-          '\'feeds\' must be an object that use input names as keys and OnnxValue as corresponding values.');
+        "'feeds' must be an object that use input names as keys and OnnxValue as corresponding values.",
+      );
     }
 
     let isFetchesEmpty = true;
@@ -37,18 +38,18 @@ export class InferenceSession implements InferenceSessionInterface {
         throw new TypeError('Unexpected argument[1]: cannot be null.');
       }
       if (arg1 instanceof Tensor) {
-        throw new TypeError('\'fetches\' cannot be a Tensor');
+        throw new TypeError("'fetches' cannot be a Tensor");
       }
 
       if (Array.isArray(arg1)) {
         if (arg1.length === 0) {
-          throw new TypeError('\'fetches\' cannot be an empty array.');
+          throw new TypeError("'fetches' cannot be an empty array.");
         }
         isFetchesEmpty = false;
         // output names
         for (const name of arg1) {
           if (typeof name !== 'string') {
-            throw new TypeError('\'fetches\' must be a string array or an object.');
+            throw new TypeError("'fetches' must be a string array or an object.");
           }
           if (this.outputNames.indexOf(name) === -1) {
             throw new RangeError(`'fetches' contains invalid output name: ${name}.`);
@@ -59,7 +60,7 @@ export class InferenceSession implements InferenceSessionInterface {
         if (typeof arg2 === 'object' && arg2 !== null) {
           options = arg2;
         } else if (typeof arg2 !== 'undefined') {
-          throw new TypeError('\'options\' must be an object.');
+          throw new TypeError("'options' must be an object.");
         }
       } else {
         // decide whether arg1 is fetches or options
@@ -81,14 +82,14 @@ export class InferenceSession implements InferenceSessionInterface {
           if (typeof arg2 === 'object' && arg2 !== null) {
             options = arg2;
           } else if (typeof arg2 !== 'undefined') {
-            throw new TypeError('\'options\' must be an object.');
+            throw new TypeError("'options' must be an object.");
           }
         } else {
           options = arg1 as RunOptions;
         }
       }
     } else if (typeof arg1 !== 'undefined') {
-      throw new TypeError('Unexpected argument[1]: must be \'fetches\' or \'options\'.');
+      throw new TypeError("Unexpected argument[1]: must be 'fetches' or 'options'.");
     }
 
     // check if all inputs are in feed
@@ -108,7 +109,7 @@ export class InferenceSession implements InferenceSessionInterface {
     // feeds, fetches and options are prepared
 
     const results = await this.handler.run(feeds, fetches, options);
-    const returnValue: {[name: string]: OnnxValue} = {};
+    const returnValue: { [name: string]: OnnxValue } = {};
     for (const key in results) {
       if (Object.hasOwnProperty.call(results, key)) {
         const result = results[key];
@@ -129,15 +130,22 @@ export class InferenceSession implements InferenceSessionInterface {
 
   static create(path: string, options?: SessionOptions): Promise<InferenceSessionInterface>;
   static create(buffer: ArrayBufferLike, options?: SessionOptions): Promise<InferenceSessionInterface>;
-  static create(buffer: ArrayBufferLike, byteOffset: number, byteLength?: number, options?: SessionOptions):
-      Promise<InferenceSessionInterface>;
+  static create(
+    buffer: ArrayBufferLike,
+    byteOffset: number,
+    byteLength?: number,
+    options?: SessionOptions,
+  ): Promise<InferenceSessionInterface>;
   static create(buffer: Uint8Array, options?: SessionOptions): Promise<InferenceSessionInterface>;
   static async create(
-      arg0: string|ArrayBufferLike|Uint8Array, arg1?: SessionOptions|number, arg2?: number,
-      arg3?: SessionOptions): Promise<InferenceSessionInterface> {
+    arg0: string | ArrayBufferLike | Uint8Array,
+    arg1?: SessionOptions | number,
+    arg2?: number,
+    arg3?: SessionOptions,
+  ): Promise<InferenceSessionInterface> {
     TRACE_FUNC_BEGIN();
     // either load from a file or buffer
-    let filePathOrUint8Array: string|Uint8Array;
+    let filePathOrUint8Array: string | Uint8Array;
     let options: SessionOptions = {};
 
     if (typeof arg0 === 'string') {
@@ -145,18 +153,19 @@ export class InferenceSession implements InferenceSessionInterface {
       if (typeof arg1 === 'object' && arg1 !== null) {
         options = arg1;
       } else if (typeof arg1 !== 'undefined') {
-        throw new TypeError('\'options\' must be an object.');
+        throw new TypeError("'options' must be an object.");
       }
     } else if (arg0 instanceof Uint8Array) {
       filePathOrUint8Array = arg0;
       if (typeof arg1 === 'object' && arg1 !== null) {
         options = arg1;
       } else if (typeof arg1 !== 'undefined') {
-        throw new TypeError('\'options\' must be an object.');
+        throw new TypeError("'options' must be an object.");
       }
     } else if (
-        arg0 instanceof ArrayBuffer ||
-        (typeof SharedArrayBuffer !== 'undefined' && arg0 instanceof SharedArrayBuffer)) {
+      arg0 instanceof ArrayBuffer ||
+      (typeof SharedArrayBuffer !== 'undefined' && arg0 instanceof SharedArrayBuffer)
+    ) {
       const buffer = arg0;
       let byteOffset = 0;
       let byteLength = arg0.byteLength;
@@ -165,7 +174,7 @@ export class InferenceSession implements InferenceSessionInterface {
       } else if (typeof arg1 === 'number') {
         byteOffset = arg1;
         if (!Number.isSafeInteger(byteOffset)) {
-          throw new RangeError('\'byteOffset\' must be an integer.');
+          throw new RangeError("'byteOffset' must be an integer.");
         }
         if (byteOffset < 0 || byteOffset >= buffer.byteLength) {
           throw new RangeError(`'byteOffset' is out of range [0, ${buffer.byteLength}).`);
@@ -174,7 +183,7 @@ export class InferenceSession implements InferenceSessionInterface {
         if (typeof arg2 === 'number') {
           byteLength = arg2;
           if (!Number.isSafeInteger(byteLength)) {
-            throw new RangeError('\'byteLength\' must be an integer.');
+            throw new RangeError("'byteLength' must be an integer.");
           }
           if (byteLength <= 0 || byteOffset + byteLength > buffer.byteLength) {
             throw new RangeError(`'byteLength' is out of range (0, ${buffer.byteLength - byteOffset}].`);
@@ -182,17 +191,17 @@ export class InferenceSession implements InferenceSessionInterface {
           if (typeof arg3 === 'object' && arg3 !== null) {
             options = arg3;
           } else if (typeof arg3 !== 'undefined') {
-            throw new TypeError('\'options\' must be an object.');
+            throw new TypeError("'options' must be an object.");
           }
         } else if (typeof arg2 !== 'undefined') {
-          throw new TypeError('\'byteLength\' must be a number.');
+          throw new TypeError("'byteLength' must be a number.");
         }
       } else if (typeof arg1 !== 'undefined') {
-        throw new TypeError('\'options\' must be an object.');
+        throw new TypeError("'options' must be an object.");
       }
       filePathOrUint8Array = new Uint8Array(buffer, byteOffset, byteLength);
     } else {
-      throw new TypeError('Unexpected argument[0]: must be \'path\' or \'buffer\'.');
+      throw new TypeError("Unexpected argument[0]: must be 'path' or 'buffer'.");
     }
 
     // resolve backend, update session options with validated EPs, and create session handler
