@@ -1,18 +1,20 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License.
 
-import {TensorToDataUrlOptions, TensorToImageDataOptions} from './tensor-conversion.js';
-import {Tensor} from './tensor.js';
+import { TensorToDataUrlOptions, TensorToImageDataOptions } from './tensor-conversion.js';
+import { Tensor } from './tensor.js';
 
 /**
  * implementation of Tensor.toDataURL()
  */
 export const tensorToDataURL = (tensor: Tensor, options?: TensorToDataUrlOptions): string => {
-  const canvas = typeof document !== 'undefined' ? document.createElement('canvas') : (new OffscreenCanvas(1, 1));
+  const canvas = typeof document !== 'undefined' ? document.createElement('canvas') : new OffscreenCanvas(1, 1);
   canvas.width = tensor.dims[3];
   canvas.height = tensor.dims[2];
-  const pixels2DContext =
-      canvas.getContext('2d') as (CanvasRenderingContext2D | OffscreenCanvasRenderingContext2D | null);
+  const pixels2DContext = canvas.getContext('2d') as
+    | CanvasRenderingContext2D
+    | OffscreenCanvasRenderingContext2D
+    | null;
 
   if (pixels2DContext != null) {
     // Default values for height and width & format
@@ -21,7 +23,8 @@ export const tensorToDataURL = (tensor: Tensor, options?: TensorToDataUrlOptions
     if (options?.tensorLayout !== undefined && options.tensorLayout === 'NHWC') {
       width = tensor.dims[2];
       height = tensor.dims[3];
-    } else {  // Default layout is NCWH
+    } else {
+      // Default layout is NCWH
       width = tensor.dims[3];
       height = tensor.dims[2];
     }
@@ -34,7 +37,7 @@ export const tensorToDataURL = (tensor: Tensor, options?: TensorToDataUrlOptions
     if (norm === undefined || norm.mean === undefined) {
       normMean = [255, 255, 255, 255];
     } else {
-      if (typeof (norm.mean) === 'number') {
+      if (typeof norm.mean === 'number') {
         normMean = [norm.mean, norm.mean, norm.mean, norm.mean];
       } else {
         normMean = [norm.mean[0], norm.mean[1], norm.mean[2], 0];
@@ -46,7 +49,7 @@ export const tensorToDataURL = (tensor: Tensor, options?: TensorToDataUrlOptions
     if (norm === undefined || norm.bias === undefined) {
       normBias = [0, 0, 0, 0];
     } else {
-      if (typeof (norm.bias) === 'number') {
+      if (typeof norm.bias === 'number') {
         normBias = [norm.bias, norm.bias, norm.bias, norm.bias];
       } else {
         normBias = [norm.bias[0], norm.bias[1], norm.bias[2], 0];
@@ -58,7 +61,10 @@ export const tensorToDataURL = (tensor: Tensor, options?: TensorToDataUrlOptions
 
     const stride = height * width;
     // Default pointer assignments
-    let rTensorPointer = 0, gTensorPointer = stride, bTensorPointer = stride * 2, aTensorPointer = -1;
+    let rTensorPointer = 0,
+      gTensorPointer = stride,
+      bTensorPointer = stride * 2,
+      aTensorPointer = -1;
 
     // Updating the pointer assignments based on the input image format
     if (inputformat === 'RGBA') {
@@ -78,12 +84,10 @@ export const tensorToDataURL = (tensor: Tensor, options?: TensorToDataUrlOptions
 
     for (let i = 0; i < height; i++) {
       for (let j = 0; j < width; j++) {
-        const R = ((tensor.data[rTensorPointer++] as number) - normBias[0]) * normMean[0];  // R value
-        const G = ((tensor.data[gTensorPointer++] as number) - normBias[1]) * normMean[1];  // G value
-        const B = ((tensor.data[bTensorPointer++] as number) - normBias[2]) * normMean[2];  // B value
-        const A = aTensorPointer === -1 ?
-            255 :
-            ((tensor.data[aTensorPointer++] as number) - normBias[3]) * normMean[3];  // A value
+        const R = ((tensor.data[rTensorPointer++] as number) - normBias[0]) * normMean[0]; // R value
+        const G = ((tensor.data[gTensorPointer++] as number) - normBias[1]) * normMean[1]; // G value
+        const B = ((tensor.data[bTensorPointer++] as number) - normBias[2]) * normMean[2]; // B value
+        const A = aTensorPointer === -1 ? 255 : ((tensor.data[aTensorPointer++] as number) - normBias[3]) * normMean[3]; // A value
         // eslint-disable-next-line @typescript-eslint/restrict-plus-operands
         pixels2DContext.fillStyle = 'rgba(' + R + ',' + G + ',' + B + ',' + A + ')';
         pixels2DContext.fillRect(j, i, 1, 1);
@@ -103,9 +107,10 @@ export const tensorToDataURL = (tensor: Tensor, options?: TensorToDataUrlOptions
  * implementation of Tensor.toImageData()
  */
 export const tensorToImageData = (tensor: Tensor, options?: TensorToImageDataOptions): ImageData => {
-  const pixels2DContext = typeof document !== 'undefined' ?
-      document.createElement('canvas').getContext('2d') :
-      new OffscreenCanvas(1, 1).getContext('2d') as OffscreenCanvasRenderingContext2D;
+  const pixels2DContext =
+    typeof document !== 'undefined'
+      ? document.createElement('canvas').getContext('2d')
+      : (new OffscreenCanvas(1, 1).getContext('2d') as OffscreenCanvasRenderingContext2D);
   let image: ImageData;
   if (pixels2DContext != null) {
     // Default values for height and width & format
@@ -116,7 +121,8 @@ export const tensorToImageData = (tensor: Tensor, options?: TensorToImageDataOpt
       width = tensor.dims[2];
       height = tensor.dims[1];
       channels = tensor.dims[3];
-    } else {  // Default layout is NCWH
+    } else {
+      // Default layout is NCWH
       width = tensor.dims[3];
       height = tensor.dims[2];
       channels = tensor.dims[1];
@@ -129,7 +135,7 @@ export const tensorToImageData = (tensor: Tensor, options?: TensorToImageDataOpt
     if (norm === undefined || norm.mean === undefined) {
       normMean = [255, 255, 255, 255];
     } else {
-      if (typeof (norm.mean) === 'number') {
+      if (typeof norm.mean === 'number') {
         normMean = [norm.mean, norm.mean, norm.mean, norm.mean];
       } else {
         normMean = [norm.mean[0], norm.mean[1], norm.mean[2], 255];
@@ -141,7 +147,7 @@ export const tensorToImageData = (tensor: Tensor, options?: TensorToImageDataOpt
     if (norm === undefined || norm.bias === undefined) {
       normBias = [0, 0, 0, 0];
     } else {
-      if (typeof (norm.bias) === 'number') {
+      if (typeof norm.bias === 'number') {
         normBias = [norm.bias, norm.bias, norm.bias, norm.bias];
       } else {
         normBias = [norm.bias[0], norm.bias[1], norm.bias[2], 0];
@@ -153,16 +159,24 @@ export const tensorToImageData = (tensor: Tensor, options?: TensorToImageDataOpt
 
     const stride = height * width;
     if (options !== undefined) {
-      if (options.format !== undefined && (channels === 4 && options.format !== 'RGBA') ||
-          (channels === 3 && (options.format !== 'RGB' && options.format !== 'BGR'))) {
-        throw new Error('Tensor format doesn\'t match input tensor dims');
+      if (
+        (options.format !== undefined && channels === 4 && options.format !== 'RGBA') ||
+        (channels === 3 && options.format !== 'RGB' && options.format !== 'BGR')
+      ) {
+        throw new Error("Tensor format doesn't match input tensor dims");
       }
     }
 
     // Default pointer assignments
     const step = 4;
-    let rImagePointer = 0, gImagePointer = 1, bImagePointer = 2, aImagePointer = 3;
-    let rTensorPointer = 0, gTensorPointer = stride, bTensorPointer = stride * 2, aTensorPointer = -1;
+    let rImagePointer = 0,
+      gImagePointer = 1,
+      bImagePointer = 2,
+      aImagePointer = 3;
+    let rTensorPointer = 0,
+      gTensorPointer = stride,
+      bTensorPointer = stride * 2,
+      aTensorPointer = -1;
 
     // Updating the pointer assignments based on the input image format
     if (inputformat === 'RGBA') {
@@ -182,16 +196,17 @@ export const tensorToImageData = (tensor: Tensor, options?: TensorToImageDataOpt
 
     image = pixels2DContext.createImageData(width, height);
 
-    for (let i = 0; i < height * width;
-         rImagePointer += step, gImagePointer += step, bImagePointer += step, aImagePointer += step, i++) {
-      image.data[rImagePointer] = ((tensor.data[rTensorPointer++] as number) - normBias[0]) * normMean[0];  // R value
-      image.data[gImagePointer] = ((tensor.data[gTensorPointer++] as number) - normBias[1]) * normMean[1];  // G value
-      image.data[bImagePointer] = ((tensor.data[bTensorPointer++] as number) - normBias[2]) * normMean[2];  // B value
-      image.data[aImagePointer] = aTensorPointer === -1 ?
-          255 :
-          ((tensor.data[aTensorPointer++] as number) - normBias[3]) * normMean[3];  // A value
+    for (
+      let i = 0;
+      i < height * width;
+      rImagePointer += step, gImagePointer += step, bImagePointer += step, aImagePointer += step, i++
+    ) {
+      image.data[rImagePointer] = ((tensor.data[rTensorPointer++] as number) - normBias[0]) * normMean[0]; // R value
+      image.data[gImagePointer] = ((tensor.data[gTensorPointer++] as number) - normBias[1]) * normMean[1]; // G value
+      image.data[bImagePointer] = ((tensor.data[bTensorPointer++] as number) - normBias[2]) * normMean[2]; // B value
+      image.data[aImagePointer] =
+        aTensorPointer === -1 ? 255 : ((tensor.data[aTensorPointer++] as number) - normBias[3]) * normMean[3]; // A value
     }
-
   } else {
     throw new Error('Can not access image data');
   }
