@@ -9,8 +9,10 @@ class AttributeWithCacheKeyImpl {
   private key: string;
   public get cacheKey(): string {
     if (!this.key) {
-      this.key =
-          Object.getOwnPropertyNames(this).sort().map(name => `${(this as Record<string, unknown>)[name]}`).join(';');
+      this.key = Object.getOwnPropertyNames(this)
+        .sort()
+        .map((name) => `${(this as Record<string, unknown>)[name]}`)
+        .join(';');
     }
     return this.key;
   }
@@ -20,5 +22,6 @@ export interface AttributeWithCacheKey {
   readonly cacheKey: string;
 }
 
-export const createAttributeWithCacheKey = <T extends Record<string, unknown>>(attribute: T): T&AttributeWithCacheKey =>
-    new AttributeWithCacheKeyImpl(attribute) as unknown as T & AttributeWithCacheKey;
+export const createAttributeWithCacheKey = <T extends Record<string, unknown>>(
+  attribute: T,
+): T & AttributeWithCacheKey => new AttributeWithCacheKeyImpl(attribute) as unknown as T & AttributeWithCacheKey;
