@@ -1,22 +1,25 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License.
 
-import {Graph} from '../../../graph';
-import {OperatorImplementation, OperatorInitialization} from '../../../operators';
-import {Tensor} from '../../../tensor';
-import {ShapeUtil} from '../../../util';
-import {WebGLInferenceHandler} from '../inference-handler';
+import { Graph } from '../../../graph';
+import { OperatorImplementation, OperatorInitialization } from '../../../operators';
+import { Tensor } from '../../../tensor';
+import { ShapeUtil } from '../../../util';
+import { WebGLInferenceHandler } from '../inference-handler';
 
-export const flatten: OperatorImplementation<number> =
-    (inferenceHandler: WebGLInferenceHandler, inputs: Tensor[], axis: number): Tensor[] => {
-      validateInputs(inputs, axis);
+export const flatten: OperatorImplementation<number> = (
+  inferenceHandler: WebGLInferenceHandler,
+  inputs: Tensor[],
+  axis: number,
+): Tensor[] => {
+  validateInputs(inputs, axis);
 
-      const outputDims = ShapeUtil.flattenShape(inputs[0].dims, axis);
-      return [inferenceHandler.reshapeUnpacked(inputs[0], outputDims)];
-    };
+  const outputDims = ShapeUtil.flattenShape(inputs[0].dims, axis);
+  return [inferenceHandler.reshapeUnpacked(inputs[0], outputDims)];
+};
 
 export const parseFlattenAttributes: OperatorInitialization<number> = (node: Graph.Node): number =>
-    node.attributes.getInt('axis', 1);  // default axis is 1
+  node.attributes.getInt('axis', 1); // default axis is 1
 
 const validateInputs = (inputs: Tensor[], axis: number): void => {
   if (!inputs || inputs.length !== 1) {
@@ -36,4 +39,4 @@ const validateInputs = (inputs: Tensor[], axis: number): void => {
   if (inputs[0].type === 'string') {
     throw new Error('string tensor is not supported.');
   }
-};
\ No newline at end of file
+};
