@@ -1,13 +1,24 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License.
 
-import {Tensor} from 'onnxruntime-common';
+import { Tensor } from 'onnxruntime-common';
 
-import {tensorTypeToTypedArrayConstructor} from '../wasm-common';
+import { tensorTypeToTypedArrayConstructor } from '../wasm-common';
 
-export const createView = (dataBuffer: ArrayBuffer, type: Tensor.Type): Int32Array|Uint32Array|BigInt64Array|
-    BigUint64Array|Uint8Array|Float32Array|Float64Array|Int8Array|Int16Array|Uint16Array =>
-        new (tensorTypeToTypedArrayConstructor(type))(dataBuffer);
+export const createView = (
+  dataBuffer: ArrayBuffer,
+  type: Tensor.Type,
+):
+  | Int32Array
+  | Uint32Array
+  | BigInt64Array
+  | BigUint64Array
+  | Uint8Array
+  | Float32Array
+  | Float64Array
+  | Int8Array
+  | Int16Array
+  | Uint16Array => new (tensorTypeToTypedArrayConstructor(type))(dataBuffer);
 
 /**
  * a TensorView does not own the data.
@@ -17,6 +28,11 @@ export interface TensorView {
   readonly dataType: number;
   readonly dims: readonly number[];
 
+  /**
+   * get a Float16Array data view of the tensor data. tensor data must be on CPU.
+   */
+  getUint16Array(): Uint16Array;
+
   /**
    * get a Float32Array data view of the tensor data. tensor data must be on CPU.
    */
@@ -32,6 +48,11 @@ export interface TensorView {
    */
   getInt32Array(): Int32Array;
 
+  /**
+   * get a Uint16Array data view of the tensor data. tensor data must be on CPU.
+   */
+  getUint16Array(): Uint16Array;
+
   /**
    * create a new tensor view with the same data but different dimensions.
    */
