@@ -1,12 +1,21 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License.
 
-import {DataType} from '../../../wasm-common';
-import {TensorView} from '../../tensor-view';
-import {ShapeUtil} from '../../util';
-import {ComputeContext, ProgramInfo, ProgramUniform} from '../types';
+import { DataType } from '../../../wasm-common';
+import { TensorView } from '../../tensor-view';
+import { ShapeUtil } from '../../util';
+import { ComputeContext, ProgramInfo, ProgramUniform } from '../types';
 
-import {castToF32, getMaxComponents, inputVariable, outputVariable, ShaderHelper, sumVector, tensorTypeToWsglStorageType, UniformsArrayType} from './common';
+import {
+  castToF32,
+  getMaxComponents,
+  inputVariable,
+  outputVariable,
+  ShaderHelper,
+  sumVector,
+  tensorTypeToWsglStorageType,
+  UniformsArrayType,
+} from './common';
 
 export interface SkipLayerNormAttributes {
   simplified: boolean;
@@ -69,71 +78,72 @@ const validateInputs = (inputs: readonly TensorView[]): void => {
   }
 };
 
-const createSkipLayerNormProgramInfo =
-    (inputs: readonly TensorView[], attributes: SkipLayerNormAttributes, outputCount: number, isTraining: boolean):
-        ProgramInfo => {
-          const simplified = attributes.simplified;
+const createSkipLayerNormProgramInfo = (
+  inputs: readonly TensorView[],
+  attributes: SkipLayerNormAttributes,
+  outputCount: number,
+  isTraining: boolean,
+): ProgramInfo => {
+  const simplified = attributes.simplified;
 
-          const inputShape = inputs[0].dims;
-          const inputSize = ShapeUtil.size(inputShape);
-          const outputShape = inputShape;
-          const outputSize = inputSize;
-          const hiddenSize = inputShape.slice(-1)[0];
-          const meanInvStdDevDim = isTraining ? inputShape.slice(0, -1).concat(1) : [];
-          const hasBetaInput = !simplified && inputs.length > 3;
-          const hasBiasInput = inputs.length > 4;
-          const hasMeanOutput = isTraining && outputCount > 1;
-          const hasInvStdDevOutput = isTraining && outputCount > 2;
-          const hasInputSkipBiasSumOutput = outputCount > 3;
-          const workgroupSize = 64;
+  const inputShape = inputs[0].dims;
+  const inputSize = ShapeUtil.size(inputShape);
+  const outputShape = inputShape;
+  const outputSize = inputSize;
+  const hiddenSize = inputShape.slice(-1)[0];
+  const meanInvStdDevDim = isTraining ? inputShape.slice(0, -1).concat(1) : [];
+  const hasBetaInput = !simplified && inputs.length > 3;
+  const hasBiasInput = inputs.length > 4;
+  const hasMeanOutput = isTraining && outputCount > 1;
+  const hasInvStdDevOutput = isTraining && outputCount > 2;
+  const hasInputSkipBiasSumOutput = outputCount > 3;
+  const workgroupSize = 64;
 
-          const components = getMaxComponents(hiddenSize);
+  const components = getMaxComponents(hiddenSize);
 
-          const programUniforms: ProgramUniform[] = [
-            {type: DataType.uint32, data: outputSize},
-            {type: DataType.uint32, data: components},
-            {type: DataType.uint32, data: hiddenSize},
-            {type: DataType.float, data: attributes.epsilon},
-          ];
-          const getShaderSource = (shaderHelper: ShaderHelper) => {
-            const uniformsArray: UniformsArrayType = [
-              {name: 'output_size', type: 'u32'},
-              {name: 'components', type: 'u32'},
-              {name: 'hidden_size', type: 'u32'},
-              {name: 'epsilon', type: 'f32'},
-            ];
-            const variables = [
-              inputVariable('x', inputs[0].dataType, inputs[0].dims, components),
-              inputVariable('skip', inputs[1].dataType, inputs[1].dims, components),
-              inputVariable('gamma', inputs[2].dataType, inputs[2].dims, components),
-            ];
-            if (hasBetaInput) {
-              variables.push(inputVariable('beta', inputs[3].dataType, inputs[3].dims, components));
-            }
-            if (hasBiasInput) {
-              variables.push(inputVariable('bias', inputs[4].dataType, inputs[4].dims, components));
-            }
-            variables.push(outputVariable('output', inputs[0].dataType, outputShape, components));
-            if (hasMeanOutput) {
-              variables.push(outputVariable('mean_output', DataType.float, meanInvStdDevDim));
-            }
-            if (hasInvStdDevOutput) {
-              variables.push(outputVariable('inv_std_output', DataType.float, meanInvStdDevDim));
-            }
-            if (hasInputSkipBiasSumOutput) {
-              variables.push(outputVariable('input_skip_bias_sum', inputs[0].dataType, outputShape, components));
-            }
-            const dataType = tensorTypeToWsglStorageType(inputs[0].dataType);
-            const vecDataType = tensorTypeToWsglStorageType(DataType.float, components);
-            return `
+  const programUniforms: ProgramUniform[] = [
+    { type: DataType.uint32, data: outputSize },
+    { type: DataType.uint32, data: components },
+    { type: DataType.uint32, data: hiddenSize },
+    { type: DataType.float, data: attributes.epsilon },
+  ];
+  const getShaderSource = (shaderHelper: ShaderHelper) => {
+    const uniformsArray: UniformsArrayType = [
+      { name: 'output_size', type: 'u32' },
+      { name: 'components', type: 'u32' },
+      { name: 'hidden_size', type: 'u32' },
+      { name: 'epsilon', type: 'f32' },
+    ];
+    const variables = [
+      inputVariable('x', inputs[0].dataType, inputs[0].dims, components),
+      inputVariable('skip', inputs[1].dataType, inputs[1].dims, components),
+      inputVariable('gamma', inputs[2].dataType, inputs[2].dims, components),
+    ];
+    if (hasBetaInput) {
+      variables.push(inputVariable('beta', inputs[3].dataType, inputs[3].dims, components));
+    }
+    if (hasBiasInput) {
+      variables.push(inputVariable('bias', inputs[4].dataType, inputs[4].dims, components));
+    }
+    variables.push(outputVariable('output', inputs[0].dataType, outputShape, components));
+    if (hasMeanOutput) {
+      variables.push(outputVariable('mean_output', DataType.float, meanInvStdDevDim));
+    }
+    if (hasInvStdDevOutput) {
+      variables.push(outputVariable('inv_std_output', DataType.float, meanInvStdDevDim));
+    }
+    if (hasInputSkipBiasSumOutput) {
+      variables.push(outputVariable('input_skip_bias_sum', inputs[0].dataType, outputShape, components));
+    }
+    const dataType = tensorTypeToWsglStorageType(inputs[0].dataType);
+    const vecDataType = tensorTypeToWsglStorageType(DataType.float, components);
+    return `
 
       ${shaderHelper.registerUniforms(uniformsArray).declareVariables(...variables)}
       var<workgroup> sum_shared : array<${vecDataType}, ${workgroupSize}>;
       var<workgroup> sum_squared_shared : array<${vecDataType}, ${workgroupSize}>;
 
-      ${shaderHelper.mainStart([
-              workgroupSize, 1, 1
-            ])}
+      ${shaderHelper.mainStart([workgroupSize, 1, 1])}
         let ix = local_id.x;
         let iy = global_id.x / ${workgroupSize};
 
@@ -171,7 +181,8 @@ const createSkipLayerNormProgramInfo =
         let square_sum = sum_squared_shared[0];
         let mean = ${sumVector('sum', components)} / f32(uniforms.hidden_size);
         let inv_std_dev = inverseSqrt(${sumVector('square_sum', components)} / f32(uniforms.hidden_size) ${
-                simplified ? '' : '- mean * mean'} + uniforms.epsilon);
+          simplified ? '' : '- mean * mean'
+        } + uniforms.epsilon);
         ${hasMeanOutput ? 'mean_output[global_idx] = mean;' : ''}
         ${hasInvStdDevOutput ? 'inv_std_output[global_idx] = inv_std_dev;' : ''}
 
@@ -181,33 +192,33 @@ const createSkipLayerNormProgramInfo =
             ${hasBetaInput ? '+ beta[offset1d + i]' : ''};
         }
       }`;
-          };
-          const outputs = [{dims: outputShape, dataType: inputs[0].dataType}];
-          if (outputCount > 1) {
-            outputs.push({dims: meanInvStdDevDim, dataType: DataType.float});
-          }
-          if (outputCount > 2) {
-            outputs.push({dims: meanInvStdDevDim, dataType: DataType.float});
-          }
-          if (outputCount > 3) {
-            outputs.push({dims: inputShape, dataType: inputs[0].dataType});
-          }
-          return {
-            name: 'SkipLayerNormalization',
-            shaderCache: {
-              hint: `${components};${hasMeanOutput};${hasInvStdDevOutput};${hasInputSkipBiasSumOutput}`,
-              inputDependencies: inputs.map((_input, _index) => 'type')
-            },
-            getShaderSource,
-            getRunData: () => ({
-              outputs,
-              dispatchGroup: {
-                x: Math.ceil(outputSize / hiddenSize),
-              },
-              programUniforms
-            }),
-          };
-        };
+  };
+  const outputs = [{ dims: outputShape, dataType: inputs[0].dataType }];
+  if (outputCount > 1) {
+    outputs.push({ dims: meanInvStdDevDim, dataType: DataType.float });
+  }
+  if (outputCount > 2) {
+    outputs.push({ dims: meanInvStdDevDim, dataType: DataType.float });
+  }
+  if (outputCount > 3) {
+    outputs.push({ dims: inputShape, dataType: inputs[0].dataType });
+  }
+  return {
+    name: 'SkipLayerNormalization',
+    shaderCache: {
+      hint: `${components};${hasMeanOutput};${hasInvStdDevOutput};${hasInputSkipBiasSumOutput}`,
+      inputDependencies: inputs.map((_input, _index) => 'type'),
+    },
+    getShaderSource,
+    getRunData: () => ({
+      outputs,
+      dispatchGroup: {
+        x: Math.ceil(outputSize / hiddenSize),
+      },
+      programUniforms,
+    }),
+  };
+};
 
 export const skipLayerNorm = (context: ComputeContext, attributes: SkipLayerNormAttributes): void => {
   // TODO: initialize isTraining from ComputeContext
@@ -225,6 +236,7 @@ export const skipLayerNorm = (context: ComputeContext, attributes: SkipLayerNorm
   if (context.outputCount > 3) {
     outputs.push(3);
   }
-  context.compute(
-      createSkipLayerNormProgramInfo(context.inputs, attributes, context.outputCount, isTraining), {outputs});
+  context.compute(createSkipLayerNormProgramInfo(context.inputs, attributes, context.outputCount, isTraining), {
+    outputs,
+  });
 };
