@@ -1,13 +1,21 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License.
 
-import {DataType} from '../../../wasm-common';
-import {TensorView} from '../../tensor-view';
-import {ShapeUtil} from '../../util';
-import {AttributeWithCacheKey, createAttributeWithCacheKey} from '../attribute-with-cache-key';
-import {ComputeContext, ProgramInfo, ProgramUniform, TensorInfo} from '../types';
+import { DataType } from '../../../wasm-common';
+import { TensorView } from '../../tensor-view';
+import { ShapeUtil } from '../../util';
+import { AttributeWithCacheKey, createAttributeWithCacheKey } from '../attribute-with-cache-key';
+import { ComputeContext, ProgramInfo, ProgramUniform, TensorInfo } from '../types';
 
-import {createTensorShapeVariables, getElementAt, IndicesHelper, inputVariable, outputVariable, ShaderHelper, UniformsArrayType} from './common';
+import {
+  createTensorShapeVariables,
+  getElementAt,
+  IndicesHelper,
+  inputVariable,
+  outputVariable,
+  ShaderHelper,
+  UniformsArrayType,
+} from './common';
 
 export interface SliceAttributes extends AttributeWithCacheKey {
   readonly starts: number[];
@@ -37,9 +45,9 @@ const readInput = (inputs: readonly TensorView[], idx: number): number[] => {
   const input: number[] = [];
   if (inputs.length > idx) {
     if (inputs[idx].dataType === DataType.int64) {
-      inputs[idx].getBigInt64Array().forEach(v => input.push(Number(v)));
+      inputs[idx].getBigInt64Array().forEach((v) => input.push(Number(v)));
     } else if (inputs[idx].dataType === DataType.int32) {
-      inputs[idx].getInt32Array().forEach(v => input.push(Number(v)));
+      inputs[idx].getInt32Array().forEach((v) => input.push(Number(v)));
     } else {
       throw new Error(`Input ${idx} must be an array of int32 or int64`);
     }
@@ -47,38 +55,47 @@ const readInput = (inputs: readonly TensorView[], idx: number): number[] => {
   return input;
 };
 
-const createSliceAttributesFromInputs =
-    (inputs: readonly TensorView[], attributes: SliceAttributes): SliceAttributes => {
-      if (inputs.length > 1) {
-        const starts: number[] = readInput(inputs, 1);
-        const ends: number[] = readInput(inputs, 2);
-        let axes: number[] = readInput(inputs, 3);
-        if (axes.length === 0) {
-          axes = [...Array(inputs[0].dims.length).keys()];
-        }
-        return createAttributeWithCacheKey({starts, ends, axes});
-      } else {
-        return attributes;
-      }
-    };
+const createSliceAttributesFromInputs = (
+  inputs: readonly TensorView[],
+  attributes: SliceAttributes,
+): SliceAttributes => {
+  if (inputs.length > 1) {
+    const starts: number[] = readInput(inputs, 1);
+    const ends: number[] = readInput(inputs, 2);
+    let axes: number[] = readInput(inputs, 3);
+    if (axes.length === 0) {
+      axes = [...Array(inputs[0].dims.length).keys()];
+    }
+    return createAttributeWithCacheKey({ starts, ends, axes });
+  } else {
+    return attributes;
+  }
+};
 
-const fixStartEndValues =
-    (value: number, index: number, inputShape: readonly number[], axes: readonly number[], steps: readonly number[]):
-        number => {
-          let newValue = value;
-          if (value < 0) {
-            newValue += inputShape[axes[index]];
-          }
-          if (steps[index] < 0) {
-            return Math.max(0, Math.min(newValue, inputShape[axes[index]] - 1));
-          } else {
-            return Math.max(0, Math.min(newValue, inputShape[axes[index]]));
-          }
-        };
+const fixStartEndValues = (
+  value: number,
+  index: number,
+  inputShape: readonly number[],
+  axes: readonly number[],
+  steps: readonly number[],
+): number => {
+  let newValue = value;
+  if (value < 0) {
+    newValue += inputShape[axes[index]];
+  }
+  if (steps[index] < 0) {
+    return Math.max(0, Math.min(newValue, inputShape[axes[index]] - 1));
+  } else {
+    return Math.max(0, Math.min(newValue, inputShape[axes[index]]));
+  }
+};
 
-const calculateInputIndicesImpl =
-    (input: IndicesHelper, output: IndicesHelper, inputShape: readonly number[]): string =>
-        `fn calculateInputIndices(output_indices: ${output.type.indices}) -> ${input.type.indices} {
+const calculateInputIndicesImpl = (
+  input: IndicesHelper,
+  output: IndicesHelper,
+  inputShape: readonly number[],
+): string =>
+  `fn calculateInputIndices(output_indices: ${output.type.indices}) -> ${input.type.indices} {
           var input_indices: ${input.type.indices};
           var carry = 0u;
           for (var i = ${inputShape.length}; i >= 0; i--) {
@@ -101,12 +118,18 @@ const calculateInputIndicesImpl =
 const createSliceProgramInfo = (inputs: readonly TensorView[], attributes: SliceAttributes): ProgramInfo => {
   const inputShape = inputs[0].dims;
   const inputSize = ShapeUtil.size(inputShape);
-  const axes = (attributes.axes.length > 0) ? ShapeUtil.normalizeAxes(attributes.axes, inputShape.length) :
-                                              [...Array(inputShape.length).keys()];
+  const axes =
+    attributes.axes.length > 0
+      ? ShapeUtil.normalizeAxes(attributes.axes, inputShape.length)
+      : [...Array(inputShape.length).keys()];
   let steps = readInput(inputs, 4);
-  steps.forEach((step) => step !== 0 || (() => {
-                            throw new Error('step cannot be 0');
-                          }));
+  steps.forEach(
+    (step) =>
+      step !== 0 ||
+      (() => {
+        throw new Error('step cannot be 0');
+      }),
+  );
   if (steps.length === 0) {
     steps = Array(axes.length).fill(1);
   }
@@ -127,7 +150,7 @@ const createSliceProgramInfo = (inputs: readonly TensorView[], attributes: Slice
       }
     }
   }
-  const signs = steps.map(step => Math.sign(step));
+  const signs = steps.map((step) => Math.sign(step));
   // Convert negative steps to positive steps and reverse starts and ends
   steps.forEach((step, i, array) => {
     if (step < 0) {
@@ -144,20 +167,24 @@ const createSliceProgramInfo = (inputs: readonly TensorView[], attributes: Slice
   axes.forEach((axis, _) => {
     outputShape[axis] = Math.ceil((ends[axis] - starts[axis]) / steps[axis]);
   });
-  const outputTensorInfo: TensorInfo = {dims: outputShape, dataType: inputs[0].dataType};
+  const outputTensorInfo: TensorInfo = { dims: outputShape, dataType: inputs[0].dataType };
 
   const output = outputVariable('output', inputs[0].dataType, outputShape.length);
   const input = inputVariable('input', inputs[0].dataType, inputs[0].dims.length);
   const outputSize = ShapeUtil.size(outputShape);
   const uniforms: UniformsArrayType = [
-    {name: 'outputSize', type: 'u32'}, {name: 'starts', type: 'u32', length: starts.length},
-    {name: 'signs', type: 'i32', length: signs.length}, {name: 'steps', type: 'u32', length: steps.length}
+    { name: 'outputSize', type: 'u32' },
+    { name: 'starts', type: 'u32', length: starts.length },
+    { name: 'signs', type: 'i32', length: signs.length },
+    { name: 'steps', type: 'u32', length: steps.length },
   ];
 
   const programUniforms: ProgramUniform[] = [
-    {type: DataType.uint32, data: outputSize}, {type: DataType.uint32, data: starts},
-    {type: DataType.int32, data: signs}, {type: DataType.uint32, data: steps},
-    ...createTensorShapeVariables(inputs[0].dims, outputShape)
+    { type: DataType.uint32, data: outputSize },
+    { type: DataType.uint32, data: starts },
+    { type: DataType.int32, data: signs },
+    { type: DataType.uint32, data: steps },
+    ...createTensorShapeVariables(inputs[0].dims, outputShape),
   ];
 
   const getShaderSource = (shaderHelper: ShaderHelper) => `
@@ -171,20 +198,20 @@ const createSliceProgramInfo = (inputs: readonly TensorView[], attributes: Slice
       }`;
   return {
     name: 'Slice',
-    shaderCache: {hint: `${signs.length}_${starts.length}_${steps.length}`, inputDependencies: ['rank']},
+    shaderCache: { hint: `${signs.length}_${starts.length}_${steps.length}`, inputDependencies: ['rank'] },
     getShaderSource,
     getRunData: () => ({
       outputs: [outputTensorInfo],
-      dispatchGroup: {x: Math.ceil(inputSize / 64 /* workgroup size */)},
-      programUniforms
-    })
+      dispatchGroup: { x: Math.ceil(inputSize / 64 /* workgroup size */) },
+      programUniforms,
+    }),
   };
 };
 
 export const slice = (context: ComputeContext, attributes: SliceAttributes): void => {
   validateInputs(context.inputs, attributes);
   const updatedAttributes = createSliceAttributesFromInputs(context.inputs, attributes);
-  context.compute(createSliceProgramInfo(context.inputs, updatedAttributes), {inputs: [0]});
+  context.compute(createSliceProgramInfo(context.inputs, updatedAttributes), { inputs: [0] });
   // if (ShapeUtil.size(program.outputs[0].dims) > 0) {
   //   context.compute(programInfoLoader, {inputs: [0]});
   // } else {
@@ -197,5 +224,5 @@ export const parseSliceAttributes = (attributes: Record<string, unknown>): Slice
   const starts = attributes.starts as number[];
   const ends = attributes.ends as number[];
   const axes = attributes.axes as number[];
-  return createAttributeWithCacheKey({starts, ends, axes});
+  return createAttributeWithCacheKey({ starts, ends, axes });
 };
