@@ -1,13 +1,17 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License.
 
-import type {Env, InferenceSession, Tensor} from 'onnxruntime-common';
+import type { Env, InferenceSession, Tensor } from 'onnxruntime-common';
 
 /**
  * Among all the tensor locations, only 'cpu' is serializable.
  */
-export type SerializableTensorMetadata =
-    [dataType: Tensor.Type, dims: readonly number[], data: Tensor.DataType, location: 'cpu'];
+export type SerializableTensorMetadata = [
+  dataType: Tensor.Type,
+  dims: readonly number[],
+  data: Tensor.DataType,
+  location: 'cpu',
+];
 
 export type GpuBufferMetadata = {
   gpuBuffer: Tensor.GpuBufferType;
@@ -15,12 +19,19 @@ export type GpuBufferMetadata = {
   dispose?: () => void;
 };
 
+export type MLTensorMetadata = {
+  mlTensor: Tensor.MLTensorType;
+  download?: () => Promise<Tensor.DataTypeMap[Tensor.MLTensorDataTypes]>;
+  dispose?: () => void;
+};
+
 /**
- * Tensors on location 'cpu-pinned' and 'gpu-buffer' are not serializable.
+ * Tensors on location 'cpu-pinned', 'gpu-buffer', and 'ml-tensor' are not serializable.
  */
 export type UnserializableTensorMetadata =
-    [dataType: Tensor.Type, dims: readonly number[], data: GpuBufferMetadata, location: 'gpu-buffer']|
-    [dataType: Tensor.Type, dims: readonly number[], data: Tensor.DataType, location: 'cpu-pinned'];
+  | [dataType: Tensor.Type, dims: readonly number[], data: GpuBufferMetadata, location: 'gpu-buffer']
+  | [dataType: Tensor.Type, dims: readonly number[], data: MLTensorMetadata, location: 'ml-tensor']
+  | [dataType: Tensor.Type, dims: readonly number[], data: Tensor.DataType, location: 'cpu-pinned'];
 
 /**
  * Tensor metadata is a tuple of [dataType, dims, data, location], where
@@ -30,9 +41,10 @@ export type UnserializableTensorMetadata =
  *   - cpu: Uint8Array
  *   - cpu-pinned: Uint8Array
  *   - gpu-buffer: GpuBufferMetadata
+ *   - ml-tensor: MLTensorMetadata
  * - location: tensor data location
  */
-export type TensorMetadata = SerializableTensorMetadata|UnserializableTensorMetadata;
+export type TensorMetadata = SerializableTensorMetadata | UnserializableTensorMetadata;
 
 export type SerializableSessionMetadata = [sessionHandle: number, inputNames: string[], outputNames: string[]];
 
@@ -44,38 +56,41 @@ interface MessageError {
 
 interface MessageInitWasm extends MessageError {
   type: 'init-wasm';
-  in ?: Env;
+  in?: Env;
   out?: never;
 }
 
 interface MessageInitEp extends MessageError {
   type: 'init-ep';
-  in ?: {env: Env; epName: string};
+  in?: { env: Env; epName: string };
   out?: never;
 }
 
 interface MessageCopyFromExternalBuffer extends MessageError {
   type: 'copy-from';
-  in ?: {buffer: Uint8Array};
+  in?: { buffer: Uint8Array };
   out?: SerializableInternalBuffer;
 }
 
 interface MessageCreateSession extends MessageError {
   type: 'create';
-  in ?: {model: SerializableInternalBuffer|Uint8Array; options?: InferenceSession.SessionOptions};
+  in?: { model: SerializableInternalBuffer | Uint8Array; options?: InferenceSession.SessionOptions };
   out?: SerializableSessionMetadata;
 }
 
 interface MessageReleaseSession extends MessageError {
   type: 'release';
-  in ?: number;
+  in?: number;
   out?: never;
 }
 
 interface MessageRun extends MessageError {
   type: 'run';
-  in ?: {
-    sessionId: number; inputIndices: number[]; inputs: SerializableTensorMetadata[]; outputIndices: number[];
+  in?: {
+    sessionId: number;
+    inputIndices: number[];
+    inputs: SerializableTensorMetadata[];
+    outputIndices: number[];
     options: InferenceSession.RunOptions;
   };
   out?: SerializableTensorMetadata[];
@@ -83,9 +98,15 @@ interface MessageRun extends MessageError {
 
 interface MesssageEndProfiling extends MessageError {
   type: 'end-profiling';
-  in ?: number;
+  in?: number;
   out?: never;
 }
 
-export type OrtWasmMessage = MessageInitWasm|MessageInitEp|MessageCopyFromExternalBuffer|MessageCreateSession|
-    MessageReleaseSession|MessageRun|MesssageEndProfiling;
+export type OrtWasmMessage =
+  | MessageInitWasm
+  | MessageInitEp
+  | MessageCopyFromExternalBuffer
+  | MessageCreateSession
+  | MessageReleaseSession
+  | MessageRun
+  | MesssageEndProfiling;
