@@ -1,19 +1,25 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License.
 
-import {env, InferenceSession} from 'onnxruntime-common';
+import { env, InferenceSession } from 'onnxruntime-common';
 
-import {OrtWasmMessage, SerializableInternalBuffer, SerializableSessionMetadata, SerializableTensorMetadata, TensorMetadata} from './proxy-messages';
+import {
+  OrtWasmMessage,
+  SerializableInternalBuffer,
+  SerializableSessionMetadata,
+  SerializableTensorMetadata,
+  TensorMetadata,
+} from './proxy-messages';
 import * as core from './wasm-core-impl';
-import {initializeWebAssembly} from './wasm-factory';
-import {importProxyWorker} from './wasm-utils-import';
+import { initializeWebAssembly } from './wasm-factory';
+import { importProxyWorker, inferWasmPathPrefixFromScriptSrc } from './wasm-utils-import';
 
 const isProxy = (): boolean => !!env.wasm.proxy && typeof document !== 'undefined';
-let proxyWorker: Worker|undefined;
+let proxyWorker: Worker | undefined;
 let initializing = false;
 let initialized = false;
 let aborted = false;
-let temporaryObjectUrl: string|undefined;
+let temporaryObjectUrl: string | undefined;
 
 type PromiseCallbacks<T = void> = [resolve: (result: T) => void, reject: (reason: unknown) => void];
 let initWasmCallbacks: PromiseCallbacks;
@@ -68,16 +74,15 @@ const onProxyWorkerMessage = (ev: MessageEvent<OrtWasmMessage>): void => {
   }
 };
 
-
-export const initializeWebAssemblyAndOrtRuntime = async(): Promise<void> => {
+export const initializeWebAssemblyAndOrtRuntime = async (): Promise<void> => {
   if (initialized) {
     return;
   }
   if (initializing) {
-    throw new Error('multiple calls to \'initWasm()\' detected.');
+    throw new Error("multiple calls to 'initWasm()' detected.");
   }
   if (aborted) {
-    throw new Error('previous call to \'initWasm()\' failed.');
+    throw new Error("previous call to 'initWasm()' failed.");
   }
 
   initializing = true;
@@ -92,7 +97,41 @@ export const initializeWebAssemblyAndOrtRuntime = async(): Promise<void> => {
           proxyWorker.onerror = (ev: ErrorEvent) => reject(ev);
           proxyWorker.onmessage = onProxyWorkerMessage;
           initWasmCallbacks = [resolve, reject];
-          const message: OrtWasmMessage = {type: 'init-wasm', in : env};
+          const message: OrtWasmMessage = { type: 'init-wasm', in: env };
+
+          // if the proxy worker is loaded from a blob URL, we need to make sure the path information is not lost.
+          //
+          // when `env.wasm.wasmPaths` is not set, we need to pass the path information to the worker.
+          //
+          if (!BUILD_DEFS.ENABLE_BUNDLE_WASM_JS && !message.in!.wasm.wasmPaths && objectUrl) {
+            // for a build not bundled the wasm JS, we need to pass the path prefix to the worker.
+            // the path prefix will be used to resolve the path to both the wasm JS and the wasm file.
+            const inferredWasmPathPrefix = inferWasmPathPrefixFromScriptSrc();
+            if (inferredWasmPathPrefix) {
+              message.in!.wasm.wasmPaths = inferredWasmPathPrefix;
+            }
+          }
+
+          if (
+            BUILD_DEFS.IS_ESM &&
+            BUILD_DEFS.ENABLE_BUNDLE_WASM_JS &&
+            !message.in!.wasm.wasmPaths &&
+            (objectUrl || BUILD_DEFS.ESM_IMPORT_META_URL?.startsWith('file:'))
+          ) {
+            // for a build bundled the wasm JS, if either of the following conditions is met:
+            // - the proxy worker is loaded from a blob URL
+            // - `import.meta.url` is a file URL, it means it is overwriten by the bundler.
+            //
+            // in either case, the path information is lost, we need to pass the path of the .wasm file to the worker.
+            // we need to use the bundler preferred URL format:
+            // new URL('filename', import.meta.url)
+            // so that the bundler can handle the file using corresponding loaders.
+            message.in!.wasm.wasmPaths = {
+              wasm: !BUILD_DEFS.DISABLE_JSEP
+                ? new URL('ort-wasm-simd-threaded.jsep.wasm', BUILD_DEFS.ESM_IMPORT_META_URL).href
+                : new URL('ort-wasm-simd-threaded.wasm', BUILD_DEFS.ESM_IMPORT_META_URL).href,
+            };
+          }
           proxyWorker.postMessage(message);
           temporaryObjectUrl = objectUrl;
         } catch (e) {
@@ -100,7 +139,6 @@ export const initializeWebAssemblyAndOrtRuntime = async(): Promise<void> => {
         }
       }, reject);
     });
-
   } else {
     try {
       await initializeWebAssembly(env.wasm);
@@ -115,12 +153,12 @@ export const initializeWebAssemblyAndOrtRuntime = async(): Promise<void> => {
   }
 };
 
-export const initializeOrtEp = async(epName: string): Promise<void> => {
+export const initializeOrtEp = async (epName: string): Promise<void> => {
   if (!BUILD_DEFS.DISABLE_WASM_PROXY && isProxy()) {
     ensureWorker();
     return new Promise<void>((resolve, reject) => {
       enqueueCallbacks('init-ep', [resolve, reject]);
-      const message: OrtWasmMessage = {type: 'init-ep', in : {epName, env}};
+      const message: OrtWasmMessage = { type: 'init-ep', in: { epName, env } };
       proxyWorker!.postMessage(message);
     });
   } else {
@@ -128,12 +166,12 @@ export const initializeOrtEp = async(epName: string): Promise<void> => {
   }
 };
 
-export const copyFromExternalBuffer = async(buffer: Uint8Array): Promise<SerializableInternalBuffer> => {
+export const copyFromExternalBuffer = async (buffer: Uint8Array): Promise<SerializableInternalBuffer> => {
   if (!BUILD_DEFS.DISABLE_WASM_PROXY && isProxy()) {
     ensureWorker();
     return new Promise<SerializableInternalBuffer>((resolve, reject) => {
       enqueueCallbacks('copy-from', [resolve, reject]);
-      const message: OrtWasmMessage = {type: 'copy-from', in : {buffer}};
+      const message: OrtWasmMessage = { type: 'copy-from', in: { buffer } };
       proxyWorker!.postMessage(message, [buffer.buffer]);
     });
   } else {
@@ -141,35 +179,36 @@ export const copyFromExternalBuffer = async(buffer: Uint8Array): Promise<Seriali
   }
 };
 
-export const createSession =
-    async(model: SerializableInternalBuffer|Uint8Array, options?: InferenceSession.SessionOptions):
-        Promise<SerializableSessionMetadata> => {
-          if (!BUILD_DEFS.DISABLE_WASM_PROXY && isProxy()) {
-            // check unsupported options
-            if (options?.preferredOutputLocation) {
-              throw new Error('session option "preferredOutputLocation" is not supported for proxy.');
-            }
-            ensureWorker();
-            return new Promise<SerializableSessionMetadata>((resolve, reject) => {
-              enqueueCallbacks('create', [resolve, reject]);
-              const message: OrtWasmMessage = {type: 'create', in : {model, options: {...options}}};
-              const transferable: Transferable[] = [];
-              if (model instanceof Uint8Array) {
-                transferable.push(model.buffer);
-              }
-              proxyWorker!.postMessage(message, transferable);
-            });
-          } else {
-            return core.createSession(model, options);
-          }
-        };
+export const createSession = async (
+  model: SerializableInternalBuffer | Uint8Array,
+  options?: InferenceSession.SessionOptions,
+): Promise<SerializableSessionMetadata> => {
+  if (!BUILD_DEFS.DISABLE_WASM_PROXY && isProxy()) {
+    // check unsupported options
+    if (options?.preferredOutputLocation) {
+      throw new Error('session option "preferredOutputLocation" is not supported for proxy.');
+    }
+    ensureWorker();
+    return new Promise<SerializableSessionMetadata>((resolve, reject) => {
+      enqueueCallbacks('create', [resolve, reject]);
+      const message: OrtWasmMessage = { type: 'create', in: { model, options: { ...options } } };
+      const transferable: Transferable[] = [];
+      if (model instanceof Uint8Array) {
+        transferable.push(model.buffer);
+      }
+      proxyWorker!.postMessage(message, transferable);
+    });
+  } else {
+    return core.createSession(model, options);
+  }
+};
 
-export const releaseSession = async(sessionId: number): Promise<void> => {
+export const releaseSession = async (sessionId: number): Promise<void> => {
   if (!BUILD_DEFS.DISABLE_WASM_PROXY && isProxy()) {
     ensureWorker();
     return new Promise<void>((resolve, reject) => {
       enqueueCallbacks('release', [resolve, reject]);
-      const message: OrtWasmMessage = {type: 'release', in : sessionId};
+      const message: OrtWasmMessage = { type: 'release', in: sessionId };
       proxyWorker!.postMessage(message);
     });
   } else {
@@ -177,24 +216,31 @@ export const releaseSession = async(sessionId: number): Promise<void> => {
   }
 };
 
-export const run = async(
-    sessionId: number, inputIndices: number[], inputs: TensorMetadata[], outputIndices: number[],
-    outputs: Array<TensorMetadata|null>, options: InferenceSession.RunOptions): Promise<TensorMetadata[]> => {
+export const run = async (
+  sessionId: number,
+  inputIndices: number[],
+  inputs: TensorMetadata[],
+  outputIndices: number[],
+  outputs: Array<TensorMetadata | null>,
+  options: InferenceSession.RunOptions,
+): Promise<TensorMetadata[]> => {
   if (!BUILD_DEFS.DISABLE_WASM_PROXY && isProxy()) {
     // check inputs location
-    if (inputs.some(t => t[3] !== 'cpu')) {
+    if (inputs.some((t) => t[3] !== 'cpu')) {
       throw new Error('input tensor on GPU is not supported for proxy.');
     }
     // check outputs location
-    if (outputs.some(t => t)) {
+    if (outputs.some((t) => t)) {
       throw new Error('pre-allocated output tensor is not supported for proxy.');
     }
     ensureWorker();
     return new Promise<SerializableTensorMetadata[]>((resolve, reject) => {
       enqueueCallbacks('run', [resolve, reject]);
-      const serializableInputs = inputs as SerializableTensorMetadata[];  // every input is on CPU.
-      const message: OrtWasmMessage =
-          {type: 'run', in : {sessionId, inputIndices, inputs: serializableInputs, outputIndices, options}};
+      const serializableInputs = inputs as SerializableTensorMetadata[]; // every input is on CPU.
+      const message: OrtWasmMessage = {
+        type: 'run',
+        in: { sessionId, inputIndices, inputs: serializableInputs, outputIndices, options },
+      };
       proxyWorker!.postMessage(message, core.extractTransferableBuffers(serializableInputs));
     });
   } else {
@@ -202,12 +248,12 @@ export const run = async(
   }
 };
 
-export const endProfiling = async(sessionId: number): Promise<void> => {
+export const endProfiling = async (sessionId: number): Promise<void> => {
   if (!BUILD_DEFS.DISABLE_WASM_PROXY && isProxy()) {
     ensureWorker();
     return new Promise<void>((resolve, reject) => {
       enqueueCallbacks('end-profiling', [resolve, reject]);
-      const message: OrtWasmMessage = {type: 'end-profiling', in : sessionId};
+      const message: OrtWasmMessage = { type: 'end-profiling', in: sessionId };
       proxyWorker!.postMessage(message);
     });
   } else {
