@@ -1,20 +1,21 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License.
 
-import {Attribute} from '../../../../lib/onnxjs/attribute';
-import {Backend, InferenceHandler, resolveBackend, SessionHandler} from '../../../../lib/onnxjs/backend';
-import {Profiler} from '../../../../lib/onnxjs/instrument';
-import {Tensor} from '../../../../lib/onnxjs/tensor';
-import {PoolConvUtil} from '../../../../lib/onnxjs/util';
-import {TensorResultValidator} from '../../../test-runner';
-import {createMockGraph} from '../../../test-shared';
+import { Attribute } from '../../../../lib/onnxjs/attribute';
+import { Backend, InferenceHandler, resolveBackend, SessionHandler } from '../../../../lib/onnxjs/backend';
+import { Profiler } from '../../../../lib/onnxjs/instrument';
+import { Tensor } from '../../../../lib/onnxjs/tensor';
+import { PoolConvUtil } from '../../../../lib/onnxjs/util';
+import { TensorResultValidator } from '../../../test-runner';
+import { createMockGraph } from '../../../test-shared';
 
-import {conv2d} from './test-conv-utils';
+import { conv2d } from './test-conv-utils';
 
 function createRandomArray(size: number): Float32Array {
   const randomTable = [0, 3, 6, 9, 2, 5, 8, 1, 4, 7];
   return new Float32Array(
-      Array.from({length: size}, (_v, k) => randomTable[k % 10] * 0.1 + randomTable[Math.trunc(k / 10) % 10] * 0.01));
+    Array.from({ length: size }, (_v, k) => randomTable[k % 10] * 0.1 + randomTable[Math.trunc(k / 10) % 10] * 0.01),
+  );
 }
 interface TestData {
   inputShape: number[];
@@ -35,7 +36,7 @@ function getTestData(): TestData[] {
       autoPad: 'SAME_UPPER',
       dilations: [1, 1],
       strides: [1, 1],
-      group: 1
+      group: 1,
     },
     {
       inputShape: [1, 3, 224, 224],
@@ -44,7 +45,7 @@ function getTestData(): TestData[] {
       pads: [0, 0, 0, 0],
       dilations: [1, 1],
       strides: [2, 2],
-      group: 1
+      group: 1,
     },
     {
       inputShape: [1, 64, 55, 55],
@@ -53,7 +54,7 @@ function getTestData(): TestData[] {
       pads: [0, 0, 0, 0],
       dilations: [1, 1],
       strides: [1, 1],
-      group: 1
+      group: 1,
     },
     // {
     //   inputShape: [1, 16, 55, 55],
@@ -278,7 +279,7 @@ function getTestData(): TestData[] {
       pads: [1, 1, 1, 1],
       dilations: [1, 1],
       strides: [1, 1],
-      group: 1
+      group: 1,
     },
     {
       inputShape: [1, 2, 3, 3],
@@ -287,7 +288,7 @@ function getTestData(): TestData[] {
       pads: [0, 0, 0, 0],
       dilations: [1, 1],
       strides: [1, 1],
-      group: 1
+      group: 1,
     },
     {
       inputShape: [1, 3, 224, 224],
@@ -296,7 +297,7 @@ function getTestData(): TestData[] {
       pads: [3, 3, 3, 3],
       dilations: [1, 1],
       strides: [2, 2],
-      group: 1
+      group: 1,
     },
     // {
     //   inputShape: [1, 64, 56, 56],
@@ -765,7 +766,7 @@ function getTestData(): TestData[] {
       pads: [1, 1, 1, 1],
       dilations: [1, 1],
       strides: [1, 1],
-      group: 1
+      group: 1,
     },
     {
       inputShape: [1, 512, 7, 7],
@@ -775,7 +776,7 @@ function getTestData(): TestData[] {
       pads: [0, 0, 0, 0],
       dilations: [1, 1],
       strides: [1, 1],
-      group: 1
+      group: 1,
     },
     // {
     //   inputShape: [1, 2048, 7, 7],
@@ -811,13 +812,19 @@ function getTestData(): TestData[] {
 }
 
 const validator = new TensorResultValidator('webgl');
-let webglBackend: Backend|undefined;
-let webglSessionhandler: SessionHandler|undefined;
-let webglInferenceHandler: InferenceHandler|undefined;
+let webglBackend: Backend | undefined;
+let webglSessionhandler: SessionHandler | undefined;
+let webglInferenceHandler: InferenceHandler | undefined;
 
 function webglConv(
-    inputTensor: Tensor, kernelTensor: Tensor, biasTensor: Tensor|null, autoPad: string|undefined, dilations: number[],
-    pads: number[]|undefined, strides: number[]): Tensor {
+  inputTensor: Tensor,
+  kernelTensor: Tensor,
+  biasTensor: Tensor | null,
+  autoPad: string | undefined,
+  dilations: number[],
+  pads: number[] | undefined,
+  strides: number[],
+): Tensor {
   const attributes = new Attribute(undefined);
   attributes.set('dilations', 'ints', dilations);
   attributes.set('auto_pad', 'string', autoPad ? autoPad : '');
@@ -827,16 +834,22 @@ function webglConv(
   }
   attributes.set('strides', 'ints', strides);
   const graph = createMockGraph('Conv', attributes);
-  const op = webglSessionhandler!.resolve(graph.getNodes()[0], [{domain: '', version: 7}], graph);
+  const op = webglSessionhandler!.resolve(graph.getNodes()[0], [{ domain: '', version: 7 }], graph);
   const inputs = [inputTensor, kernelTensor];
   if (biasTensor) {
     inputs.push(biasTensor);
   }
-  return (op.impl(webglInferenceHandler!, inputs, op.context))[0];
+  return op.impl(webglInferenceHandler!, inputs, op.context)[0];
 }
 function cpuConv(
-    inputTensor: Tensor, kernelTensor: Tensor, biasTensor: Tensor|null, autoPad: string|undefined, dilations: number[],
-    pads: number[]|undefined, strides: number[]): Tensor {
+  inputTensor: Tensor,
+  kernelTensor: Tensor,
+  biasTensor: Tensor | null,
+  autoPad: string | undefined,
+  dilations: number[],
+  pads: number[] | undefined,
+  strides: number[],
+): Tensor {
   const attributes = new Attribute(undefined);
   attributes.set('dilations', 'ints', dilations);
   attributes.set('auto_pad', 'string', autoPad ? autoPad : '');
@@ -852,7 +865,14 @@ function cpuConv(
 
   const adjustedPads = pads ? pads.slice(0) : [0, 0, 0, 0];
   const outputDims = PoolConvUtil.computeConvOutputShape(
-      x.dims, w.dims, strides, dilations, kernelTensor.dims.slice(2), adjustedPads, autoPad);
+    x.dims,
+    w.dims,
+    strides,
+    dilations,
+    kernelTensor.dims.slice(2),
+    adjustedPads,
+    autoPad,
+  );
   const y = new Tensor(outputDims, x.type);
   conv2d(y, x, w, b, dilations, 1, adjustedPads, strides);
   return y;
@@ -861,7 +881,7 @@ describe('New Conv tests', () => {
   before(async () => {
     const profiler = Profiler.create();
     webglBackend = await resolveBackend('webgl');
-    webglSessionhandler = webglBackend.createSessionHandler({profiler});
+    webglSessionhandler = webglBackend.createSessionHandler({ profiler });
     webglInferenceHandler = webglSessionhandler.createInferenceHandler();
   });
   const testDataSet = getTestData();
@@ -872,9 +892,9 @@ describe('New Conv tests', () => {
       const kernelData = createRandomArray(testData.kernelShape.reduce((a, b) => a * b));
       const biasData = testData.biasShape.length === 1 ? createRandomArray(testData.biasShape[0]) : null;
       const rgbas = [false];
-      rgbas.forEach(rgba => {
+      rgbas.forEach((rgba) => {
         describe(`RGBA: ${rgba}`, () => {
-          before(function() {
+          before(function () {
             const patchSize = testData.kernelShape.slice(1).reduce((a, b) => a * b);
             if (rgba && patchSize % 4 !== 0) {
               // eslint-disable-next-line no-invalid-this
@@ -885,14 +905,27 @@ describe('New Conv tests', () => {
             // create new Tensors otherwise the session/inference level caching would cause issues
             const inputTensor = new Tensor(testData.inputShape, 'float32', undefined, undefined, inputData);
             const kernelTensor = new Tensor(testData.kernelShape, 'float32', undefined, undefined, kernelData);
-            const biasTensor =
-                biasData ? new Tensor(testData.biasShape, 'float32', undefined, undefined, biasData) : null;
+            const biasTensor = biasData
+              ? new Tensor(testData.biasShape, 'float32', undefined, undefined, biasData)
+              : null;
             const actual = webglConv(
-                inputTensor, kernelTensor, biasTensor, testData.autoPad, testData.dilations, testData.pads,
-                testData.strides);
+              inputTensor,
+              kernelTensor,
+              biasTensor,
+              testData.autoPad,
+              testData.dilations,
+              testData.pads,
+              testData.strides,
+            );
             const expected = cpuConv(
-                inputTensor, kernelTensor, biasTensor, testData.autoPad, testData.dilations, testData.pads,
-                testData.strides);
+              inputTensor,
+              kernelTensor,
+              biasTensor,
+              testData.autoPad,
+              testData.dilations,
+              testData.pads,
+              testData.strides,
+            );
             try {
               validator.checkTensorResult([actual], [expected]);
             } catch {
