@@ -3,7 +3,7 @@
  ******************************************************************************/
 #pragma once
 
-#include <cute/algorithm/copy.hpp>
+#include <cute/tensor.hpp>
 
 #include <cutlass/cutlass.h>
 #include <cutlass/layout/layout.h>
@@ -25,17 +25,15 @@ struct Flash_kernel_traits {
 #endif
 
   using ElementAccum = float;
-  using index_t = uint32_t;
+  using index_t = int64_t;
 
 #if defined(__CUDA_ARCH__) && __CUDA_ARCH__ >= 800
   using MMA_Atom_Arch = std::conditional_t<
       std::is_same_v<elem_type, cutlass::half_t>,
       MMA_Atom<SM80_16x8x16_F32F16F16F32_TN>,
       MMA_Atom<SM80_16x8x16_F32BF16BF16F32_TN>>;
-  using ValLayoutMNK = cute::Layout<cute::Shape<_1, _2, _1>>;
 #else
   using MMA_Atom_Arch = MMA_Atom<SM75_16x8x8_F32F16F16F32_TN>;
-  using ValLayoutMNK = cute::Layout<cute::Shape<_1, _2, _2>>;
 #endif
 
 #if defined(__CUDA_ARCH__) && __CUDA_ARCH__ >= 750
@@ -77,8 +75,7 @@ struct Flash_fwd_kernel_traits : public Base {
   using TiledMma = TiledMMA<
       typename Base::MMA_Atom_Arch,
       Layout<Shape<Int<kNWarps>, _1, _1>>,  // 4x1x1 or 8x1x1 thread group
-      typename Base::ValLayoutMNK>;         // 1x2x1 or 1x2x2 value group for 16x16x16 MMA and LDSM
-
+      Tile<Int<16 * kNWarps>, _16, _16>>;
   using SmemLayoutAtomQ = decltype(composition(Swizzle<kSwizzle, 3, 3>{},
                                                // This has to be kBlockKSmem, using kHeadDim gives wrong results for d=128
                                                Layout<Shape<_8, Int<kBlockKSmem>>,
@@ -91,19 +88,9 @@ struct Flash_fwd_kernel_traits : public Base {
       SmemLayoutAtomQ{},
       Shape<Int<kBlockN>, Int<kHeadDim>>{}));
 
-  // This has to be kBlockN and not 8, otherwise we get wrong results for d=128
-  using SmemLayoutAtomVtransposedNoSwizzle = Layout<Shape<Int<kBlockKSmem>, Int<kBlockN>>,
-                                                    Stride<_1, Int<kBlockKSmem>>>;
-  using SmemLayoutAtomVtransposed = decltype(composition(Swizzle<kSwizzle, 3, 3>{}, SmemLayoutAtomVtransposedNoSwizzle{}));
-  using SmemLayoutVtransposed = decltype(tile_to_shape(
-      SmemLayoutAtomVtransposed{},
-      Shape<Int<kHeadDim>, Int<kBlockN>>{}));
-  // Maybe the VtransposeNoSwizzle just needs to have the right shape
-  // And the strides don't matter?
-  using SmemLayoutVtransposedNoSwizzle = decltype(tile_to_shape(
-      SmemLayoutAtomVtransposedNoSwizzle{},
-      Shape<Int<kHeadDim>, Int<kBlockN>>{}));
-  // using SmemLayoutVtransposedNoSwizzle = decltype(SmemLayoutVtransposed{}.layout_fn());
+  // https://github.com/ColfaxResearch/cutlass-kernels/blob/a222587e6d59b93ba704853d3946fb686d8b8892/src/fmha/fmha_forward.cu#L434
+  using SmemLayoutVtransposed = decltype(composition(SmemLayoutKV{}, make_layout(Shape<Int<kHeadDim>, Int<kBlockN>>{}, GenRowMajor{})));
+  using SmemLayoutVtransposedNoSwizzle = decltype(get_nonswizzle_portion(SmemLayoutVtransposed{}));
 
   using SmemLayoutAtomO = decltype(composition(Swizzle<kSwizzle, 3, 3>{},
                                                Layout<Shape<Int<8>, Int<kBlockKSmem>>,
@@ -114,10 +101,8 @@ struct Flash_fwd_kernel_traits : public Base {
   using SmemCopyAtomO = Copy_Atom<DefaultCopy, Element>;
   using SmemCopyAtomOaccum = Copy_Atom<DefaultCopy, ElementAccum>;
 
-  static constexpr int kSmemQCount = cute::size(SmemLayoutQ{});
-  static constexpr int kSmemKVCount = cute::size(SmemLayoutKV{}) * 2;
-  static constexpr int kSmemQSize = kSmemQCount * sizeof(Element);
-  static constexpr int kSmemKVSize = kSmemKVCount * sizeof(Element);
+  static constexpr int kSmemQSize = size(SmemLayoutQ{}) * sizeof(Element);
+  static constexpr int kSmemKVSize = size(SmemLayoutKV{}) * 2 * sizeof(Element);
   static constexpr int kSmemSize = Share_Q_K_smem ? std::max(kSmemQSize, kSmemKVSize) : kSmemQSize + kSmemKVSize;
 
   static constexpr int kGmemElemsPerLoad = sizeof(cute::uint128_t) / sizeof(Element);
@@ -129,8 +114,8 @@ struct Flash_fwd_kernel_traits : public Base {
   // to the same banks.
   static constexpr int kGmemThreadsPerRow = kBlockKSmem / kGmemElemsPerLoad;
   static_assert(kNThreads % kGmemThreadsPerRow == 0, "kNThreads must be a multiple of kGmemThreadsPerRow");
-  using GmemLayoutAtom = cute::Layout<cute::Shape<cute::Int<kNThreads / kGmemThreadsPerRow>, cute::Int<kGmemThreadsPerRow>>,
-                                      cute::Stride<cute::Int<kGmemThreadsPerRow>, _1>>;
+  using GmemLayoutAtom = Layout<Shape<Int<kNThreads / kGmemThreadsPerRow>, Int<kGmemThreadsPerRow>>,
+                                Stride<Int<kGmemThreadsPerRow>, _1>>;
 
   // We use CACHEGLOBAL instead of CACHEALWAYS for both Q and K/V, since we won't be reading
   // from the same address by the same threadblock. This is slightly faster.
@@ -138,30 +123,29 @@ struct Flash_fwd_kernel_traits : public Base {
       Has_cp_async,
       SM80_CP_ASYNC_CACHEGLOBAL<cute::uint128_t>,
       DefaultCopy>;
-  using GmemTiledCopyQKV = decltype(make_tiled_copy(Copy_Atom<Gmem_copy_struct, elem_type>{},
+  using GmemTiledCopyQKV = decltype(make_tiled_copy(Copy_Atom<Gmem_copy_struct, Element>{},
                                                     GmemLayoutAtom{},
-                                                    cute::Layout<cute::Shape<_1, _8>>{}));  // Val layout, 8 vals per read
+                                                    Layout<Shape<_1, _8>>{}));  // Val layout, 8 vals per read
   using GmemTiledCopyO = decltype(make_tiled_copy(Copy_Atom<DefaultCopy, elem_type>{},
                                                   GmemLayoutAtom{},
-                                                  cute::Layout<cute::Shape<_1, _8>>{}));  // Val layout, 8 vals per store
-  static constexpr int kGmemThreadsPerRowP = kBlockN / kGmemElemsPerLoad;
-  static_assert(kNThreads % kGmemThreadsPerRowP == 0, "kNThreads must be a multiple of kGmemThreadsPerRowP");
-  using GmemLayoutAtomP = cute::Layout<cute::Shape<cute::Int<kNThreads / kGmemThreadsPerRowP>, cute::Int<kGmemThreadsPerRowP>>,
-                                       cute::Stride<cute::Int<kGmemThreadsPerRowP>, _1>>;
-
-  using GmemTiledCopyP = decltype(make_tiled_copy(Copy_Atom<DefaultCopy, elem_type>{},
-                                                  GmemLayoutAtomP{},
-                                                  cute::Layout<cute::Shape<_1, _8>>{}));  // Val layout, 8 vals per store
+                                                  Layout<Shape<_1, _8>>{}));  // Val layout, 8 vals per store
 
   using GmemLayoutAtomOaccum = std::conditional_t<
       kBlockKSmem == 32,
-      cute::Layout<cute::Shape<_16, _8>,  // Thread layout, 8 threads per row
-                   cute::Stride<_8, _1>>,
-      cute::Layout<cute::Shape<_8, _16>,  // Thread layout, 16 threads per row
-                   cute::Stride<_16, _1>>>;
+      Layout<Shape<_16, _8>,  // Thread layout, 8 threads per row
+             Stride<_8, _1>>,
+      Layout<Shape<_8, _16>,  // Thread layout, 16 threads per row
+             Stride<_16, _1>>>;
   using GmemTiledCopyOaccum = decltype(make_tiled_copy(Copy_Atom<DefaultCopy, ElementAccum>{},
                                                        GmemLayoutAtomOaccum{},
-                                                       cute::Layout<cute::Shape<_1, _4>>{}));  // Val layout, 4 vals per store
+                                                       Layout<Shape<_1, _4>>{}));  // Val layout, 4 vals per store
+  using GmemLayoutAtomRotcossin = GmemLayoutAtom;
+  using GmemTiledCopyRotcossin = decltype(make_tiled_copy(Copy_Atom<UniversalCopy<uint64_t>, Element>{},
+                                                          GmemLayoutAtomRotcossin{},
+                                                          Layout<Shape<_1, _4>>{}));  // Val layout, 4 vals per load
+  using GmemTiledCopyRotcossinCont = decltype(make_tiled_copy(Copy_Atom<DefaultCopy, Element>{},
+                                                              GmemLayoutAtomRotcossin{},
+                                                              Layout<Shape<_1, _8>>{}));  // Val layout, 8 vals per load
 };
 
 // Is_V_in_regs is an option to reduce smem usage, but will increase register pressue.
@@ -200,84 +184,61 @@ struct Flash_bwd_kernel_traits : public Base {
 
   using TiledMmaSdP = TiledMMA<
       typename Base::MMA_Atom_Arch,
-      cute::Layout<cute::Shape<cute::Int<AtomLayoutMSdP>, cute::Int<kNWarps / AtomLayoutMSdP>, _1>>,
-      typename Base::ValLayoutMNK>;  // 1x2x1 or 1x2x2 value group for 16x16x16 MMA and LDSM
+      Layout<Shape<Int<AtomLayoutMSdP>, Int<kNWarps / AtomLayoutMSdP>, _1>>,
+      Tile<Int<16 * AtomLayoutMSdP>, Int<16 * kNWarps / AtomLayoutMSdP>, _16>>;
 
   using TiledMmadKV = TiledMMA<
       typename Base::MMA_Atom_Arch,
-      cute::Layout<cute::Shape<cute::Int<AtomLayoutNdKV>, cute::Int<kNWarps / AtomLayoutNdKV>, _1>>,
-      typename Base::ValLayoutMNK>;  // 1x2x1 or 1x2x2 value group for 16x16x16 MMA and LDSM
+      Layout<Shape<Int<AtomLayoutNdKV>, Int<kNWarps / AtomLayoutNdKV>, _1>>,
+      Tile<Int<16 * AtomLayoutNdKV>, Int<16 * kNWarps / AtomLayoutNdKV>, _16>>;
 
   using TiledMmadQ = TiledMMA<
       typename Base::MMA_Atom_Arch,
-      cute::Layout<cute::Shape<cute::Int<AtomLayoutMdQ>, cute::Int<kNWarps / AtomLayoutMdQ>, _1>>,  // 2x4x1 or 4x2x1 thread group
-      typename Base::ValLayoutMNK>;                                                                 // 1x2x1 or 1x2x2 value group for 16x16x16 MMA and LDSM
+      Layout<Shape<Int<AtomLayoutMdQ>, Int<kNWarps / AtomLayoutMdQ>, _1>>,  // 2x4x1 or 4x2x1 thread group
+      Tile<Int<16 * AtomLayoutMdQ>, Int<16 * kNWarps / AtomLayoutMdQ>, _16>>;
 
   using SmemLayoutAtomQdO = decltype(composition(Swizzle<kSwizzle, 3, 3>{},
-                                                 cute::Layout<cute::Shape<_8, cute::Int<kBlockKSmem>>,
-                                                              cute::Stride<cute::Int<kBlockKSmem>, _1>>{}));
+                                                 Layout<Shape<_8, Int<kBlockKSmem>>,
+                                                        Stride<Int<kBlockKSmem>, _1>>{}));
   using SmemLayoutQdO = decltype(tile_to_shape(
       SmemLayoutAtomQdO{},
-      cute::make_shape(cute::Int<kBlockM>{}, cute::Int<kHeadDim>{})));
+      make_shape(Int<kBlockM>{}, Int<kHeadDim>{})));
 
   using SmemLayoutAtomKV = decltype(composition(Swizzle<kSwizzle, 3, 3>{},
-                                                cute::Layout<cute::Shape<cute::Int<kBlockM / kNWarps>, cute::Int<kBlockKSmem>>,
-                                                             cute::Stride<cute::Int<kBlockKSmem>, _1>>{}));
+                                                Layout<Shape<Int<kBlockM / kNWarps>, Int<kBlockKSmem>>,
+                                                       Stride<Int<kBlockKSmem>, _1>>{}));
   using SmemLayoutKV = decltype(tile_to_shape(
       // SmemLayoutAtomQdO{},
       SmemLayoutAtomKV{},
-      cute::make_shape(cute::Int<kBlockN>{}, cute::Int<kHeadDim>{})));
+      make_shape(Int<kBlockN>{}, Int<kHeadDim>{})));
 
-  using SmemLayoutAtomKtransposedNoSwizzle = Layout<Shape<Int<kBlockKSmem>, Int<kBlockN>>,
-                                                    Stride<_1, Int<kBlockKSmem>>>;
-  using SmemLayoutAtomKtransposed = decltype(composition(Swizzle<kSwizzle, 3, 3>{}, SmemLayoutAtomKtransposedNoSwizzle{}));
-  using SmemLayoutKtransposed = decltype(tile_to_shape(
-      SmemLayoutAtomKtransposed{},
-      make_shape(Int<kHeadDim>{}, Int<kBlockN>{})));
-  // Maybe the KtransposeNoSwizzle just needs to have the right shape
-  // And the strides don't matter?
-  using SmemLayoutKtransposedNoSwizzle = decltype(tile_to_shape(
-      SmemLayoutAtomKtransposedNoSwizzle{},
-      make_shape(Int<kHeadDim>{}, Int<kBlockN>{})));
-  // using SmemLayoutKtransposedNoSwizzle = decltype(SmemLayoutKtransposed{}.layout_fn());
+  using SmemLayoutKtransposed = decltype(composition(SmemLayoutKV{}, make_layout(Shape<Int<kHeadDim>, Int<kBlockN>>{}, GenRowMajor{})));
+  using SmemLayoutKtransposedNoSwizzle = decltype(get_nonswizzle_portion(SmemLayoutKtransposed{}));
 
   // TODO: generalize to other values of kBlockN
   // TODO: what should be the Swizzle here? 3 is faster than 1, and 1 is faster than 2
   // static constexpr int kPBlockN = kBlockN;
-  static_assert(kBlockN >= 64);
+  // Temporarily disabling this for hdim 256 on sm86 and sm89
+  // static_assert(kBlockN >= 64);
+  static_assert(kBlockN >= 32);
   // TD [2023-03-19]: Idk why kPBlockN = 16 and kSwizzlePdS=3 is the fastest.
-  static constexpr int kPBlockN = 64;
+  static constexpr int kPBlockN = kBlockN >= 64 ? 64 : 32;
   static_assert(kPBlockN == 16 || kPBlockN == 32 || kPBlockN == 64);
   // static constexpr int kSwizzlePdS = kPBlockN == 16 ? 1 : (kPBlockN == 32 ? 2 : 3);
   static constexpr int kSwizzlePdS = 3;
   using SmemLayoutAtomPdS = decltype(composition(Swizzle<kSwizzlePdS, 3, 3>{},
-                                                 cute::Layout<cute::Shape<cute::Int<kBlockM>, cute::Int<kPBlockN>>,
-                                                              cute::Stride<cute::Int<kPBlockN>, _1>>{}));
+                                                 Layout<Shape<Int<kBlockM>, Int<kPBlockN>>,
+                                                        Stride<Int<kPBlockN>, _1>>{}));
   using SmemLayoutPdS = decltype(tile_to_shape(
       SmemLayoutAtomPdS{},
-      cute::make_shape(cute::Int<kBlockM>{}, cute::Int<kBlockN>{})));
-  using SmemLayoutAtomPdStransposedNoSwizzle = Layout<Shape<Int<kPBlockN>, Int<kBlockM>>,
-                                                      Stride<_1, Int<kPBlockN>>>;
-  using SmemLayoutAtomPdStransposed = decltype(composition(Swizzle<kSwizzlePdS, 3, 3>{}, SmemLayoutAtomPdStransposedNoSwizzle{}));
-  using SmemLayoutPdStransposed = decltype(tile_to_shape(
-      SmemLayoutAtomPdStransposed{},
-      make_shape(Int<kBlockN>{}, Int<kBlockM>{})));
-  using SmemLayoutPdStransposedNoSwizzle = decltype(tile_to_shape(
-      SmemLayoutAtomPdStransposedNoSwizzle{},
-      make_shape(Int<kBlockN>{}, Int<kBlockM>{})));
-  // using SmemLayoutPdStransposedNoSwizzle = decltype(SmemLayoutPdStransposed{}.layout_fn());
+      make_shape(Int<kBlockM>{}, Int<kBlockN>{})));
+  using SmemLayoutPdStransposed = decltype(composition(SmemLayoutPdS{}, make_layout(Shape<Int<kBlockN>, Int<kBlockM>>{}, GenRowMajor{})));
+  using SmemLayoutPdStransposedNoSwizzle = decltype(get_nonswizzle_portion(SmemLayoutPdStransposed{}));
+
   using SmemCopyAtomPdS = Copy_Atom<DefaultCopy, elem_type>;
 
-  using SmemLayoutAtomQdOtransposedNoSwizzle = Layout<Shape<Int<kBlockKSmem>, Int<kBlockM>>,
-                                                      Stride<_1, Int<kBlockKSmem>>>;
-  using SmemLayoutAtomQdOtransposed = decltype(composition(Swizzle<kSwizzle, 3, 3>{}, SmemLayoutAtomQdOtransposedNoSwizzle{}));
-  using SmemLayoutQdOtransposed = decltype(tile_to_shape(
-      SmemLayoutAtomQdOtransposed{},
-      make_shape(Int<kHeadDim>{}, Int<kBlockM>{})));
-  using SmemLayoutQdOtransposedNoSwizzle = decltype(tile_to_shape(
-      SmemLayoutAtomQdOtransposedNoSwizzle{},
-      make_shape(Int<kHeadDim>{}, Int<kBlockM>{})));
-  // using SmemLayoutQdOtransposedNoSwizzle = decltype(SmemLayoutQdOtransposed{}.layout_fn());
+  using SmemLayoutQdOtransposed = decltype(composition(SmemLayoutQdO{}, make_layout(Shape<Int<kHeadDim>, Int<kBlockM>>{}, GenRowMajor{})));
+  using SmemLayoutQdOtransposedNoSwizzle = decltype(get_nonswizzle_portion(SmemLayoutQdOtransposed{}));
 
   using SmemLayoutAtomdKV = decltype(composition(Swizzle<kSwizzle, 3, 3>{},
                                                  Layout<Shape<_8, Int<kBlockKSmem>>,
@@ -295,25 +256,18 @@ struct Flash_bwd_kernel_traits : public Base {
       make_shape(Int<kBlockM>{}, Int<kHeadDim>{})));
   using SmemCopyAtomdQ = Copy_Atom<DefaultCopy, elem_type>;
 
-  static constexpr int kSmemQdOCount = cute::size(SmemLayoutQdO{}) * (No_double_buffer ? 2 : 3);  // Double buffer for sQ
-  static constexpr int kSmemKVCount = cute::size(SmemLayoutKV{}) * 2;
-  static constexpr int kSmemdSCount = cute::size(SmemLayoutPdS{});
-  static constexpr int kSmemPCount = cute::size(SmemLayoutPdS{});
-  static constexpr int kSmemdQCount = cute::size(SmemLayoutdQ{});
-  //   static constexpr int kSmemdPsumCount = kBlockM;
-  static constexpr int kSmemQdOSize = kSmemQdOCount * sizeof(Element);
-  static constexpr int kSmemKVSize = kSmemKVCount * sizeof(Element);
-  static constexpr int kSmemdSSize = kSmemdSCount * sizeof(Element);
-  static constexpr int kSmemPSize = kSmemPCount * sizeof(Element);
-  static constexpr int kSmemdQSize = kSmemdQCount * sizeof(Element);
-  //   static constexpr int kSmemdPsumSize = kSmemdPsumCount * sizeof(ElementAccum);
+  // Double buffer for sQ
+  static constexpr int kSmemQdOSize = size(SmemLayoutQdO{}) * (No_double_buffer ? 2 : 3) * sizeof(Element);
+  static constexpr int kSmemKVSize = size(SmemLayoutKV{}) * 2 * sizeof(Element);
+  static constexpr int kSmemdSSize = size(SmemLayoutPdS{}) * sizeof(Element);
+  static constexpr int kSmemPSize = size(SmemLayoutPdS{}) * sizeof(Element);
+  static constexpr int kSmemdQSize = size(SmemLayoutdQ{}) * sizeof(Element);
   static constexpr int kSmemSize = kSmemQdOSize + (!Is_V_in_regs
                                                        ? kSmemKVSize + kSmemdSSize + std::max(kSmemPSize, kSmemdQSize)
                                                        : std::max(kSmemKVSize, kSmemKVSize / 2 + kSmemdSSize + std::max(kSmemPSize, kSmemdQSize)));
   static constexpr int kSmemSize1colblock = kSmemQdOSize + (!Is_V_in_regs
                                                                 ? kSmemKVSize + kSmemdSSize + kSmemPSize
                                                                 : std::max(kSmemKVSize, kSmemKVSize / 2 + kSmemdSSize + kSmemPSize));
-  static constexpr int kSmemSize1rowblock = kSmemQdOSize / 3 * 2 + kSmemKVSize / 2 * 3 + kSmemdSSize + kSmemPSize;
 
   static constexpr int kGmemElemsPerLoad = sizeof(cute::uint128_t) / sizeof(Element);
   static_assert(kHeadDim % kGmemElemsPerLoad == 0, "kHeadDim must be a multiple of kGmemElemsPerLoad");
@@ -321,8 +275,8 @@ struct Flash_bwd_kernel_traits : public Base {
   // to affect speed in practice.
   static constexpr int kGmemThreadsPerRow = kBlockKSmem / kGmemElemsPerLoad;
   static_assert(kNThreads % kGmemThreadsPerRow == 0, "kNThreads must be a multiple of kGmemThreadsPerRow");
-  using GmemLayoutAtom = cute::Layout<cute::Shape<cute::Int<kNThreads / kGmemThreadsPerRow>, cute::Int<kGmemThreadsPerRow>>,
-                                      cute::Stride<cute::Int<kGmemThreadsPerRow>, _1>>;
+  using GmemLayoutAtom = Layout<Shape<Int<kNThreads / kGmemThreadsPerRow>, Int<kGmemThreadsPerRow>>,
+                                Stride<Int<kGmemThreadsPerRow>, _1>>;
 
   // We use CACHEGLOBAL instead of CACHEALWAYS for both Q and K/V, since we won't be reading
   // from the same address by the same threadblock. This is slightly faster.
@@ -332,30 +286,30 @@ struct Flash_bwd_kernel_traits : public Base {
       DefaultCopy>;
   using GmemTiledCopyQKV = decltype(make_tiled_copy(Copy_Atom<Gmem_copy_struct, elem_type>{},
                                                     GmemLayoutAtom{},
-                                                    cute::Layout<cute::Shape<_1, _8>>{}));  // Val layout, 8 vals per read
+                                                    Layout<Shape<_1, _8>>{}));  // Val layout, 8 vals per read
   using GmemTiledCopydO = decltype(make_tiled_copy(Copy_Atom<DefaultCopy, elem_type>{},
                                                    GmemLayoutAtom{},
-                                                   cute::Layout<cute::Shape<_1, _8>>{}));  // Val layout, 8 vals per store
+                                                   Layout<Shape<_1, _8>>{}));  // Val layout, 8 vals per store
   using GmemTiledCopydKV = decltype(make_tiled_copy(Copy_Atom<DefaultCopy, elem_type>{},
                                                     GmemLayoutAtom{},
-                                                    cute::Layout<cute::Shape<_1, _8>>{}));  // Val layout, 8 vals per store
+                                                    Layout<Shape<_1, _8>>{}));  // Val layout, 8 vals per store
   using GmemTiledCopydQ = decltype(make_tiled_copy(Copy_Atom<DefaultCopy, elem_type>{},
                                                    GmemLayoutAtom{},
-                                                   cute::Layout<cute::Shape<_1, _8>>{}));  // Val layout, 8 vals per store
+                                                   Layout<Shape<_1, _8>>{}));  // Val layout, 8 vals per store
   using GmemLayoutAtomdQaccum = std::conditional_t<
       kBlockKSmem == 32,
-      cute::Layout<cute::Shape<_32, _8>,  // Thread layout, 8 threads per row
-                   cute::Stride<_8, _1>>,
-      cute::Layout<cute::Shape<_16, _16>,  // Thread layout, 16 threads per row
-                   cute::Stride<_16, _1>>>;
+      Layout<Shape<_32, _8>,  // Thread layout, 8 threads per row
+             Stride<_8, _1>>,
+      Layout<Shape<_16, _16>,  // Thread layout, 16 threads per row
+             Stride<_16, _1>>>;
   using GmemTiledCopydQaccum = decltype(make_tiled_copy(Copy_Atom<DefaultCopy, ElementAccum>{},
                                                         GmemLayoutAtomdQaccum{},
-                                                        cute::Layout<cute::Shape<_1, _4>>{}));  // Val layout, 4 vals per store
+                                                        Layout<Shape<_1, _4>>{}));  // Val layout, 4 vals per store
 
   using GmemTiledCopydQaccumAtomicAdd = decltype(make_tiled_copy(Copy_Atom<DefaultCopy, ElementAccum>{},
-                                                                 cute::Layout<cute::Shape<_8, _32>,  // Thread layout, 8 threads per row
-                                                                              cute::Stride<_32, _1>>{},
-                                                                 cute::Layout<cute::Shape<_1, _1>>{}));  // Val layout, 1 val per store
+                                                                 Layout<Shape<_8, _32>,  // Thread layout, 8 threads per row
+                                                                        Stride<_32, _1>>{},
+                                                                 Layout<Shape<_1, _1>>{}));  // Val layout, 1 val per store
 };
 
 }  // namespace flash
