@@ -6,8 +6,10 @@
 #include "contrib_ops/cuda/bert/multihead_attention.h"
 #include "contrib_ops/cpu/bert/multihead_attention_helper.h"
 #include "contrib_ops/cuda/bert/cutlass_fmha/memory_efficient_attention.h"
+#include "contrib_ops/cuda/bert/cudnn_fmha/cudnn_flash_attention.h"
 #include "contrib_ops/cuda/bert/flash_attention/flash_api.h"
 #include "contrib_ops/cuda/utils/dump_cuda_tensor.h"
+#include "contrib_ops/cuda/bert/lean_attention/lean_api.h"
 
 using namespace onnxruntime::cuda;
 using namespace ::onnxruntime::common;
@@ -53,10 +55,16 @@ MultiHeadAttention<T>::MultiHeadAttention(const OpKernelInfo& info)
 
   disable_flash_attention_ = sizeof(T) != 2 || !kernel_options_->UseFlashAttention();
 
+#if USE_LEAN_ATTENTION
+  enable_lean_attention_ = sizeof(T) == 2 && kernel_options_->UseLeanAttention();
+#endif
+
   disable_memory_efficient_attention_ = !kernel_options_->UseEfficientAttention();
 
   disable_fused_cross_attention_ = sizeof(T) != 2 || !kernel_options_->UseTrtCrossAttention();
 
+  enable_cudnn_flash_attention_ = sizeof(T) == 2 && kernel_options_->UseCudnnFlashAttention();
+
   // Allocate cache buffers
   constexpr size_t cache_bytes = sizeof(int32_t) * (static_cast<size_t>(kCumulatedSequenceLengthCacheMaxBatchSize) + 1);
   cumulated_sequence_length_q_cache_.buffer = GetTransientScratchBuffer<void>(cache_bytes);
@@ -72,7 +80,7 @@ Status MultiHeadAttention<T>::ComputeInternal(OpKernelContext* context) const {
   const Tensor* value = context->Input<Tensor>(2);
   const Tensor* bias = context->Input<Tensor>(3);
   const Tensor* key_padding_mask = context->Input<Tensor>(4);
-  const Tensor* relative_position_bias = context->Input<Tensor>(5);
+  const Tensor* attention_bias = context->Input<Tensor>(5);
   const Tensor* past_key = context->Input<Tensor>(6);
   const Tensor* past_value = context->Input<Tensor>(7);
 
@@ -85,7 +93,7 @@ Status MultiHeadAttention<T>::ComputeInternal(OpKernelContext* context) const {
                                                                       value,
                                                                       bias,
                                                                       key_padding_mask,
-                                                                      relative_position_bias,
+                                                                      attention_bias,
                                                                       past_key,
                                                                       past_value,
                                                                       nullptr,  // past_seq_len
@@ -146,9 +154,67 @@ Status MultiHeadAttention<T>::ComputeInternal(OpKernelContext* context) const {
   // Check whether we can use fused kernel
   int sm = device_prop.major * 10 + device_prop.minor;
 
+  AttentionKernelType kernel_type = AttentionKernelType::AttentionKernel_Default;
+
+  typedef typename ToCudaType<T>::MappedType CudaT;
+  AttentionData<CudaT> data;
+
+#if USE_LEAN_ATTENTION || USE_FLASH_ATTENTION
+  size_t softmax_lse_bytes = 0;
+  size_t softmax_lse_accum_bytes = 0;
+  size_t out_accum_bytes = 0;
+#endif
+
+#if USE_LEAN_ATTENTION
+  // Lean attention only supports token-generation phase with sequence_length == 1.
+  bool use_lean_attention = enable_lean_attention_ &&
+                            parameters.sequence_length == 1 &&
+                            parameters.past_sequence_length > 0 &&
+                            nullptr == attention_bias &&
+                            nullptr == key_padding_mask &&
+                            parameters.head_size == parameters.v_head_size &&
+                            onnxruntime::lean::is_supported(device_prop,
+                                                            parameters.head_size,
+                                                            parameters.num_heads,
+                                                            parameters.num_heads);
+
+  size_t sync_flag_bytes = 0;
+  if (use_lean_attention) {
+    softmax_lse_bytes = onnxruntime::lean::get_softmax_lse_size(parameters.sequence_length,
+                                                                parameters.batch_size,
+                                                                parameters.num_heads);
+
+    auto [num_splits, slse_accum_bytes, o_accum_bytes, sflag_bytes, griddimz, max_tiles_tb, hload_tbs, tiles_per_head] = onnxruntime::lean::get_num_splits_and_buffer_sizes(
+        parameters.batch_size,
+        parameters.sequence_length,
+        parameters.total_sequence_length,
+        parameters.num_heads,  // q heads
+        parameters.num_heads,  // kv heads
+        parameters.head_size,
+        device_prop.multiProcessorCount,
+        parameters.is_unidirectional);
+
+    data.num_splits = static_cast<int>(num_splits);
+    data.grid_dim_z = static_cast<int>(griddimz);
+    data.max_tiles_per_tb = static_cast<int>(max_tiles_tb);
+    data.high_load_tbs = static_cast<int>(hload_tbs);
+    data.tiles_per_head = static_cast<int>(tiles_per_head);
+    softmax_lse_accum_bytes = slse_accum_bytes;
+    out_accum_bytes = o_accum_bytes;
+    sync_flag_bytes = sflag_bytes;
+    kernel_type = AttentionKernelType::AttentionKernel_LeanAttention;
+  }
+
+  auto lean_sync_flag_buffer = GetScratchBuffer<void>(sync_flag_bytes, context->GetComputeStream());
+  data.lean_sync_flag = reinterpret_cast<int*>(lean_sync_flag_buffer.get());
+#else
+  constexpr bool use_lean_attention = false;
+#endif
+
 #if USE_FLASH_ATTENTION
-  bool use_flash_attention = !disable_flash_attention_ &&
-                             nullptr == relative_position_bias &&
+  bool use_flash_attention = kernel_type == AttentionKernelType::AttentionKernel_Default &&
+                             !disable_flash_attention_ &&
+                             nullptr == attention_bias &&
                              nullptr == key_padding_mask &&
                              parameters.head_size == parameters.v_head_size &&
                              onnxruntime::flash::is_supported(device_prop,
@@ -160,34 +226,60 @@ Status MultiHeadAttention<T>::ComputeInternal(OpKernelContext* context) const {
       parameters.sequence_length < kernel_options_->MinSeqLenForFlashAttentionPackedQkv()) {
     use_flash_attention = false;
   }
+
   // Allocate buffers
-  size_t softmax_lse_accum_bytes = 0;
-  size_t out_accum_bytes = 0;
   if (use_flash_attention) {
+    softmax_lse_bytes = onnxruntime::flash::get_softmax_lse_size(parameters.sequence_length,
+                                                                 parameters.batch_size,
+                                                                 parameters.num_heads);
+
     using namespace std;
     auto [num_splits, slse_accum_bytes, o_accum_bytes] = onnxruntime::flash::get_num_splits_and_buffer_sizes(
         parameters.batch_size, parameters.sequence_length, parameters.total_sequence_length, parameters.num_heads,
         parameters.head_size, device_prop.multiProcessorCount);
-    parameters.num_splits = static_cast<int>(num_splits);
+    data.num_splits = static_cast<int>(num_splits);
     softmax_lse_accum_bytes = slse_accum_bytes;
     out_accum_bytes = o_accum_bytes;
+    kernel_type = AttentionKernelType::AttentionKernel_FlashAttention;
   }
-  auto softmax_lse_accum_buffer = GetScratchBuffer<void>(softmax_lse_accum_bytes, context->GetComputeStream());
-  auto out_accum_buffer = GetScratchBuffer<void>(out_accum_bytes, context->GetComputeStream());
 #else
   constexpr bool use_flash_attention = false;
-  auto softmax_lse_accum_buffer = GetScratchBuffer<void>(0, context->GetComputeStream());  // nullptr
-  auto out_accum_buffer = GetScratchBuffer<void>(0, context->GetComputeStream());          // nullptr
+#endif
+
+#if USE_LEAN_ATTENTION || USE_FLASH_ATTENTION
+  auto softmax_lse_buffer = GetScratchBuffer<void>(softmax_lse_bytes, context->GetComputeStream());
+  auto softmax_lse_accum_buffer = GetScratchBuffer<void>(softmax_lse_accum_bytes, context->GetComputeStream());
+  auto out_accum_buffer = GetScratchBuffer<void>(out_accum_bytes, context->GetComputeStream());
+  if (use_flash_attention || use_lean_attention) {
+    data.softmax_lse = reinterpret_cast<CudaT*>(softmax_lse_buffer.get());
+    data.softmax_lse_accum = reinterpret_cast<CudaT*>(softmax_lse_accum_buffer.get());
+    data.out_accum = reinterpret_cast<CudaT*>(out_accum_buffer.get());
+  }
 #endif
 
   bool is_mask_none_or_1d_k_len = parameters.mask_type == AttentionMaskType::MASK_NONE ||
                                   parameters.mask_type == AttentionMaskType::MASK_1D_KEY_SEQ_LEN;
+  bool use_cudnn_sdpa = kernel_type == AttentionKernelType::AttentionKernel_Default &&
+                        enable_cudnn_flash_attention_ &&
+                        is_mask_none_or_1d_k_len &&
+                        onnxruntime::cudnn_sdpa::is_supported(device_prop,
+                                                              parameters.num_heads,              // num_heads_q
+                                                              parameters.num_heads,              // num_heads_kv
+                                                              parameters.head_size,              // head_size_qk
+                                                              parameters.v_head_size,            // head_size_v
+                                                              parameters.sequence_length,        // seq_len_q
+                                                              parameters.total_sequence_length,  // seq_len_kv
+                                                              is_unidirectional_);
+  if (use_cudnn_sdpa) {
+    kernel_type = AttentionKernelType::AttentionKernel_CudnnFlashAttention;
+  }
+
   bool use_fused_cross_attention =
-      !use_flash_attention &&
+      kernel_type == AttentionKernelType::AttentionKernel_Default &&
       !disable_fused_cross_attention_ &&
       !is_unidirectional_ &&
       nullptr == key_padding_mask &&
-      nullptr == relative_position_bias &&
+      nullptr == attention_bias &&
       nullptr == past_key && nullptr == present_key &&
       (parameters.qkv_format == Q_K_V_BSNH || (parameters.qkv_format == Q_KV_BSNH_BSN2H && bias == nullptr)) &&
       parameters.hidden_size == parameters.v_hidden_size &&
@@ -203,15 +295,15 @@ Status MultiHeadAttention<T>::ComputeInternal(OpKernelContext* context) const {
     // The kernel has no limit on sequence length, and this checks whether the kernel has been loaded.
     if (fused_fp16_cross_attention_kernel_->isValid(sequence_length)) {
       fused_cross_attention_kernel = fused_fp16_cross_attention_kernel_;
+      kernel_type = AttentionKernelType::AttentionKernel_TrtFusedCrossAttention;
     }
   }
 
   bool use_fused_runner =
-      !use_flash_attention &&
+      kernel_type == AttentionKernelType::AttentionKernel_Default &&
       !disable_fused_self_attention_ &&
-      fused_cross_attention_kernel == nullptr &&
       !is_unidirectional_ &&
-      nullptr == relative_position_bias &&
+      nullptr == attention_bias &&
       (parameters.qkv_format == Q_K_V_BSNH || parameters.qkv_format == QKV_BSN3H) &&
       nullptr == past_key && nullptr == present_key &&
       is_mask_none_or_1d_k_len &&
@@ -232,6 +324,8 @@ Status MultiHeadAttention<T>::ComputeInternal(OpKernelContext* context) const {
     const int normalized_seq_len = fused_fp16_runner_->NormalizeSequenceLength(sequence_length);
     if (fused_fp16_runner_->IsValid(normalized_seq_len)) {
       fused_runner = fused_fp16_runner_.get();
+      // could also be AttentionKernel_TrtFlashAttention, but we don't classify it here.
+      kernel_type = AttentionKernelType::AttentionKernel_TrtFusedAttention;
     }
   }
 
@@ -241,25 +335,26 @@ Status MultiHeadAttention<T>::ComputeInternal(OpKernelContext* context) const {
                           parameters.sequence_length >= length_threshold ||
                           parameters.kv_sequence_length >= length_threshold;
 
-  // Check whether the relative position bias alignment is good for memory efficient attention.
-  bool is_good_for_rpb = relative_position_bias != nullptr && parameters.sequence_length % (4 * sizeof(T)) == 0;
-
   bool use_memory_efficient_attention =
-      !use_flash_attention &&
-      fused_runner == nullptr &&
-      fused_cross_attention_kernel == nullptr &&
+      kernel_type == AttentionKernelType::AttentionKernel_Default &&
       !disable_memory_efficient_attention_ &&
       is_long_sequence &&
-      (relative_position_bias == nullptr || is_good_for_rpb) &&
+      // Check whether the attention bias alignment is good for memory efficient attention.
+      (attention_bias == nullptr || parameters.sequence_length % (4 * sizeof(T)) == 0) &&
       (nullptr == key_padding_mask || parameters.mask_type == AttentionMaskType::MASK_1D_KEY_SEQ_LEN_START) &&
       has_memory_efficient_attention(sm, std::is_same<T, MLFloat16>::value,
                                      parameters.head_size, parameters.v_head_size);
+  if (use_memory_efficient_attention) {
+    kernel_type = AttentionKernelType::AttentionKernel_CutlassMemoryEfficientAttention;
+  }
 #else
   constexpr bool use_memory_efficient_attention = false;
 #endif
 
-  typedef typename ToCudaType<T>::MappedType CudaT;
-  AttentionData<CudaT> data;
+  if (kernel_type == AttentionKernelType::AttentionKernel_Default) {
+    kernel_type = AttentionKernelType::AttentionKernel_Unfused;
+  }
+
   data.bias = (nullptr == bias) ? nullptr : reinterpret_cast<const CudaT*>(bias->Data<T>());
   data.query = reinterpret_cast<const CudaT*>(query->Data<T>());
   data.key = (nullptr == key) ? nullptr : reinterpret_cast<const CudaT*>(key->Data<T>());
@@ -268,14 +363,19 @@ Status MultiHeadAttention<T>::ComputeInternal(OpKernelContext* context) const {
   data.mask_index_dims = (nullptr == key_padding_mask) ? gsl::span<const int64_t>() : key_padding_mask->Shape().GetDims();
   data.past_key = (nullptr == past_key) ? nullptr : reinterpret_cast<const CudaT*>(past_key->Data<T>());
   data.past_value = (nullptr == past_value) ? nullptr : reinterpret_cast<const CudaT*>(past_value->Data<T>());
-  data.relative_position_bias = (nullptr == relative_position_bias) ? nullptr : reinterpret_cast<const CudaT*>(relative_position_bias->Data<T>());
+  if (nullptr != attention_bias) {
+    data.attention_bias = reinterpret_cast<const CudaT*>(attention_bias->Data<T>());
+  }
   data.output = reinterpret_cast<CudaT*>(output->MutableData<T>());
   data.present_key = (nullptr == present_key) ? nullptr : reinterpret_cast<CudaT*>(present_key->MutableData<T>());
   data.present_value = (nullptr == present_value) ? nullptr : reinterpret_cast<CudaT*>(present_value->MutableData<T>());
   data.fused_runner = reinterpret_cast<void*>(fused_runner);
   data.fused_cross_attention_kernel = fused_cross_attention_kernel;
   data.use_flash_attention = use_flash_attention;
+  data.use_lean_attention = use_lean_attention;
   data.use_memory_efficient_attention = use_memory_efficient_attention;
+  data.kernel_type = kernel_type;
+  data.allocator = Info().GetAllocator(OrtMemType::OrtMemTypeDefault);
 
   // Cache of cumulated sequence length that could help when sequence length does not change (for example, image model).
   // The cache will be initialized only once, and become readonly after that.
@@ -301,8 +401,10 @@ Status MultiHeadAttention<T>::ComputeInternal(OpKernelContext* context) const {
                                                      parameters.total_sequence_length,
                                                      fused_runner,
                                                      use_flash_attention,
+                                                     use_lean_attention,
                                                      use_fused_cross_attention,
                                                      use_memory_efficient_attention,
+                                                     use_cudnn_sdpa,
                                                      no_qkv_workspace);
   auto work_space = GetScratchBuffer<void>(workspace_bytes, context->GetComputeStream());
 
@@ -311,16 +413,12 @@ Status MultiHeadAttention<T>::ComputeInternal(OpKernelContext* context) const {
   data.workspace_bytes = workspace_bytes;
 
   data.allow_debug_info = kernel_options_->AllowDebugInfo();
-  if (softmax_lse_accum_buffer != nullptr) {
-    data.softmax_lse_accum = reinterpret_cast<CudaT*>(softmax_lse_accum_buffer.get());
-  }
-  if (out_accum_buffer != nullptr) {
-    data.out_accum = reinterpret_cast<CudaT*>(out_accum_buffer.get());
-  }
 
   if (data.allow_debug_info) {
     AttentionKernelDebugInfo debug_info;
     debug_info.use_flash_attention = use_flash_attention;
+    debug_info.use_lean_attention = use_lean_attention;
+    debug_info.use_cudnn_flash_attention = use_cudnn_sdpa;
     debug_info.use_trt_cross_attention = fused_cross_attention_kernel != nullptr;
     debug_info.use_efficient_attention = use_memory_efficient_attention;
     if (fused_fp16_runner_ != nullptr) {
@@ -335,8 +433,9 @@ Status MultiHeadAttention<T>::ComputeInternal(OpKernelContext* context) const {
   }
 
   cublasHandle_t cublas = GetCublasHandle(context);
+  cudnnHandle_t cudnn = GetCudnnHandle(context);
   return QkvToContext<CudaT>(
-      device_prop, cublas, context->GetComputeStream(), parameters, data);
+      device_prop, cublas, cudnn, context->GetComputeStream(), parameters, data);
 }
 
 }  // namespace cuda
