@@ -17,7 +17,7 @@ namespace onnxruntime {
 namespace contrib {
 namespace cuda {
 
-unsigned char cubin_fmha_v2_fp16_32_32_sm80_cu_cubin[] = {
+extern const unsigned char cubin_fmha_v2_fp16_32_32_sm80_cu_cubin[] = {
   0x7f, 0x45, 0x4c, 0x46, 0x02, 0x01, 0x01, 0x33, 0x07, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0xbe, 0x00, 0x73, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -713,7 +713,7 @@ unsigned char cubin_fmha_v2_fp16_32_32_sm80_cu_cubin[] = {
   0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00
 };
-unsigned int cubin_fmha_v2_fp16_32_32_sm80_cu_cubin_len = 8320;
+extern const unsigned int cubin_fmha_v2_fp16_32_32_sm80_cu_cubin_len = 8320;
 
 }  // namespace cuda
 }  // namespace contrib
