@@ -20,6 +20,7 @@ Abstract:
 --*/
 
 #include "mlasi.h"
+#include "softmax.h"
 
 //
 // Bundles the constants for use by kernels written in assembly.
@@ -68,12 +69,13 @@ MLAS_INTERNAL_DATA const float MlasMinimumF32Value = std::numeric_limits<float>:
 // threads.
 //
 
+template <typename T>
 struct MLAS_SOFTMAX_WORK_BLOCK {
     ptrdiff_t ThreadCountN;
     bool LogSoftmax;
     bool SmoothSoftmax;
-    const float* Input;
-    float* Output;
+    const T* Input;
+    T* Output;
     size_t N;
     size_t D;
 };
@@ -244,9 +246,10 @@ Return Value:
     }
 }
 
+template <>
 void
 MLASCALL
-MlasComputeExp(
+MlasComputeExp<float>(
     const float* Input,
     float* Output,
     size_t N
@@ -280,6 +283,20 @@ Return Value:
 #endif
 }
 
+template <>
+void MLASCALL
+MlasComputeExp<MLAS_FP16>(
+    const MLAS_FP16* Input,
+    MLAS_FP16* Output,
+    size_t N
+) {
+    const auto* dispatch = GetMlasPlatform().SoftmaxDispatch;
+    if (dispatch == nullptr || dispatch->Exp_Fp16 == nullptr) {
+        MLAS_THROW_EX(std::runtime_error, "Exp_Fp16 is not supported.");
+    }
+    dispatch->Exp_Fp16(Input, Output, N);
+}
+
 MLAS_FORCEINLINE
 MLAS_FLOAT32X4
 MlasComputeSumExpVector(
@@ -783,10 +800,18 @@ Return Value:
     }
 }
 
+template <typename T>
 void
 MlasComputeSoftmaxThreaded(
     void* Context,
     ptrdiff_t Index
+);
+
+template <>
+void
+MlasComputeSoftmaxThreaded<float>(
+    void* Context,
+    ptrdiff_t Index
 )
 /*++
 
@@ -807,7 +832,7 @@ Return Value:
 
 --*/
 {
-    const auto* WorkBlock = (MLAS_SOFTMAX_WORK_BLOCK*)Context;
+    const auto* WorkBlock = (MLAS_SOFTMAX_WORK_BLOCK<float>*)Context;
 
     //
     // Partition the operation along the N dimension.
@@ -906,11 +931,85 @@ Return Value:
     }
 }
 
+template <>
+void
+MlasComputeSoftmaxThreaded<MLAS_FP16>(
+    void* Context,
+    ptrdiff_t Index
+)
+/*++
+
+Routine Description:
+
+    This routine is invoked from a worker thread to execute a segment of a
+    softmax or log softmax operation.
+
+Arguments:
+
+    Context - Supplies the pointer to the context for the threaded operation.
+
+    ThreadId - Supplies the current index of the threaded operation.
+
+Return Value:
+
+    None.
+
+--*/
+{
+    const auto* WorkBlock = (MLAS_SOFTMAX_WORK_BLOCK<MLAS_FP16>*)Context;
+    size_t n;
+    size_t CountN;
+    MlasPartitionWork(Index, WorkBlock->ThreadCountN, WorkBlock->N, &n, &CountN);
+
+    const size_t D = WorkBlock->D;
+    const bool LogSoftmax = WorkBlock->LogSoftmax;
+    const bool SmoothSoftmax = WorkBlock->SmoothSoftmax;
+
+    const MLAS_FP16* Input = WorkBlock->Input + n * D;
+    MLAS_FP16* Output = WorkBlock->Output + n * D;
+
+    const auto* dispatch = GetMlasPlatform().SoftmaxDispatch;
+    if (dispatch == nullptr ||
+        dispatch->ReduceMax_Fp16 == nullptr ||
+        dispatch->SumExp_Fp16 == nullptr ||
+        (LogSoftmax && dispatch->LogSoftmax_Fp16 == nullptr) ||
+        (!LogSoftmax && dispatch->Softmax_Fp16 == nullptr)) {
+        MLAS_THROW_EX(std::runtime_error, "Lacks kernels for fp16 softmax.");
+    }
+
+    while (CountN > 0) {
+        MLAS_FP16 Maximum = dispatch->ReduceMax_Fp16(Input, D);
+        MLAS_FP16 NegativeMaximum = Maximum.Negate();
+        if (SmoothSoftmax && !NegativeMaximum.IsNegative()) {
+            NegativeMaximum = MLAS_FP16::FromBits(0);
+        }
+
+        MLAS_FP16* Temp = LogSoftmax ? nullptr : Output;
+        MLAS_FP16 Accumulation = dispatch->SumExp_Fp16(Input, Temp, D, NegativeMaximum);
+        float accumulation_fp32 = Accumulation.ToFloat();
+
+        if (SmoothSoftmax) {
+            accumulation_fp32 += expf(NegativeMaximum.ToFloat());
+        }
+
+        if (LogSoftmax) {
+            dispatch->LogSoftmax_Fp16(Input, Output, D, NegativeMaximum, MLAS_FP16(std::log(accumulation_fp32)));
+        } else {
+            dispatch->Softmax_Fp16(Output, Output, D, MLAS_FP16(accumulation_fp32));
+        }
+
+        Input += D;
+        Output += D;
+        CountN--;
+    }
+}
+
+template <typename T>
 void
 MLASCALL
 MlasComputeSoftmax(
-    const float* Input,
-    float* Output,
+    const T* Input,
+    T* Output,
     size_t N,
     size_t D,
     bool LogSoftmax,
@@ -949,7 +1048,7 @@ Return Value:
 
 --*/
 {
-    MLAS_SOFTMAX_WORK_BLOCK WorkBlock;
+    MLAS_SOFTMAX_WORK_BLOCK<T> WorkBlock;
 
     //
     // Capture the softmax parameters to the work block.
@@ -985,5 +1084,67 @@ Return Value:
 
     WorkBlock.ThreadCountN = ThreadCountN;
 
-    MlasExecuteThreaded(MlasComputeSoftmaxThreaded, &WorkBlock, ThreadCountN, ThreadPool);
+    MlasExecuteThreaded(MlasComputeSoftmaxThreaded<T>, &WorkBlock, ThreadCountN, ThreadPool);
+}
+
+template
+void
+MLASCALL
+MlasComputeSoftmax<float>(
+    const float* Input,
+    float* Output,
+    size_t N,
+    size_t D,
+    bool LogSoftmax,
+    bool SmoothSoftmax,
+    MLAS_THREADPOOL* ThreadPool
+);
+
+template
+void
+MLASCALL
+MlasComputeSoftmax<MLAS_FP16>(
+    const MLAS_FP16* Input,
+    MLAS_FP16* Output,
+    size_t N,
+    size_t D,
+    bool LogSoftmax,
+    bool SmoothSoftmax,
+    MLAS_THREADPOOL* ThreadPool
+);
+
+template <>
+bool
+MLASCALL
+MlasGQASupported<MLAS_FP16>(
+    CBLAS_TRANSPOSE TransA,
+    CBLAS_TRANSPOSE TransB
+) {
+    if (!MlasHGemmSupported(TransA, TransB)) {
+        return false;
+    }
+
+    const auto* softmax_dispatch = GetMlasPlatform().SoftmaxDispatch;
+    if (softmax_dispatch == nullptr ||
+        softmax_dispatch->Tanh_Fp16 == nullptr ||
+        softmax_dispatch->Softcap_Fp16 == nullptr ||
+        softmax_dispatch->SumExp_Fp16 == nullptr ||
+        softmax_dispatch->Softmax_Fp16 == nullptr ||
+        softmax_dispatch->ReduceMax_Fp16 == nullptr) {
+        return false;
+    }
+
+    return true;
+}
+
+template <>
+bool
+MLASCALL
+MlasGQASupported<float>(
+    CBLAS_TRANSPOSE TransA,
+    CBLAS_TRANSPOSE TransB
+) {
+    MLAS_UNREFERENCED_PARAMETER(TransA);
+    MLAS_UNREFERENCED_PARAMETER(TransB);
+    return true;
 }
