@@ -552,6 +552,119 @@ MlasQLinearAddKernelHelper(
           InputA, ScaleA, ZeroPointA, InputB, ScaleB, ZeroPointB, ScaleC, ZeroPointC, OutputC, N);
     }
 }
+#elif defined(MLAS_LSX_INTRINSICS)
+
+template<typename DataType, bool IsScalarB>
+static
+void
+MlasQLinearAddKernelHelper(
+    const DataType* InputA,
+    float ScaleA,
+    int32_t ZeroPointA,
+    const DataType* InputB,
+    float ScaleB,
+    int32_t ZeroPointB,
+    float ScaleC,
+    int32_t ZeroPointC,
+    DataType* OutputC,
+    size_t N
+    )
+{
+    const float ScaleRatio_AC = ScaleA / ScaleC;
+    const float ScaleRatio_BC = ScaleB / ScaleC;
+    const auto VectorScaleRatio_AC = MlasBroadcastFloat32x4(ScaleRatio_AC);
+    const auto VectorScaleRatio_BC = MlasBroadcastFloat32x4(ScaleRatio_BC);
+    auto VectorFixedPart = MlasBroadcastFloat32x4((float)ZeroPointC - (ScaleRatio_AC * ZeroPointA + ScaleRatio_BC * ZeroPointB));
+
+    MLAS_FLOAT32X4 va_lo, va_hi, vb_lo, vb_hi;
+    if (IsScalarB) {
+        float tmp_f = (float)*InputB;
+        uint32_t *tmp_p = (uint32_t *)&tmp_f;
+        vb_lo = MlasReinterpretAsFloat32x4(__lsx_vreplgr2vr_w(*tmp_p));
+        VectorFixedPart = __lsx_vfmadd_s(vb_lo, VectorScaleRatio_BC, VectorFixedPart);
+    }
+
+    __m128i tmp, tmp1;
+
+    while (N >= 8) {
+        const auto va_low_half = __lsx_vinsgr2vr_d(__lsx_vld((const MLAS_INT32X4*)InputA, 0), 0 ,1);
+        const auto va_i16x8 = __lsx_vilvl_b(va_low_half, va_low_half);
+        InputA += 8;
+        va_lo = __lsx_vffint_s_w(MlasShiftRightInt32<DataType>(__lsx_vilvl_h(va_i16x8, va_i16x8), 24));
+        va_hi = __lsx_vffint_s_w(MlasShiftRightInt32<DataType>(__lsx_vilvh_h(va_i16x8, va_i16x8), 24));
+
+        if (!IsScalarB) {
+            const auto vb_low_half = __lsx_vinsgr2vr_d(__lsx_vld((const MLAS_INT32X4*)InputB, 0), 0 ,1);
+            const auto vb_i16x8 = __lsx_vilvl_b(vb_low_half, vb_low_half);
+            InputB += 8;
+            vb_lo = __lsx_vffint_s_w(MlasShiftRightInt32<DataType>(__lsx_vilvl_h(vb_i16x8, vb_i16x8), 24));
+            vb_hi = __lsx_vffint_s_w(MlasShiftRightInt32<DataType>(__lsx_vilvh_h(vb_i16x8, vb_i16x8), 24));
+        }
+
+        MLAS_INT32X4 r_lo, r_hi;
+        if (IsScalarB) {
+            r_lo = __lsx_vftint_w_s(__lsx_vfmadd_s(va_lo, VectorScaleRatio_AC, VectorFixedPart));
+            r_hi = __lsx_vftint_w_s(__lsx_vfmadd_s(va_hi, VectorScaleRatio_AC, VectorFixedPart));
+        } else {
+            r_lo = __lsx_vftint_w_s(__lsx_vfadd_s(__lsx_vfmadd_s(va_lo, VectorScaleRatio_AC, VectorFixedPart), __lsx_vfmul_s(vb_lo, VectorScaleRatio_BC)));
+            r_hi = __lsx_vftint_w_s(__lsx_vfadd_s(__lsx_vfmadd_s(va_hi, VectorScaleRatio_AC, VectorFixedPart), __lsx_vfmul_s(vb_hi, VectorScaleRatio_BC)));
+        }
+        tmp = __lsx_vsat_w(r_lo, 15);
+        tmp1 = __lsx_vsat_w(r_hi, 15);
+         const auto vc_i16x8 = __lsx_vpickev_h(tmp1, tmp);
+
+        MLAS_INT32X4 vc = MlasPackS16_128<DataType>(vc_i16x8, vc_i16x8);
+
+        N -= 8;
+        __lsx_vst(__lsx_vinsgr2vr_d(__lsx_vld((MLAS_INT32X4*)OutputC, 0), __lsx_vpickve2gr_d(vc, 0), 0), (MLAS_INT32X4*)OutputC, 0);
+        OutputC += 8;
+    }
+
+    if (N > 0) {
+        uint8_t TailData[8] = { 0 };
+
+        MlasCopyTailBytes(TailData, (const uint8_t*)InputA, N);
+        const auto va_low_half = __lsx_vinsgr2vr_d(__lsx_vld((const MLAS_INT32X4*)TailData, 0), 0 ,1);
+        const auto va_i16x8 = __lsx_vilvl_b(va_low_half, va_low_half);
+        va_lo = __lsx_vffint_s_w(MlasShiftRightInt32<DataType>(__lsx_vilvl_h(va_i16x8, va_i16x8), 24));
+        va_hi = __lsx_vffint_s_w(MlasShiftRightInt32<DataType>(__lsx_vilvh_h(va_i16x8, va_i16x8), 24));
+
+        if (!IsScalarB) {
+            MlasCopyTailBytes(TailData, (const uint8_t*)InputB, N);
+            const auto vb_low_half = __lsx_vinsgr2vr_d(__lsx_vld((const MLAS_INT32X4*)TailData, 0), 0 ,1);
+            const auto vb_i16x8 = __lsx_vilvl_b(vb_low_half, vb_low_half);
+            vb_lo = __lsx_vffint_s_w(MlasShiftRightInt32<DataType>(__lsx_vilvl_h(vb_i16x8, vb_i16x8), 24));
+            vb_hi = __lsx_vffint_s_w(MlasShiftRightInt32<DataType>(__lsx_vilvh_h(vb_i16x8, vb_i16x8), 24));
+        }
+
+        MLAS_INT32X4 r_lo, r_hi;
+        if (IsScalarB) {
+            r_lo = __lsx_vftint_w_s(__lsx_vfmadd_s(va_lo, VectorScaleRatio_AC, VectorFixedPart));
+            r_hi = __lsx_vftint_w_s(__lsx_vfmadd_s(va_hi, VectorScaleRatio_AC, VectorFixedPart));
+        } else {
+            r_lo = __lsx_vftint_w_s(__lsx_vfadd_s(__lsx_vfmadd_s(va_lo, VectorScaleRatio_AC, VectorFixedPart), __lsx_vfmul_s(vb_lo, VectorScaleRatio_BC)));
+            r_hi = __lsx_vftint_w_s(__lsx_vfadd_s(__lsx_vfmadd_s(va_hi, VectorScaleRatio_AC, VectorFixedPart), __lsx_vfmul_s(vb_hi, VectorScaleRatio_BC)));
+        }
+        tmp = __lsx_vsat_w(r_lo, 15);
+        tmp1 = __lsx_vsat_w(r_hi, 15);
+        const auto vc_i16x8 = __lsx_vpickev_h(tmp1, tmp);
+
+        MLAS_INT32X4 vc = MlasPackS16_128<DataType>(vc_i16x8, vc_i16x8);
+
+        if (N & 4) {
+            __lsx_vstelm_w(vc, (int*)OutputC, 0, 0);
+            N -= 4;
+            OutputC += 4;
+            vc = __lsx_vshuf4i_w(vc, 0x39); //_MM_SHUFFLE(0, 3, 2, 1)
+        }
+
+        uint32_t PackedValueC = (uint32_t)__lsx_vpickve2gr_w(vc, 0);
+        for (size_t i = 0; i < N; ++i) {
+            *((uint8_t*)OutputC + i) = (uint8_t)PackedValueC;
+            PackedValueC >>= 8;
+        }
+    }
+}
 #else
 
 template<typename DataType, bool IsScalarB>
