@@ -144,7 +144,7 @@ Status SliceOpBuilder::AddToModelBuilderImpl(ModelBuilder& model_builder, const
       }
     }
 
-    // Only int32 and float are supported by CoreML slice_by_index.
+    // Int32, float and float16 are supported by CoreML slice_by_index.
     // We convert any int64 model input to int32 when running the CoreML model for the partition.
     // Any other integer data created at runtime is the output from CoreML operations, and should int32 not int64.
     // Based on that, we assume that the actual input when running will be int32, so we override the output data
@@ -214,18 +214,29 @@ Status SliceOpBuilder::AddToModelBuilderImpl(ModelBuilder& model_builder, const
   return Status::OK();
 }
 
-bool SliceOpBuilder::HasSupportedInputsImpl(const Node& node, const OpBuilderInputParams& /*input_params*/,
+bool SliceOpBuilder::HasSupportedInputsImpl(const Node& node,
+                                            [[maybe_unused]] const OpBuilderInputParams& input_params,
                                             const logging::Logger& logger) const {
   int32_t input_type;
   if (!GetType(*node.InputDefs()[0], input_type, logger)) {
     return false;
   }
 
-  if (input_type != ONNX_NAMESPACE::TensorProto_DataType_FLOAT &&
-      input_type != ONNX_NAMESPACE::TensorProto_DataType_INT64) {
-    LOGS(logger, VERBOSE) << "[" << node.OpType() << "] Input type: [" << input_type << "] is not supported";
-    return false;
-  }
+#ifdef COREML_ENABLE_MLPROGRAM
+  // The [Doc](https://apple.github.io/coremltools/source/coremltools.converters.mil.mil.ops.defs.html#coremltools.converters.mil.mil.ops.defs.iOS15.tensor_transformation.slice_by_index)
+  // says ML Program slice_by_index supports fp16 in CoreML 5 (iOS 15).
+  // It's incorrect and CoreML 6+ (iOS16, CoreML spec version >= 7) is required otherwise only float is supported.
+  // CoreML 5:https://github.com/apple/coremltools/blob/89d058ffdcb0b39a03031782d8a448b6889ac425/coremltools/converters/mil/mil/ops/defs/tensor_transformation.py#L515
+  // CoreML 6:https://github.com/apple/coremltools/blob/c3ea4cf56fef1176417246c1b85363417f3e713d/coremltools/converters/mil/mil/ops/defs/iOS15/tensor_transformation.py#L495
+  if (input_params.create_mlprogram && input_params.coreml_version >= 6 &&
+      input_type == ONNX_NAMESPACE::TensorProto_DataType_FLOAT16) {
+  } else
+#endif  // nolint
+    if (input_type != ONNX_NAMESPACE::TensorProto_DataType_FLOAT &&
+        input_type != ONNX_NAMESPACE::TensorProto_DataType_INT64) {
+      LOGS(logger, VERBOSE) << "[" << node.OpType() << "] Input type: [" << input_type << "] is not supported";
+      return false;
+    }
 
   return true;
 }
