@@ -14,12 +14,6 @@ class Expand final : public CudaKernel {
   Status ComputeInternal(OpKernelContext* context) const override;
 };
 
-Status ComputeOutputShape(
-    const std::string& node_name,
-    const TensorShape& lhs_shape,
-    const TensorShape& rhs_shape,
-    TensorShape& out_shape);
-
 Status FuncExpand(
     const CudaKernel* cuda_kernel,
     OpKernelContext* ctx,
