@@ -4,7 +4,7 @@
 #pragma once
 
 #include "core/providers/shared_library/provider_api.h"
-#include "core/common/gsl.h"
+#include <gsl/gsl>
 #include "core/providers/cuda/cuda_kernel.h"
 #include "core/providers/cpu/tensor/transpose.h"
 
