@@ -333,7 +333,7 @@ namespace Dml::GraphDescBuilder
             EdgeShapes inputShapesOverrides(node.InputDefs().size());
 
             // Override the input shapes with shapes that were previously inferred
-            for (int inputIndex = 0; inputIndex < node.InputDefs().size(); ++inputIndex)
+            for (size_t inputIndex = 0; inputIndex < node.InputDefs().size(); ++inputIndex)
             {
                 auto inputDef = node.InputDefs()[inputIndex];
 
@@ -344,7 +344,8 @@ namespace Dml::GraphDescBuilder
                 }
                 else if (inputDef->HasTensorOrScalarShape())
                 {
-                    for (int i = 0; i < inputDef->Shape()->dim_size(); ++i)
+                    int dimSize = gsl::narrow_cast<int>(inputDef->Shape()->dim_size());
+                    for (int i = 0; i < dimSize; ++i)
                     {
                         ORT_THROW_HR_IF(E_INVALIDARG, !inputDef->Shape()->dim(i).has_dim_value());
                         inputShapesOverrides.GetMutableShape(inputIndex).push_back(gsl::narrow_cast<uint32_t>(inputDef->Shape()->dim(i).dim_value()));
@@ -364,7 +365,7 @@ namespace Dml::GraphDescBuilder
             );
 
             ORT_THROW_HR_IF(E_UNEXPECTED, outputShapes.EdgeCount() != node.OutputDefs().size());
-            for (int i = 0; i < node.OutputDefs().size(); ++i)
+            for (size_t i = 0; i < node.OutputDefs().size(); ++i)
             {
                 inferredOutputShapes[node.OutputDefs()[i]->Name()] = outputShapes.GetShape(i);
             }
