@@ -10,6 +10,7 @@
 
 namespace Dml
 {
+    class ExecutionProviderImpl;
 
     // Applies transforms to a Lotus graph. The graph transformer is responsible for setting the execution provider
     // on the graph nodes which DML supports.
@@ -17,16 +18,17 @@ namespace Dml
     {
     public:
         GraphTransformer(
-            const std::string& name
-        ) : onnxruntime::GraphTransformer(name)
-        {
-        }
+            const std::string& name,
+            const onnxruntime::IExecutionProvider* provider
+        );
 
     private:
      onnxruntime::common::Status ApplyImpl(onnxruntime::Graph& graph, bool& modified, int graph_level, const onnxruntime::logging::Logger& logger) const final;
 
     private:
-        void PerformOperatorFusion(onnxruntime::Graph* graph, bool* modified) const;
+        void PerformOperatorFusion(onnxruntime::Graph* graph, bool isMcdmDevice, bool* modified) const;
+
+        const ExecutionProviderImpl* m_providerImpl = nullptr;
     };
 
 } // namespace Dml
