@@ -127,51 +127,51 @@ public:
             DML_OPERATOR_DESC& desc = descs[i];
             ActivationOperatorDescUnion& activationDesc = m_activationDescs[i];
             desc.Desc = &activationDesc;
-
-            if (activationName == AttrValue::ActivationRelu)
+         
+            if (CompareActivationName(activationName, AttrValue::ActivationRelu))
             { 
                 desc.Type = DML_OPERATOR_ACTIVATION_RELU;
-            }
-            else if (activationName == AttrValue::ActivationLeakyRelu)
+            }  
+            else if (CompareActivationName(activationName, AttrValue::ActivationLeakyRelu))
             { 
                 desc.Type = DML_OPERATOR_ACTIVATION_LEAKY_RELU;
                 activationDesc.leakyRelu.Alpha = NextAlpha(desc.Type);
             }
-            else if (activationName == AttrValue::ActivationThresholdedRelu)
+            else if (CompareActivationName(activationName, AttrValue::ActivationThresholdedRelu))
             { 
                 desc.Type = DML_OPERATOR_ACTIVATION_THRESHOLDED_RELU;
                 activationDesc.thresholdedRelu.Alpha = NextAlpha(desc.Type);
-            }
-            else if (activationName == AttrValue::ActivationTanh)
+            }           
+            else if (CompareActivationName(activationName, AttrValue::ActivationTanh))
             { 
                 desc.Type = DML_OPERATOR_ACTIVATION_TANH;
-            }
-            else if (activationName == AttrValue::ActivationScaledTanh)
+            }           
+            else if (CompareActivationName(activationName, AttrValue::ActivationScaledTanh))
             { 
                 desc.Type = DML_OPERATOR_ACTIVATION_SCALED_TANH;
                 activationDesc.scaledTanh.Alpha = NextAlpha(desc.Type);
                 activationDesc.scaledTanh.Beta = NextBeta(desc.Type);
-            }
-            else if (activationName == AttrValue::ActivationSigmoid)
+            }     
+            else if (CompareActivationName(activationName, AttrValue::ActivationSigmoid))
             { 
                 desc.Type = DML_OPERATOR_ACTIVATION_SIGMOID;
-            }
-            else if (activationName == AttrValue::ActivationSigmoidHard)
+            }          
+            else if (CompareActivationName(activationName, AttrValue::ActivationSigmoidHard))
             { 
                 desc.Type = DML_OPERATOR_ACTIVATION_HARD_SIGMOID;
                 activationDesc.hardSigmoid.Alpha = NextAlpha(desc.Type);
                 activationDesc.hardSigmoid.Beta = NextBeta(desc.Type);
-            }
-            else if (activationName == AttrValue::ActivationElu)
+            }         
+            else if (CompareActivationName(activationName, AttrValue::ActivationElu))
             { 
                 desc.Type = DML_OPERATOR_ACTIVATION_ELU;
                 activationDesc.elu.Alpha = NextAlpha(desc.Type);
-            }
-            else if (activationName == AttrValue::ActivationSoftsign)
+            }          
+            else if (CompareActivationName(activationName, AttrValue::ActivationSoftsign))
             { 
                 desc.Type = DML_OPERATOR_ACTIVATION_SOFTSIGN;
-            }
-            else if (activationName == AttrValue::ActivationSoftplus)
+            }         
+            else if (CompareActivationName(activationName, AttrValue::ActivationSoftplus))
             { 
                 desc.Type = DML_OPERATOR_ACTIVATION_SOFTPLUS;
             }
@@ -182,6 +182,12 @@ public:
         }
     }
     
+    bool CompareActivationName(std::string_view activationName, std::string_view attrValue)
+    {
+        auto comparer = [](char a, char b) {return std::tolower(a) == std::tolower(b);};
+        return std::equal(activationName.begin(), activationName.end(), attrValue.begin(), attrValue.end(), comparer);
+    }
+
     void Compute(const MLOperatorKernelContext& kernelContext) override
     {
         // Assume that enough GPU work has been queued up after the RNN operator that it is worth
