@@ -446,6 +446,15 @@ namespace OperatorHelper
         static const int sc_sinceVer_Reshape = 21;
         static const int sc_sinceVer_Cast = 21;
         static const int sc_sinceVer_Shape = 21;
+        static const int sc_sinceVer_Size = 21;
+        static const int sc_sinceVer_CastLike = 21;
+        static const int sc_sinceVer_ConstantOfShape = 21;
+        static const int sc_sinceVer_Flatten = 21;
+        static const int sc_sinceVer_Pad = 21;
+        static const int sc_sinceVer_Transpose = 21;
+        static const int sc_sinceVer_Identity = 21;
+        static const int sc_sinceVer_QLinearMatMul = 21;
+        static const int sc_sinceVer_GroupNorm = 21;
     }
 
     namespace MsftOperatorSet1
