@@ -66,14 +66,6 @@ constexpr ProviderInfo kProvidersInPriorityOrder[] =
             true,
 #else
             false,
-#endif
-        },
-        {
-            kTvmExecutionProvider,
-#ifdef USE_TVM
-            true,
-#else
-            false,
 #endif
         },
         {
@@ -162,6 +154,14 @@ constexpr ProviderInfo kProvidersInPriorityOrder[] =
             true,
 #else
             false,
+#endif
+        },
+        {
+            kWebGpuExecutionProvider,
+#ifdef USE_WEBGPU
+            true,
+#else
+            false,
 #endif
         },
         {
