@@ -9,12 +9,24 @@ namespace js {
 using SupportedTypes =
     TypeList<
         float,
+        MLFloat16,
         int32_t,
         uint32_t>;
 
+using SupportedFloats =
+    TypeList<
+        float,
+        MLFloat16>;
+
 const std::vector<MLDataType>& JsepSupportedDataTypes() {
   static const std::vector<MLDataType> supportedDataTypes = BuildKernelDefConstraintsFromTypeList<SupportedTypes>();
   return supportedDataTypes;
 }
+
+const std::vector<MLDataType>& JsepSupportedFloatTypes() {
+  static const std::vector<MLDataType> supportedDataTypes = BuildKernelDefConstraintsFromTypeList<SupportedFloats>();
+  return supportedDataTypes;
+}
+
 }  // namespace js
-}  // namespace onnxruntime
\ No newline at end of file
+}  // namespace onnxruntime
