@@ -20,6 +20,16 @@ namespace js {
 // a new opset version update applies to Reduce* operators, we may need to add another macro like
 // REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL_WITH_AXIS_IN_INPUT to set input memory type.
 // i.e. we cannot use REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL to version 18 when the opset version is increased.
+#define REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL_WITH_AXIS_IN_INPUT(ReduceOp, sinceVersion, endVersion) \
+  ONNX_OPERATOR_VERSIONED_KERNEL_EX(                                                                        \
+      ReduceOp,                                                                                             \
+      kOnnxDomain,                                                                                          \
+      sinceVersion, endVersion,                                                                             \
+      kJsExecutionProvider,                                                                                 \
+      (*KernelDefBuilder::Create())                                                                         \
+          .TypeConstraint("T", JsepSupportedFloatTypes())                                                   \
+          .InputMemoryType(OrtMemTypeCPU, 1),                                                               \
+      ReduceOp<true>);
 
 #define REGISTER_REDUCE_ELEMENTWISE_KERNEL(ReduceOp, sinceVersion) \
   ONNX_OPERATOR_KERNEL_EX(                                         \
@@ -41,13 +51,15 @@ REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL(ReduceMax, 1, 10);
 REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL(ReduceMax, 11, 11);
 REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL(ReduceMax, 12, 12);
 REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL(ReduceMax, 13, 17);
-REGISTER_REDUCE_ELEMENTWISE_KERNEL(ReduceMax, 18);
+REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL_WITH_AXIS_IN_INPUT(ReduceMax, 18, 19);
+REGISTER_REDUCE_ELEMENTWISE_KERNEL(ReduceMax, 20);
 
 REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL(ReduceMin, 1, 10);
 REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL(ReduceMin, 11, 11);
 REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL(ReduceMin, 12, 12);
 REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL(ReduceMin, 13, 17);
-REGISTER_REDUCE_ELEMENTWISE_KERNEL(ReduceMin, 18);
+REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL_WITH_AXIS_IN_INPUT(ReduceMin, 18, 19);
+REGISTER_REDUCE_ELEMENTWISE_KERNEL(ReduceMin, 20);
 
 REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL(ReduceProd, 1, 10);
 REGISTER_REDUCE_ELEMENTWISE_VERSIONED_KERNEL(ReduceProd, 11, 12);
