@@ -151,6 +151,9 @@ ONNX_OPERATOR_KERNEL_EX(Clip, kOnnxDomain, 13, kJsExecutionProvider,
 JSEP_CLASS_IMPL_ATTRIBUTE_FLOAT_DEFAULT(Elu, Elu, alpha, 1.0)
 JSEP_ELEMENTWISE_KERNEL(Elu, 6, Elu)
 
+JSEP_KERNEL_IMPL(Gelu, Gelu)
+JSEP_ELEMENTWISE_KERNEL(Gelu, 20, Gelu)
+
 JSEP_KERNEL_IMPL(Relu, Relu)
 JSEP_ELEMENTWISE_VERSIONED_KERNEL(Relu, 6, 12, Relu)
 JSEP_ELEMENTWISE_VERSIONED_KERNEL(Relu, 13, 13, Relu)
