@@ -6,12 +6,14 @@
 #include <string>
 #include <memory>
 #include "core/providers/openvino/ov_versions/data_ops.h"
+#include "core/providers/openvino/onnx_ctx_model_helper.h"
 
 namespace onnxruntime {
 namespace openvino_ep {
 
 class GetCapability {
  private:
+  const EPCtxHandler& ep_ctx_handler_;
   const GraphViewer& graph_viewer_;
   std::string device_type_;
   DataOps* data_ops_;
@@ -19,7 +21,8 @@ class GetCapability {
   bool has_external_weights_ = false;
 
  public:
-  GetCapability(const GraphViewer& graph_viewer_param,
+  GetCapability(const EPCtxHandler& ep_ctx_handler,
+                const GraphViewer& graph_viewer_param,
                 const std::string device_type_param,
                 const bool enable_qdq_optimizer);
   virtual std::vector<std::unique_ptr<ComputeCapability>> Execute();
