@@ -4,7 +4,7 @@
 
 #pragma once
 
-#include <onnx/onnx_pb.h>
+#include "core/graph/onnx_protobuf.h"
 #include <vector>
 #include <string>
 
