@@ -20,9 +20,9 @@ namespace EinsumOp {
 
 // Holds ROCM assets required for ROCM ops that need to be executed as part of the Einsum flow
 struct EinsumRocmAssets {
-  explicit EinsumRocmAssets(rocblas_handle rocblas_handle,
+  explicit EinsumRocmAssets(hipblasHandle_t hipblas_handle,
                             const ROCMExecutionProvider* rocm_ep,
-                            Stream* ort_stream, AllocatorPtr gpu_allocator) : rocblas_handle_(rocblas_handle),
+                            Stream* ort_stream, AllocatorPtr gpu_allocator) : hipblas_handle_(hipblas_handle),
                                                                               rocm_ep_(rocm_ep),
                                                                               ort_stream_(ort_stream),
                                                                               gpu_allocator_(gpu_allocator) {}
@@ -31,7 +31,7 @@ struct EinsumRocmAssets {
     return ort_stream_ ? static_cast<hipStream_t>(ort_stream_->GetHandle()) : nullptr;
   }
 
-  rocblas_handle rocblas_handle_;
+  hipblasHandle_t hipblas_handle_;
   const ROCMExecutionProvider* rocm_ep_;
   Stream* ort_stream_;
   AllocatorPtr gpu_allocator_;
