@@ -34,9 +34,9 @@ class RocmProfiler final : public EpProfiler {
   ORT_DISALLOW_COPY_ASSIGNMENT_AND_MOVE(RocmProfiler);
   ~RocmProfiler() {}
   bool StartProfiling(TimePoint) override { return true; }
-  void EndProfiling(TimePoint, Events&) override {};
-  void Start(uint64_t) override{};
-  void Stop(uint64_t) override{};
+  void EndProfiling(TimePoint, Events&) override {}
+  void Start(uint64_t) override {}
+  void Stop(uint64_t) override {}
 };
 
 }  // namespace profiling
