@@ -50,7 +50,7 @@ struct VitisAI_Provider : Provider {
   // Called right after loading the shared library, if this throws any errors Shutdown() will be called and the library unloaded
   void Initialize() override { initialize_vitisai_ep(); }
   // Called right before unloading the shared library
-  void Shutdown() override {}
+  void Shutdown() override { deinitialize_vitisai_ep(); }
 } g_provider;
 
 }  // namespace onnxruntime
