@@ -69,17 +69,17 @@ bool IsNodeSupported(const Node& node, const GraphViewer& graph_viewer, const We
   }
 }
 
-bool IsInputSupported(const NodeArg& input, const std::string& parent_name, const logging::Logger& logger) {
-  const auto& input_name = input.Name();
-  const auto* shape_proto = input.Shape();
+bool IsTensorShapeSupported(const NodeArg& node_arg, const std::string& parent_name,
+                            const logging::Logger& logger, bool allow_empty_input) {
+  const auto& node_arg_name = node_arg.Name();
+  const auto* shape_proto = node_arg.Shape();
   // Optional tensors can be indicated by an empty name, just ignore it.
-  if (input_name.empty()) {
+  if (node_arg_name.empty()) {
     return true;
   }
-  // We do not support input with no shape.
+  // We do not support input/output with no shape.
   if (!shape_proto) {
-    LOGS(logger, VERBOSE) << "Input [" << input_name << "] of [" << parent_name
-                          << "] has not shape";
+    LOGS(logger, VERBOSE) << "Node arg [" << node_arg_name << "] of [" << parent_name << "] has not shape";
     return false;
   }
 
@@ -87,8 +87,11 @@ bool IsInputSupported(const NodeArg& input, const std::string& parent_name, cons
     // WebNN doesn't support dynamic shape - use sessionOptions.freeDimensionOverrides to fix the shape.
     if (!dim.has_dim_value()) {
       LOGS(logger, VERBOSE) << "Dynamic shape is not supported, "
-                            << "use sessionOptions.FreeDimensionOverrides to set a fixed shape for input: "
-                            << input_name;
+                            << "use sessionOptions.FreeDimensionOverrides to set a fixed shape: " << node_arg_name;
+      return false;
+    }
+    if (dim.dim_value() == 0 && !allow_empty_input) {
+      LOGS(logger, VERBOSE) << "The shape of [" << node_arg_name << "] has 0 dimension which is not supported by WebNN";
       return false;
     }
   }
@@ -96,55 +99,29 @@ bool IsInputSupported(const NodeArg& input, const std::string& parent_name, cons
   return true;
 }
 
-std::vector<std::vector<NodeIndex>> GetSupportedNodes(const GraphViewer& graph_viewer,
-                                                      const emscripten::val& wnn_builder,
-                                                      const WebnnDeviceType device_type,
-                                                      const emscripten::val& wnn_limits,
-                                                      const logging::Logger& logger) {
-  std::vector<std::vector<size_t>> supported_node_groups;
+std::unordered_set<const Node*> GetSupportedNodes(const GraphViewer& graph_viewer,
+                                                  const emscripten::val& wnn_builder,
+                                                  const WebnnDeviceType device_type,
+                                                  const emscripten::val& wnn_limits,
+                                                  const logging::Logger& logger) {
+  std::unordered_set<const Node*> supported_nodes;
 
-  for (const auto* input : graph_viewer.GetInputs()) {
-    if (!IsInputSupported(*input, "graph", logger)) {
-      return supported_node_groups;
-    }
-  }
-
-  std::vector<size_t> supported_node_group;
-  const auto& node_indices = graph_viewer.GetNodesInTopologicalOrder();
-
-  for (size_t i = 0; i < node_indices.size(); i++) {
-    auto node_idx = node_indices[i];
-    const auto* node(graph_viewer.GetNode(node_idx));
-    bool supported = false;
-    // Firstly check if platform supports the WebNN op.
-    if (CheckSingleOp(node->OpType(), wnn_builder, device_type)) {
-      LOGS(logger, VERBOSE) << "Operator type: [" << node->OpType() << "] is supported by browser";
-      supported = IsNodeSupported(*node, graph_viewer, device_type, wnn_limits, logger);
-    }
-
-    LOGS(logger, VERBOSE) << "Operator type: [" << node->OpType()
-                          << "] index: [" << node_idx
-                          << "] name: [" << node->Name()
+  for (const auto& node : graph_viewer.Nodes()) {
+    const bool supported = IsNodeSupported(node, graph_viewer, device_type, wnn_limits, logger);
+    LOGS(logger, VERBOSE) << "Operator type: [" << node.OpType()
+                          << "] index: [" << node.Index()
+                          << "] name: [" << node.Name()
                           << "] supported: [" << supported
                           << "]";
     if (supported) {
-      supported_node_group.push_back(node_idx);
-    } else {
-      if (!supported_node_group.empty()) {
-        supported_node_groups.push_back(supported_node_group);
-        supported_node_group.clear();
-      }
+      supported_nodes.insert(&node);
     }
   }
 
-  if (!supported_node_group.empty()) {
-    supported_node_groups.push_back(supported_node_group);
-  }
-
-  return supported_node_groups;
+  return supported_nodes;
 }
 
-bool AreInputDataTypesSame(const std::string& op_type,
+bool AreInputDataTypesSame(const std::string_view op_type,
                            gsl::span<const int32_t> input_types,
                            const logging::Logger& logger) {
   for (size_t i = 1; i < input_types.size(); i++) {
@@ -164,33 +141,51 @@ bool IsSupportedDataType(const int32_t onnx_data_type, const emscripten::val& we
   if (it == onnx_to_webnn_data_type_map.end())
     return false;
 
-  std::string webnn_data_type = it->second;
+  const std::string_view webnn_data_type = it->second;
 
   // Check if WebNN supports the data type.
-  emscripten::val is_supported = webnn_supported_data_types.call<emscripten::val>("includes",
-                                                                                  emscripten::val(webnn_data_type));
+  emscripten::val is_supported =
+      webnn_supported_data_types.call<emscripten::val>("includes", emscripten::val(std::string(webnn_data_type)));
   return is_supported.as<bool>();
 }
 
 // Check if the input or output data type of ONNX node is supported by the WebNN operator.
-bool IsDataTypeSupportedByOp(const std::string& onnx_op_type,
+bool IsDataTypeSupportedByOp(const std::string_view onnx_op_type,
                              const int32_t onnx_data_type,
                              const emscripten::val& wnn_limits,
-                             const std::string& webnn_input_output_name,
-                             const std::string& onnx_input_output_name,
+                             const std::string_view webnn_input_output_name,
+                             const std::string_view onnx_input_output_name,
                              const logging::Logger& logger) {
-  std::string webnn_op_type;
-  if (!GetWebNNOpType(onnx_op_type, webnn_op_type))
-    return false;
+  const std::string_view webnn_op_type = GetWebNNOpType(onnx_op_type);
 
-  if (!IsSupportedDataType(onnx_data_type, wnn_limits[webnn_op_type][webnn_input_output_name]["dataTypes"])) {
-    LOGS(logger, VERBOSE) << "[" << onnx_op_type
-                          << "] " << onnx_input_output_name
-                          << " type: [" << onnx_data_type
-                          << "] is not supported for now";
+  return !webnn_op_type.empty() &&
+         IsDataTypeSupportedByWebNNOp(onnx_op_type, webnn_op_type, onnx_data_type, wnn_limits,
+                                      webnn_input_output_name, onnx_input_output_name, logger);
+}
+
+bool IsDataTypeSupportedByWebNNOp(const std::string_view onnx_op_type,
+                                  const std::string_view webnn_op_type,
+                                  const int32_t onnx_data_type,
+                                  const emscripten::val& wnn_limits,
+                                  const std::string_view webnn_input_output_name,
+                                  const std::string_view onnx_input_output_name,
+                                  const logging::Logger& logger) {
+  if (wnn_limits[std::string(webnn_op_type)].isUndefined()) {
+    LOGS(logger, VERBOSE) << "[" << onnx_op_type << "] WebNN op [" << webnn_op_type << "] is not supported for now";
     return false;
   }
 
+  if (wnn_limits[std::string(webnn_op_type)][std::string(webnn_input_output_name)].isUndefined()) {
+    LOGS(logger, VERBOSE) << "[" << onnx_op_type << "] WebNN op [" << webnn_op_type << "] doesn't have parameter ["
+                          << webnn_input_output_name << "]";
+    return false;
+  }
+  if (!IsSupportedDataType(
+          onnx_data_type, wnn_limits[std::string(webnn_op_type)][std::string(webnn_input_output_name)]["dataTypes"])) {
+    LOGS(logger, VERBOSE) << "[" << onnx_op_type << "] " << onnx_input_output_name << "'s data type: ["
+                          << onnx_data_type << "] is not supported by WebNN op [" << webnn_op_type << "] for now";
+    return false;
+  }
   return true;
 }
 
@@ -226,6 +221,12 @@ bool GetBidirectionalBroadcastShape(std::vector<int64_t>& shape_a,
 
 bool SetWebnnDataType(emscripten::val& desc, const int32_t data_type) {
   switch (data_type) {
+    case ONNX_NAMESPACE::TensorProto_DataType_INT4:
+      desc.set("dataType", emscripten::val("int4"));
+      return true;
+    case ONNX_NAMESPACE::TensorProto_DataType_UINT4:
+      desc.set("dataType", emscripten::val("uint4"));
+      return true;
     case ONNX_NAMESPACE::TensorProto_DataType_BOOL:
     case ONNX_NAMESPACE::TensorProto_DataType_UINT8:
       desc.set("dataType", emscripten::val("uint8"));
@@ -261,5 +262,67 @@ bool IsMLTensorSupported() {
   return is_supported;
 }
 
+// Convert int8 to uint4/int4 (stored as uint8)
+uint8_t PackInt8ToUint8AsNibble(int8_t value, const int32_t& data_type) {
+  uint8_t result = 0;
+  if (data_type == ONNX_NAMESPACE::TensorProto_DataType_UINT4) {
+    if (value < 0 || value > 15) {
+      ORT_THROW("Value cannot be safely converted to uint4.");
+    }
+    result |= (static_cast<uint8_t>(value) << 4);
+  } else {
+    if (value < -8 || value > 7) {
+      ORT_THROW("Value cannot be safely converted to int4.");
+    }
+    result |= (value << 4);
+  }
+
+  return result;
+}
+
+// Convert float32 to float16 (stored as uint16)
+uint16_t PackFloat32ToUint16AsFloat16(float value) {
+  uint32_t float32_bits;
+
+  // Safely copy the float bits into an integer
+  std::memcpy(&float32_bits, &value, sizeof(float));
+
+  // Extract the sign, exponent, and mantissa from the float32 bits
+  uint32_t sign = (float32_bits >> 31) & 0x1;
+  uint32_t exponent = (float32_bits >> 23) & 0xFF;
+  uint32_t mantissa = float32_bits & 0x7FFFFF;
+
+  // Shift the sign for float16
+  uint16_t sign_float16 = sign << 15;
+
+  // Handle special cases: Infinity and NaN
+  if (exponent == 255) {
+    return sign_float16 | (0x1F << 10) | (mantissa ? 0x200 : 0);
+  }
+  // Handle zero and subnormal numbers in float32
+  if (exponent == 0) {
+    return sign_float16 | (mantissa >> 13);
+  }
+
+  // Adjust the exponent for float16 (subtract bias difference: 127 - 15 = 112)
+  int exponent_float16 = exponent - 112;
+
+  // Handle exponent overflow (larger than float16 can represent)
+  if (exponent_float16 >= 0x1F) {
+    return sign_float16 | (0x1F << 10);
+  }
+  // Handle exponent underflow (smaller than float16 can represent)
+  if (exponent_float16 <= 0) {
+    mantissa = (mantissa | 0x800000) >> (1 - exponent_float16);
+    return sign_float16 | (mantissa >> 13);
+  }
+
+  // Adjust the mantissa by shifting it to fit float16 format (round to nearest even)
+  uint16_t mantissa_float16 = (mantissa + 0x1000) >> 13;
+
+  // Combine sign, exponent, and mantissa into the final float16 representation
+  return sign_float16 | (exponent_float16 << 10) | mantissa_float16;
+}
+
 }  // namespace webnn
 }  // namespace onnxruntime
