@@ -10,11 +10,19 @@ namespace onnxruntime {
 namespace python {
 namespace py = pybind11;
 
-void CreateInferencePybindStateModule(py::module& m);
+#if defined(USE_MPI) && defined(ORT_USE_NCCL)
+static constexpr bool HAS_COLLECTIVE_OPS = true;
+#else
+static constexpr bool HAS_COLLECTIVE_OPS = false;
+#endif
+
+bool CreateInferencePybindStateModule(py::module& m);
 void CreateQuantPybindModule(py::module& m);
 
 PYBIND11_MODULE(onnxruntime_pybind11_state, m) {
-  CreateInferencePybindStateModule(m);
+  if (!CreateInferencePybindStateModule(m)) {
+    throw pybind11::import_error();
+  }
   // move it out of shared method since training build has a little different behavior.
   m.def(
       "get_available_providers", []() -> const std::vector<std::string>& { return GetAvailableExecutionProviderNames(); },
@@ -24,6 +32,7 @@ PYBIND11_MODULE(onnxruntime_pybind11_state, m) {
 
   m.def("get_version_string", []() -> std::string { return ORT_VERSION; });
   m.def("get_build_info", []() -> std::string { return ORT_BUILD_INFO; });
+  m.def("has_collective_ops", []() -> bool { return HAS_COLLECTIVE_OPS; });
   CreateQuantPybindModule(m);
 }
 }  // namespace python
