@@ -82,7 +82,11 @@ class ModelTestBuilder {
   }
 
   template <typename T>
-  NodeArg* MakeInput(const std::vector<int64_t>& shape, const std::vector<T>& data) {
+  NodeArg* MakeInput(const std::vector<int64_t>& shape, const std::vector<T>& data,
+                     AllocatorPtr allocator = nullptr) {
+    if (!allocator) {
+      allocator = TestCPUExecutionProvider()->CreatePreferredAllocators()[0];
+    }
     ONNX_NAMESPACE::TypeProto type_proto;
     type_proto.mutable_tensor_type()->set_elem_type(utils::ToTensorProtoElementType<T>());
 
@@ -93,7 +97,7 @@ class ModelTestBuilder {
     }
 
     OrtValue input_value;
-    CreateMLValue<T>(TestCPUExecutionProvider()->CreatePreferredAllocators()[0],
+    CreateMLValue<T>(allocator,
                      shape,
                      data,
                      &input_value);
@@ -104,17 +108,19 @@ class ModelTestBuilder {
   }
 
   template <typename T>
-  NodeArg* MakeInput(const std::vector<int64_t>& shape, T min, T max) {
-    return MakeInput<T>(shape, rand_gen_.Uniform<T>(shape, min, max));
+  NodeArg* MakeInput(const std::vector<int64_t>& shape, T min, T max,
+                     AllocatorPtr allocator = nullptr) {
+    return MakeInput<T>(shape, rand_gen_.Uniform<T>(shape, min, max), allocator);
   }
 
-  NodeArg* MakeInputBool(const std::vector<int64_t>& shape) {
+  NodeArg* MakeInputBool(const std::vector<int64_t>& shape,
+                         AllocatorPtr allocator = nullptr) {
     std::vector<uint8_t> data_uint8 = rand_gen_.Uniform<uint8_t>(shape, 0, 1);
     std::vector<bool> data;
     for (uint8_t x : data_uint8) {
       data.push_back(x != 0);
     }
-    return MakeInput<bool>(shape, data);
+    return MakeInput<bool>(shape, data, allocator);
   }
 
   template <typename T>
@@ -298,17 +304,17 @@ class ModelTestBuilder {
     return AddNode("Conv", input_args, {output_arg});
   }
 
-  template <typename T>
-  typename std::enable_if<IsTypeQuantLinearCompatible<T>::value, Node&>::type
+  template <typename ZpType, typename ScaleType = float>
+  typename std::enable_if<IsTypeQuantLinearCompatible<ZpType>::value, Node&>::type
   AddQuantizeLinearNode(NodeArg* input_arg,
-                        float input_scale,
-                        T input_zero_point,
+                        ScaleType input_scale,
+                        ZpType input_zero_point,
                         NodeArg* output_arg,
                         bool use_ms_domain = false) {
     std::vector<NodeArg*> input_args;
     input_args.push_back(input_arg);
-    input_args.push_back(MakeScalarInitializer<float>(input_scale));
-    input_args.push_back(MakeScalarInitializer<T>(input_zero_point));
+    input_args.push_back(MakeScalarInitializer<ScaleType>(input_scale));
+    input_args.push_back(MakeScalarInitializer<ZpType>(input_zero_point));
 
     std::string domain = use_ms_domain ? kMSDomain : "";
     return AddNode("QuantizeLinear", input_args, {output_arg}, domain);
@@ -376,17 +382,17 @@ class ModelTestBuilder {
                               NodeArg* output_arg,
                               bool use_ms_domain = false);
 
-  template <typename T>
-  typename std::enable_if<IsTypeDequantLinearCompatible<T>::value, Node&>::type
+  template <typename ZpType, typename ScaleType = float>
+  typename std::enable_if<IsTypeDequantLinearCompatible<ZpType>::value, Node&>::type
   AddDequantizeLinearNode(NodeArg* input_arg,
-                          float input_scale,
-                          T input_zero_point,
+                          ScaleType input_scale,
+                          ZpType input_zero_point,
                           NodeArg* output_arg,
                           bool use_ms_domain = false) {
     std::vector<NodeArg*> input_args;
     input_args.push_back(input_arg);
-    input_args.push_back(MakeScalarInitializer<float>(input_scale));
-    input_args.push_back(MakeScalarInitializer<T>(input_zero_point));
+    input_args.push_back(MakeScalarInitializer<ScaleType>(input_scale));
+    input_args.push_back(MakeScalarInitializer<ZpType>(input_zero_point));
 
     std::string domain = use_ms_domain ? kMSDomain : "";
     return AddNode("DequantizeLinear", input_args, {output_arg}, domain);
