@@ -5,9 +5,12 @@
 
 #include <vector>
 #include <string>
+#include <type_traits>
 
 #include "graph_transform_test_builder.h"
 
+#include "core/framework/int4.h"
+#include "core/common/span_utils.h"
 #include "core/optimizer/qdq_transformer/selectors_actions/qdq_selector_action_transformer.h"
 #include "core/session/inference_session.h"
 
@@ -21,26 +24,40 @@ using GetQDQTestCaseFn = std::function<void(ModelTestBuilder& builder)>;
 
 template <typename T>
 typename std::enable_if<IsTypeQuantLinearCompatible<T>::value, NodeArg*>::type
-AddQDQNodePair(ModelTestBuilder& builder, NodeArg* q_input, float scale, T zp = T()) {
+AddQDQNodePair(ModelTestBuilder& builder, NodeArg* q_input, float scale, T zp = T(), bool use_ms_domain = false) {
   auto* q_output = builder.MakeIntermediate();
   auto* dq_output = builder.MakeIntermediate();
-  builder.AddQuantizeLinearNode<T>(q_input, scale, zp, q_output);
-  builder.AddDequantizeLinearNode<T>(q_output, scale, zp, dq_output);
+  builder.AddQuantizeLinearNode<T>(q_input, scale, zp, q_output, use_ms_domain);
+  builder.AddDequantizeLinearNode<T>(q_output, scale, zp, dq_output, use_ms_domain);
   return dq_output;
 }
 
 template <typename T>
 typename std::enable_if<IsTypeQuantLinearCompatible<T>::value, NodeArg*>::type
-AddQDQNodePairWithOutputAsGraphOutput(ModelTestBuilder& builder, NodeArg* q_input, float scale, T zp = T()) {
+AddQDQNodePairWithOutputAsGraphOutput(ModelTestBuilder& builder, NodeArg* q_input, float scale, T zp = T(),
+                                      bool use_ms_domain = false) {
   auto* q_output = builder.MakeIntermediate();
   auto* dq_output = builder.MakeOutput();
-  builder.AddQuantizeLinearNode<T>(q_input, scale, zp, q_output);
-  builder.AddDequantizeLinearNode<T>(q_output, scale, zp, dq_output);
+  builder.AddQuantizeLinearNode<T>(q_input, scale, zp, q_output, use_ms_domain);
+  builder.AddDequantizeLinearNode<T>(q_output, scale, zp, dq_output, use_ms_domain);
+  return dq_output;
+}
+
+template <typename T>
+typename std::enable_if<IsTypeQuantLinearCompatible<T>::value, NodeArg*>::type
+AddQDQNodePair(ModelTestBuilder& builder, NodeArg* q_input, const std::vector<float>& scales,
+               const std::vector<T>& zero_points, const NodeAttributes* q_attrs = nullptr,
+               const NodeAttributes* dq_attrs = nullptr, bool use_ms_domain = false) {
+  auto* q_output = builder.MakeIntermediate();
+  auto* dq_output = builder.MakeIntermediate();
+  builder.AddQuantizeLinearNode<T>(q_input, scales, zero_points, q_output, q_attrs, use_ms_domain);
+  builder.AddDequantizeLinearNode<T>(q_output, scales, zero_points, dq_output, dq_attrs, use_ms_domain);
   return dq_output;
 }
 
 template <typename InputType, typename WeightType, typename BiasType, typename OutputType>
-GetQDQTestCaseFn BuildQDQConvTransposeTestCase(const std::vector<int64_t>& input_shape, const std::vector<int64_t>& weights_shape) {
+GetQDQTestCaseFn BuildQDQConvTransposeTestCase(const std::vector<int64_t>& input_shape,
+                                               const std::vector<int64_t>& weights_shape) {
   return [input_shape, weights_shape](ModelTestBuilder& builder) {
     auto* input_arg = builder.MakeInput<float>(input_shape, -1.f, 1.f);
     auto* output_arg = builder.MakeOutput();
@@ -70,7 +87,8 @@ GetQDQTestCaseFn BuildQDQConvTransposeTestCase(const std::vector<int64_t>& input
                                                 dq_w_output);
 
     auto* dq_bias_output = builder.MakeIntermediate();
-    auto* bias = builder.MakeInitializer<BiasType>({weights_shape[0]}, static_cast<BiasType>(0), static_cast<BiasType>(127));
+    auto* bias = builder.MakeInitializer<BiasType>({weights_shape[0]}, static_cast<BiasType>(0),
+                                                   static_cast<BiasType>(127));
     builder.AddDequantizeLinearNode<BiasType>(bias, .0012f,
                                               0,
                                               dq_bias_output);
@@ -92,8 +110,10 @@ GetQDQTestCaseFn BuildQDQConvTransposeTestCase(const std::vector<int64_t>& input
 }
 
 template <typename InputType, typename WeightType, typename BiasType, typename OutputType>
-GetQDQTestCaseFn BuildQDQConvTestCase(const std::vector<int64_t>& input_shape, const std::vector<int64_t>& weights_shape) {
-  return [input_shape, weights_shape](ModelTestBuilder& builder) {
+GetQDQTestCaseFn BuildQDQConvTestCase(const std::vector<int64_t>& input_shape,
+                                      const std::vector<int64_t>& weights_shape,
+                                      bool use_contrib_qdq = false) {
+  return [input_shape, weights_shape, use_contrib_qdq](ModelTestBuilder& builder) {
     auto* input_arg = builder.MakeInput<float>(input_shape, -1.f, 1.f);
     auto* output_arg = builder.MakeOutput();
 
@@ -119,34 +139,39 @@ GetQDQTestCaseFn BuildQDQConvTestCase(const std::vector<int64_t>& input_shape, c
     auto* weight = builder.MakeInitializer<WeightType>(weights_shape, weight_min_value, weight_max_value);
     builder.AddDequantizeLinearNode<WeightType>(weight, .03f,
                                                 (weight_min_value + weight_max_value) / 2 + 1,
-                                                dq_w_output);
+                                                dq_w_output,
+                                                use_contrib_qdq);
 
     auto* dq_bias_output = builder.MakeIntermediate();
-    auto* bias = builder.MakeInitializer<BiasType>({weights_shape[0]}, static_cast<BiasType>(0), static_cast<BiasType>(127));
+    auto* bias = builder.MakeInitializer<BiasType>({weights_shape[0]}, static_cast<BiasType>(0),
+                                                   static_cast<BiasType>(127));
     builder.AddDequantizeLinearNode<BiasType>(bias, .0012f,
                                               0,
-                                              dq_bias_output);
+                                              dq_bias_output,
+                                              use_contrib_qdq);
 
     auto* conv_output = builder.MakeIntermediate();
     auto* dq_output = AddQDQNodePair<InputType>(builder, input_arg, .04f,
-                                                (input_min_value + input_max_value) / 2 + 1);
+                                                (input_min_value + input_max_value) / 2 + 1, use_contrib_qdq);
     builder.AddNode("Conv", {dq_output, dq_w_output, dq_bias_output}, {conv_output});
 
     auto* q_output = builder.MakeIntermediate();
     builder.AddQuantizeLinearNode<OutputType>(conv_output, .039f,
                                               (OutputLimits::min() + OutputLimits::max()) / 2 + 1,
-                                              q_output);
+                                              q_output,
+                                              use_contrib_qdq);
 
     builder.AddDequantizeLinearNode<OutputType>(q_output, .039f,
                                                 (OutputLimits::min() + OutputLimits::max()) / 2 + 1,
-                                                output_arg);
+                                                output_arg,
+                                                use_contrib_qdq);
   };
 }
 
 template <typename InputType, typename OutputType>
 GetQDQTestCaseFn BuildQDQAveragePoolTestCase(const std::vector<int64_t>& input_shape,
-                                             int64_t count_include_pad = 0) {
-  return [input_shape, count_include_pad](ModelTestBuilder& builder) {
+                                             int64_t count_include_pad = 0, bool use_contrib_qdq = false) {
+  return [input_shape, count_include_pad, use_contrib_qdq](ModelTestBuilder& builder) {
 
 #ifdef USE_NNAPI  // NNAPI require consistent scales/ZPs for DQ -> Pool -> Q
     float dq_scale = 0.0038f;
@@ -167,7 +192,7 @@ GetQDQTestCaseFn BuildQDQAveragePoolTestCase(const std::vector<int64_t>& input_s
     auto* input_arg = builder.MakeInput<float>(input_shape, -1.f, 1.f);
     auto* output_arg = builder.MakeOutput();
     // add QDQ + AveragePool
-    auto* dq_output = AddQDQNodePair<InputType>(builder, input_arg, dq_scale, dq_zp);
+    auto* dq_output = AddQDQNodePair<InputType>(builder, input_arg, dq_scale, dq_zp, use_contrib_qdq);
     auto* averagepool_output = builder.MakeIntermediate();
     Node& pool_node = builder.AddNode("AveragePool", {dq_output}, {averagepool_output});
     std::vector<int64_t> pads((input_shape.size() - 2) * 2, 1);
@@ -183,11 +208,13 @@ GetQDQTestCaseFn BuildQDQAveragePoolTestCase(const std::vector<int64_t>& input_s
     builder.AddQuantizeLinearNode<OutputType>(averagepool_output,
                                               pool_output_scale,
                                               pool_output_zp,
-                                              q_output);
+                                              q_output,
+                                              use_contrib_qdq);
     builder.AddDequantizeLinearNode<OutputType>(q_output,
                                                 q_scale,
                                                 q_zp,
-                                                output_arg);
+                                                output_arg,
+                                                use_contrib_qdq);
   };
 }
 
@@ -227,8 +254,9 @@ GetQDQTestCaseFn BuildQDQMaxPoolTestCase(const std::vector<int64_t>& input_shape
 }
 
 template <typename InputType, typename OutputType>
-GetQDQTestCaseFn BuildQDQGlobalAveragePoolTestCase(const std::vector<int64_t>& input_shape) {
-  return [input_shape](ModelTestBuilder& builder) {
+GetQDQTestCaseFn BuildQDQGlobalAveragePoolTestCase(const std::vector<int64_t>& input_shape,
+                                                   bool use_contrib_qdq = false) {
+  return [input_shape, use_contrib_qdq](ModelTestBuilder& builder) {
     float dq_scale = 0.0035f;
     float pool_output_scale = 0.0038f;
     float q_scale = 0.0039f;
@@ -239,7 +267,7 @@ GetQDQTestCaseFn BuildQDQGlobalAveragePoolTestCase(const std::vector<int64_t>& i
     auto* input_arg = builder.MakeInput<float>(input_shape, -1.f, 1.f);
     auto* output_arg = builder.MakeOutput();
     // add QDQ + GlobalAveragePool
-    auto* dq_output = AddQDQNodePair<InputType>(builder, input_arg, dq_scale, dq_zp);
+    auto* dq_output = AddQDQNodePair<InputType>(builder, input_arg, dq_scale, dq_zp, use_contrib_qdq);
     auto* globalaveragepool_output = builder.MakeIntermediate();
     builder.AddNode("GlobalAveragePool", {dq_output}, {globalaveragepool_output});
 
@@ -248,11 +276,13 @@ GetQDQTestCaseFn BuildQDQGlobalAveragePoolTestCase(const std::vector<int64_t>& i
     builder.AddQuantizeLinearNode<OutputType>(globalaveragepool_output,
                                               pool_output_scale,
                                               pool_output_zp,
-                                              q_output);
+                                              q_output,
+                                              use_contrib_qdq);
     builder.AddDequantizeLinearNode<OutputType>(q_output,
                                                 q_scale,
                                                 q_zp,
-                                                output_arg);
+                                                output_arg,
+                                                use_contrib_qdq);
   };
 }
 
@@ -262,10 +292,11 @@ GetQDQTestCaseFn BuildQDQResizeTestCase(const std::vector<int64_t>& input_shape,
                                         const std::string& mode = "nearest",
                                         const std::string& coordinate_transformation_mode = "half_pixel",
                                         const std::string& nearest_mode = "round_prefer_floor",
-                                        bool add_dq_output_float = false) {
+                                        bool add_dq_output_float = false,
+                                        bool use_contrib_qdq = false) {
   static_assert(std::is_same_v<InputType, int8_t> || std::is_same_v<InputType, uint8_t>);
   return [input_shape, sizes_data, mode, coordinate_transformation_mode,
-          nearest_mode, add_dq_output_float](ModelTestBuilder& builder) {
+          nearest_mode, add_dq_output_float, use_contrib_qdq](ModelTestBuilder& builder) {
     auto* input1_arg = builder.MakeInput<InputType>(input_shape,
                                                     std::numeric_limits<InputType>::min(),
                                                     std::numeric_limits<InputType>::max());
@@ -276,7 +307,7 @@ GetQDQTestCaseFn BuildQDQResizeTestCase(const std::vector<int64_t>& input_shape,
 
     // add DQ
     auto* dq_output = builder.MakeIntermediate();
-    builder.AddDequantizeLinearNode<InputType>(input1_arg, .003f, 1, dq_output);
+    builder.AddDequantizeLinearNode<InputType>(input1_arg, .003f, 1, dq_output, use_contrib_qdq);
 
     // add Resize
     auto* resize_output = builder.MakeIntermediate();
@@ -292,21 +323,21 @@ GetQDQTestCaseFn BuildQDQResizeTestCase(const std::vector<int64_t>& input_shape,
     if (add_dq_output_float) {
       // add Q
       output_arg = builder.MakeIntermediate();
-      builder.AddQuantizeLinearNode<InputType>(resize_output, .003f, 1, output_arg);
+      builder.AddQuantizeLinearNode<InputType>(resize_output, .003f, 1, output_arg, use_contrib_qdq);
       auto* f_dq_output = builder.MakeOutput();
-      builder.AddDequantizeLinearNode<InputType>(output_arg, .003f, 1, f_dq_output);
+      builder.AddDequantizeLinearNode<InputType>(output_arg, .003f, 1, f_dq_output, use_contrib_qdq);
     } else {
       output_arg = builder.MakeOutput();
       // add Q
-      builder.AddQuantizeLinearNode<InputType>(resize_output, .003f, 1, output_arg);
+      builder.AddQuantizeLinearNode<InputType>(resize_output, .003f, 1, output_arg, use_contrib_qdq);
     }
   };
 }
 
 template <typename Input1Type, typename Input2Type, typename OutputType>
 GetQDQTestCaseFn BuildBinaryOpTestCase(const std::vector<int64_t>& input_shape,
-                                       const std::string& op_type) {
-  return [input_shape, op_type](ModelTestBuilder& builder) {
+                                       const std::string& op_type, bool use_contrib_qdq = false) {
+  return [input_shape, op_type, use_contrib_qdq](ModelTestBuilder& builder) {
     auto* input1_arg = builder.MakeInput<float>(input_shape, -1.f, 1.f);
     auto* input2_arg = builder.MakeInput<float>(input_shape, -1.f, 1.f);
     auto* output_arg = builder.MakeOutput();
@@ -329,11 +360,13 @@ GetQDQTestCaseFn BuildBinaryOpTestCase(const std::vector<int64_t>& input_shape,
     builder.AddQuantizeLinearNode<Input1Type>(input1_arg,
                                               q_scale,
                                               std::numeric_limits<Input1Type>::max() / 2,
-                                              q1_output);
+                                              q1_output,
+                                              use_contrib_qdq);
     builder.AddDequantizeLinearNode<Input1Type>(q1_output,
                                                 op_input_scale,
                                                 std::numeric_limits<Input1Type>::max() / 2,
-                                                dq1_output);
+                                                dq1_output,
+                                                use_contrib_qdq);
 
     // add QDQ 2
     auto* q2_output = builder.MakeIntermediate();
@@ -341,11 +374,13 @@ GetQDQTestCaseFn BuildBinaryOpTestCase(const std::vector<int64_t>& input_shape,
     builder.AddQuantizeLinearNode<Input2Type>(input2_arg,
                                               q_scale,
                                               std::numeric_limits<Input2Type>::max() / 2,
-                                              q2_output);
+                                              q2_output,
+                                              use_contrib_qdq);
     builder.AddDequantizeLinearNode<Input2Type>(q2_output,
                                                 op_input_scale,
                                                 std::numeric_limits<Input2Type>::max() / 2,
-                                                dq2_output);
+                                                dq2_output,
+                                                use_contrib_qdq);
 
     // add binary operator
     auto* binary_op_output = builder.MakeIntermediate();
@@ -356,26 +391,30 @@ GetQDQTestCaseFn BuildBinaryOpTestCase(const std::vector<int64_t>& input_shape,
     builder.AddQuantizeLinearNode<OutputType>(binary_op_output,
                                               op_output_scale,
                                               std::numeric_limits<OutputType>::max() / 2,
-                                              q3_output);
+                                              q3_output,
+                                              use_contrib_qdq);
     builder.AddDequantizeLinearNode<OutputType>(q3_output,
                                                 dq_scale,
                                                 std::numeric_limits<OutputType>::max() / 2,
-                                                output_arg);
+                                                output_arg,
+                                                use_contrib_qdq);
   };
 }
 
 template <typename InputType, typename OutputType>
 GetQDQTestCaseFn BuildConsolidationTestCase(
     const std::vector<int64_t>& input_shape,
-    const int64_t& axis) {
-  return [input_shape, axis](ModelTestBuilder& builder) {
-    auto* input_arg = builder.MakeInput<float>(input_shape, std::numeric_limits<float>::min(), std::numeric_limits<float>::max());
+    const int64_t& axis,
+    bool use_contrib_qdq = false) {
+  return [input_shape, axis, use_contrib_qdq](ModelTestBuilder& builder) {
+    auto* input_arg = builder.MakeInput<float>(input_shape, std::numeric_limits<float>::min(),
+                                               std::numeric_limits<float>::max());
     InputType dq_zp = std::numeric_limits<InputType>::max() / 2;
     OutputType q_zp = std::numeric_limits<OutputType>::max() / 2;
     auto* upper_dq_output = builder.MakeIntermediate();
     auto* upper_q_output = builder.MakeIntermediate();
-    builder.AddQuantizeLinearNode<InputType>(input_arg, .003f, q_zp, upper_q_output);
-    builder.AddDequantizeLinearNode<InputType>(upper_q_output, .003f, dq_zp, upper_dq_output);
+    builder.AddQuantizeLinearNode<InputType>(input_arg, .003f, q_zp, upper_q_output, use_contrib_qdq);
+    builder.AddDequantizeLinearNode<InputType>(upper_q_output, .003f, dq_zp, upper_dq_output, use_contrib_qdq);
 
     // add Split
 
@@ -392,21 +431,22 @@ GetQDQTestCaseFn BuildConsolidationTestCase(
     auto* lower_q_output_1 = builder.MakeIntermediate();
     auto* lower_q_output_2 = builder.MakeIntermediate();
     auto* lower_q_output_3 = builder.MakeIntermediate();
-    builder.AddQuantizeLinearNode<OutputType>(split_output_1, .003f, q_zp, lower_q_output_1);
-    builder.AddQuantizeLinearNode<OutputType>(split_output_2, .003f, q_zp, lower_q_output_2);
-    builder.AddQuantizeLinearNode<OutputType>(split_output_3, .003f, q_zp, lower_q_output_3);
+    builder.AddQuantizeLinearNode<OutputType>(split_output_1, .003f, q_zp, lower_q_output_1, use_contrib_qdq);
+    builder.AddQuantizeLinearNode<OutputType>(split_output_2, .003f, q_zp, lower_q_output_2, use_contrib_qdq);
+    builder.AddQuantizeLinearNode<OutputType>(split_output_3, .003f, q_zp, lower_q_output_3, use_contrib_qdq);
     auto* q_split_output_1 = builder.MakeOutput();
     auto* q_split_output_2 = builder.MakeOutput();
     auto* q_split_output_3 = builder.MakeOutput();
-    builder.AddDequantizeLinearNode<OutputType>(lower_q_output_1, .003f, dq_zp, q_split_output_1);
-    builder.AddDequantizeLinearNode<OutputType>(lower_q_output_2, .003f, dq_zp, q_split_output_2);
-    builder.AddDequantizeLinearNode<OutputType>(lower_q_output_3, .003f, dq_zp, q_split_output_3);
+    builder.AddDequantizeLinearNode<OutputType>(lower_q_output_1, .003f, dq_zp, q_split_output_1, use_contrib_qdq);
+    builder.AddDequantizeLinearNode<OutputType>(lower_q_output_2, .003f, dq_zp, q_split_output_2, use_contrib_qdq);
+    builder.AddDequantizeLinearNode<OutputType>(lower_q_output_3, .003f, dq_zp, q_split_output_3, use_contrib_qdq);
   };
 }
 
 template <typename Type1, typename Type2, typename Type3, typename Type4>
 GetQDQTestCaseFn BuildDoubleQDQTestCases(Type1 zp_1, Type2 zp_2, Type3 zp_3, Type4 zp_4,
-                                         float scale_1, float scale_2, float scale_3, float scale_4) {
+                                         float scale_1, float scale_2, float scale_3, float scale_4,
+                                         bool use_contrib_qdq = false) {
   return [=](ModelTestBuilder& builder) {
     auto* input_arg = builder.MakeInput<float>(
         {11, 22, 33, 44},
@@ -416,17 +456,39 @@ GetQDQTestCaseFn BuildDoubleQDQTestCases(Type1 zp_1, Type2 zp_2, Type3 zp_3, Typ
     NodeArg* dq1_output = builder.MakeIntermediate();
     NodeArg* q2_output = builder.MakeIntermediate();
     NodeArg* dq2_output = builder.MakeOutput();
-    builder.AddQuantizeLinearNode<Type1>(input_arg, scale_1, zp_1, q1_output);
-    builder.AddDequantizeLinearNode<Type2>(q1_output, scale_2, zp_2, dq1_output);
-    builder.AddQuantizeLinearNode<Type3>(dq1_output, scale_3, zp_3, q2_output);
-    builder.AddDequantizeLinearNode<Type4>(q2_output, scale_4, zp_4, dq2_output);
+    builder.AddQuantizeLinearNode<Type1>(input_arg, scale_1, zp_1, q1_output, use_contrib_qdq);
+    builder.AddDequantizeLinearNode<Type2>(q1_output, scale_2, zp_2, dq1_output, use_contrib_qdq);
+    builder.AddQuantizeLinearNode<Type3>(dq1_output, scale_3, zp_3, q2_output, use_contrib_qdq);
+    builder.AddDequantizeLinearNode<Type4>(q2_output, scale_4, zp_4, dq2_output, use_contrib_qdq);
   };
 }
 
+/// <summary>
+/// Returns a function that builds a model with a double QDQ sequence (Q1 -> DQ1 -> Q2 -> DQ2*),
+/// where DQ2 can be repeated. Must provide at least 4 zero-point and scale values.
+/// </summary>
+/// <param name="input_shape">Shape of input float data.</param>
+/// <param name="input_data">Input float data.</param>
+/// <param name="zero_points">Ordered list of zero-point values for each node in the sequence.</param>
+/// <param name="zero_point_types">Ordered list of zero-point types for each node in the sequence.</param>
+/// <param name="zero_points">Ordered list of scale values for each node in the sequence.</param>
+/// <param name="graph_output_index">Index of the node that provides a graph output.</param>
+/// <param name="use_contrib_qdq">Set to true to use the 'com.microsoft' domain for Q and DQ ops.</param>
+/// <returns>A function for building the model</returns>
+GetQDQTestCaseFn BuildDoubleQDQTestCaseWithDuplicateLastDQs(
+    gsl::span<const int64_t> input_shape,
+    gsl::span<const float> input_data,
+    gsl::span<const int64_t> zero_points,
+    gsl::span<const ONNX_NAMESPACE::TensorProto_DataType> zero_point_types,
+    gsl::span<const float> scales,
+    size_t graph_output_index,
+    bool use_contrib_qdq = false);
+
 template <typename T>
-GetQDQTestCaseFn BuildDoubleQDQWithoutLastOutput(int output_index) {
+GetQDQTestCaseFn BuildDoubleQDQWithoutLastOutput(int output_index, bool use_contrib_qdq = false) {
   return [=](ModelTestBuilder& builder) {
-    auto* input_arg = builder.MakeInput<float>({2, 3, 4}, std::numeric_limits<float>::min(), std::numeric_limits<float>::max());
+    auto* input_arg = builder.MakeInput<float>({2, 3, 4}, std::numeric_limits<float>::min(),
+                                               std::numeric_limits<float>::max());
     T zp = (std::numeric_limits<T>::max() - std::numeric_limits<T>::min()) / 2;
     float scale = 0.003f;
     std::vector<NodeArg*> outputs(4);
@@ -437,35 +499,59 @@ GetQDQTestCaseFn BuildDoubleQDQWithoutLastOutput(int output_index) {
         outputs[i] = builder.MakeIntermediate();
       }
     }
-    builder.AddQuantizeLinearNode<T>(input_arg, scale, zp, outputs[0]);
-    builder.AddDequantizeLinearNode<T>(outputs[0], scale, zp, outputs[1]);
-    builder.AddQuantizeLinearNode<T>(outputs[1], scale, zp, outputs[2]);
-    builder.AddDequantizeLinearNode<T>(outputs[2], scale, zp, outputs[3]);
+    builder.AddQuantizeLinearNode<T>(input_arg, scale, zp, outputs[0], use_contrib_qdq);
+    builder.AddDequantizeLinearNode<T>(outputs[0], scale, zp, outputs[1], use_contrib_qdq);
+    builder.AddQuantizeLinearNode<T>(outputs[1], scale, zp, outputs[2], use_contrib_qdq);
+    builder.AddDequantizeLinearNode<T>(outputs[2], scale, zp, outputs[3], use_contrib_qdq);
   };
 }
 
 template <typename InputType, typename OutputType>
-GetQDQTestCaseFn BuildQDQSplitTestCase(
-    const std::vector<int64_t>& input_shape,
-    const int64_t& axis) {
-  return [input_shape, axis](ModelTestBuilder& builder) {
-    auto* input_arg = builder.MakeInput<InputType>(input_shape,
-                                                   std::numeric_limits<InputType>::min(),
-                                                   std::numeric_limits<InputType>::max());
+GetQDQTestCaseFn BuildQDQSplitTestCase(const std::vector<int64_t>& input_shape,
+                                       const int64_t& axis,
+                                       bool use_diff_output_scale,
+                                       bool use_contrib_qdq = false) {
+  return [input_shape, axis, use_diff_output_scale, use_contrib_qdq](ModelTestBuilder& builder) {
+    InputType dq_zp{};
+    OutputType q_zp{};
+    NodeArg* input_arg = nullptr;
+
+    if constexpr (std::is_same_v<InputType, Int4x2> || std::is_same_v<InputType, UInt4x2>) {
+      input_arg = builder.MakeInput(input_shape, InputType(InputType::min_val, 0), InputType(InputType::max_val, 0));
+      dq_zp = InputType(static_cast<std::byte>(InputType::max_val / 2));
+      q_zp = OutputType(static_cast<std::byte>(OutputType::max_val / 2));
+    } else {
+      input_arg = builder.MakeInput<InputType>(input_shape, std::numeric_limits<InputType>::min(),
+                                               std::numeric_limits<InputType>::max());
+      dq_zp = std::numeric_limits<InputType>::max() / 2;
+      q_zp = std::numeric_limits<OutputType>::max() / 2;
+    }
 
-    InputType dq_zp = std::numeric_limits<InputType>::max() / 2;
-    OutputType q_zp = std::numeric_limits<OutputType>::max() / 2;
     auto* dq_output = builder.MakeIntermediate();
-    builder.AddDequantizeLinearNode<InputType>(input_arg, .003f, dq_zp, dq_output);
+    constexpr float input_scale = 0.003f;
+    builder.AddDequantizeLinearNode<InputType>(input_arg, input_scale, dq_zp, dq_output, use_contrib_qdq);
 
     // add Split
+    std::vector<NodeArg*> split_inputs;
+    split_inputs.push_back(dq_output);
+
+    // Use the optional 'split' input when testing Split 13
+    int opset = builder.DomainToVersionMap().find(kOnnxDomain)->second;
+    if (opset >= 13 && opset < 18) {
+      int64_t dim = input_shape[axis];
+      int64_t split_size = dim / 3;
+      split_inputs.push_back(builder.Make1DInitializer(std::vector<int64_t>{split_size,
+                                                                            split_size, dim - (2 * split_size)}));
+    }
 
     auto* split_output_1 = builder.MakeIntermediate();
     auto* split_output_2 = builder.MakeIntermediate();
     auto* split_output_3 = builder.MakeIntermediate();
-    Node& split_node = builder.AddNode("Split", {dq_output}, {split_output_1, split_output_2, split_output_3});
+    Node& split_node = builder.AddNode("Split", split_inputs, {split_output_1, split_output_2, split_output_3});
     split_node.AddAttribute("axis", axis);
-    if (builder.DomainToVersionMap().find(kOnnxDomain)->second >= 18) {
+
+    // Use the 'num_outputs' attribute when testing Split >= 18
+    if (opset >= 18) {
       split_node.AddAttribute("num_outputs", static_cast<int64_t>(3));
     }
 
@@ -473,17 +559,22 @@ GetQDQTestCaseFn BuildQDQSplitTestCase(
     auto* q_split_output_1 = builder.MakeOutput();
     auto* q_split_output_2 = builder.MakeOutput();
     auto* q_split_output_3 = builder.MakeOutput();
-    builder.AddQuantizeLinearNode<OutputType>(split_output_1, .003f, q_zp, q_split_output_1);  // Model input (node_token_1)
-    builder.AddQuantizeLinearNode<OutputType>(split_output_2, .003f, q_zp, q_split_output_2);  // Model input (node_token_2)
-    builder.AddQuantizeLinearNode<OutputType>(split_output_3, .003f, q_zp, q_split_output_3);
+    float output_scale = use_diff_output_scale ? input_scale + 0.001f : input_scale;
+    builder.AddQuantizeLinearNode<OutputType>(split_output_1, output_scale, q_zp, q_split_output_1,
+                                              use_contrib_qdq);  // Model input (node_token_1)
+    builder.AddQuantizeLinearNode<OutputType>(split_output_2, output_scale, q_zp, q_split_output_2,
+                                              use_contrib_qdq);  // Model input (node_token_2)
+    builder.AddQuantizeLinearNode<OutputType>(split_output_3, output_scale, q_zp, q_split_output_3,
+                                              use_contrib_qdq);
   };
 }
 template <typename InputType>
 GetQDQTestCaseFn BuildQDQWhereTestCase(
     const std::vector<int64_t>& cond_shape,
     const std::vector<int64_t>& x_shape,
-    const std::vector<int64_t>& y_shape) {
-  return [cond_shape, x_shape, y_shape](ModelTestBuilder& builder) {
+    const std::vector<int64_t>& y_shape,
+    bool use_contrib_qdq = false) {
+  return [cond_shape, x_shape, y_shape, use_contrib_qdq](ModelTestBuilder& builder) {
     auto* input_cond_arg = builder.MakeInputBool(cond_shape);
     auto* input_x_arg = builder.MakeInput<InputType>(x_shape,
                                                      std::numeric_limits<InputType>::min(),
@@ -495,8 +586,8 @@ GetQDQTestCaseFn BuildQDQWhereTestCase(
     constexpr float scale = 0.003f;
     auto* dq_x_output = builder.MakeIntermediate();
     auto* dq_y_output = builder.MakeIntermediate();
-    builder.AddDequantizeLinearNode<InputType>(input_x_arg, scale, zp, dq_x_output);
-    builder.AddDequantizeLinearNode<InputType>(input_y_arg, scale, zp, dq_y_output);
+    builder.AddDequantizeLinearNode<InputType>(input_x_arg, scale, zp, dq_x_output, use_contrib_qdq);
+    builder.AddDequantizeLinearNode<InputType>(input_y_arg, scale, zp, dq_y_output, use_contrib_qdq);
     // add Where
 
     auto* where_output = builder.MakeIntermediate();
@@ -504,15 +595,17 @@ GetQDQTestCaseFn BuildQDQWhereTestCase(
 
     // add Q
     auto* q_where_output = builder.MakeOutput();
-    builder.AddQuantizeLinearNode<InputType>(where_output, scale, zp, q_where_output);  // Model input (node_token_1)
+    builder.AddQuantizeLinearNode<InputType>(where_output, scale, zp, q_where_output,
+                                             use_contrib_qdq);  // Model input (node_token_1)
   };
 }
 
 template <typename InputType, typename OutputType>
 GetQDQTestCaseFn BuildQDQTransposeTestCase(
     const std::vector<int64_t>& input_shape,
-    const std::vector<int64_t>& perms) {
-  return [input_shape, perms](ModelTestBuilder& builder) {
+    const std::vector<int64_t>& perms,
+    bool use_contrib_qdq = false) {
+  return [input_shape, perms, use_contrib_qdq](ModelTestBuilder& builder) {
     auto* input_arg = builder.MakeInput<InputType>(input_shape,
                                                    std::numeric_limits<InputType>::min(),
                                                    std::numeric_limits<InputType>::max());
@@ -521,17 +614,34 @@ GetQDQTestCaseFn BuildQDQTransposeTestCase(
     InputType dq_zp = std::numeric_limits<InputType>::max() / 2;
     OutputType q_zp = std::numeric_limits<OutputType>::max() / 2;
 
-    // add DQ
-    auto* dq_output = builder.MakeIntermediate();
-    builder.AddDequantizeLinearNode<InputType>(input_arg, .003f, dq_zp, dq_output);
+    // In order to test additional EPs that are more sensitive to whether the Transpose is in a QDQ node unit or not,
+    // we need a QDQ node unit prior to DQ -> Transpose -> Q -> graph output.
+    // The transpose optimizer will push the transpose, convert its input to uint8, and drop the empty DQ -> Q.
+    // If there's a QDQ node unit prior, the scale and zp info can be read from the Q node feeding the standalone
+    // Transpose node, so we add a DQ -> Mul -> Q to provide that.
+    // Essentially eveything has worked correctly if the DQ -> Transpose -> Q becomes a single Transpose and the
+    // extra QDQ node unit simply allows some additional functionality to be tested.
+
+    // add DQ -> Mul -> Q
+    auto* dq_output_0 = builder.MakeIntermediate();
+    auto* mul_output = builder.MakeIntermediate();
+    auto* q_output_0 = builder.MakeIntermediate();
+    auto mul_by = builder.MakeInitializer<float>({1}, 2.f, 3.f);
+    builder.AddDequantizeLinearNode<InputType>(input_arg, .003f, dq_zp, dq_output_0, use_contrib_qdq);
+    builder.AddNode("Mul", {dq_output_0, mul_by}, {mul_output});
+    builder.AddQuantizeLinearNode<OutputType>(mul_output, .003f, q_zp, q_output_0, use_contrib_qdq);
+
+    // add DQ -> Transpose -> Q
+    auto* dq_output_1 = builder.MakeIntermediate();
+    builder.AddDequantizeLinearNode<InputType>(q_output_0, .003f, dq_zp, dq_output_1, use_contrib_qdq);
 
     // add Transpose
     auto* transpose_output = builder.MakeIntermediate();
-    Node& transpose_node = builder.AddNode("Transpose", {dq_output}, {transpose_output});
+    Node& transpose_node = builder.AddNode("Transpose", {dq_output_1}, {transpose_output});
     transpose_node.AddAttribute("perm", perms);
 
     // add Q
-    builder.AddQuantizeLinearNode<OutputType>(transpose_output, .003f, q_zp, output_arg);
+    builder.AddQuantizeLinearNode<OutputType>(transpose_output, .003f, q_zp, output_arg, use_contrib_qdq);
   };
 }
 
@@ -567,11 +677,13 @@ GetQDQTestCaseFn BuildQDQConcatTestCase(const std::vector<std::vector<int64_t>>&
                                         int64_t axis,
                                         bool has_input_float = false,
                                         bool has_input_int8 = false,
-                                        bool has_output_int8 = false);
+                                        bool has_output_int8 = false,
+                                        bool use_contrib_qdq = false);
 
 GetQDQTestCaseFn BuildQDQConcatTestCaseUnsupportedInputScaleZp();
 
-GetQDQTestCaseFn BuildQDQMatMulTestCase(const std::vector<int64_t>& input1_shape, const std::vector<int64_t>& input2_shape);
+GetQDQTestCaseFn BuildQDQMatMulTestCase(const std::vector<int64_t>& input1_shape,
+                                        const std::vector<int64_t>& input2_shape);
 
 template <typename Input1Type, typename Input2Type, typename OutputType, typename BiasType = int32_t>
 GetQDQTestCaseFn BuildQDQGemmTestCase(const std::vector<int64_t>& input1_shape,
@@ -612,7 +724,8 @@ GetQDQTestCaseFn BuildQDQGemmTestCase(const std::vector<int64_t>& input1_shape,
 
     if (has_bias) {
       auto* dq_bias_output = builder.MakeIntermediate();
-      auto* bias = builder.MakeInitializer<BiasType>({input2_shape[0]}, static_cast<BiasType>(0), static_cast<BiasType>(127));
+      auto* bias = builder.MakeInitializer<BiasType>({input2_shape[0]}, static_cast<BiasType>(0),
+                                                     static_cast<BiasType>(127));
       builder.AddDequantizeLinearNode<BiasType>(bias, 0.00156f,
                                                 0,
                                                 dq_bias_output);
@@ -639,7 +752,7 @@ GetQDQTestCaseFn BuildQDQGemmTestCase(const std::vector<int64_t>& input1_shape,
   };
 }
 
-std::vector<std::string> GetNodeOpTypesInTopologicalOrder(const Graph& graph);
+std::vector<std::string> GetNodeOpTypesInTopologicalOrder(const Graph& graph, bool include_domain = false);
 
 }  // namespace test
 }  // namespace onnxruntime
