@@ -317,6 +317,11 @@ void BaseTester::ExecuteModel(Model& model, SessionType& session,
     ASSERT_EQ(expect_result, ExpectResult::kExpectFailure) << "Initialize failed but expected success: "
                                                            << status.ErrorMessage();
 
+    // No need to check expected failure string if empty string is given.
+    if (expected_failure_string.empty()) {
+      return;
+    }
+
     // Disable expected_failure_string checks for OpenVINO EP
     if (provider_type != kOpenVINOExecutionProvider) {
       EXPECT_THAT(status.ErrorMessage(), testing::HasSubstr(expected_failure_string));
@@ -337,6 +342,11 @@ void BaseTester::ExecuteModel(Model& model, SessionType& session,
       ASSERT_EQ(expect_result, ExpectResult::kExpectFailure) << "Run failed but expected success: "
                                                              << status.ErrorMessage();
 
+      // No need to check expected failure string if empty string is given.
+      if (expected_failure_string.empty()) {
+        return;
+      }
+
       // Disable expected_failure_string checks for MKL-DNN and OpenVINO EP's
       if (provider_type != kDnnlExecutionProvider &&
           provider_type != kOpenVINOExecutionProvider) {
@@ -420,6 +430,7 @@ bool SetEpsForAllNodes(Graph& graph,
       continue;
 
     bool found = false;
+    const auto& logger = DefaultLoggingManager().DefaultLogger();
 
     for (const auto& ep : execution_providers) {
       auto provider_type = ep->Type();
@@ -438,7 +449,8 @@ bool SetEpsForAllNodes(Graph& graph,
       }
 
       // Check the EP has an impl for the node from builtin registry.
-      if (KernelRegistry::HasImplementationOf(*ep->GetKernelRegistry(), node, ep->Type(), kernel_type_str_resolver)) {
+      if (KernelRegistry::HasImplementationOf(*ep->GetKernelRegistry(), node, ep->Type(), kernel_type_str_resolver,
+                                              logger)) {
         found = true;
         break;
       }
@@ -451,6 +463,7 @@ bool SetEpsForAllNodes(Graph& graph,
                                                              std::string_view(kMSInternalNHWCDomain),
                                                              node.SinceVersion(),
                                                              type_constraint_map,
+                                                             logger,
                                                              &kci);
         if (status.IsOK() && kci != nullptr) {
           found = true;
@@ -463,7 +476,7 @@ bool SetEpsForAllNodes(Graph& graph,
           std::any_of(custom_registries->cbegin(), custom_registries->cend(),
                       [&](auto reg) {
                         return KernelRegistry::HasImplementationOf(*reg->GetKernelRegistry(), node, ep->Type(),
-                                                                   kernel_type_str_resolver);
+                                                                   kernel_type_str_resolver, logger);
                       })) {
         found = true;
         break;
