@@ -4,7 +4,7 @@
 #include "core/common/logging/logging.h"
 #include "core/graph/graph.h"
 #include "core/graph/graph_viewer.h"
-#include "core/providers/coreml/coreml_execution_provider.h"
+#include "core/providers/coreml/coreml_provider_factory_creator.h"
 #include "core/providers/coreml/coreml_provider_factory.h"
 #include "core/session/inference_session.h"
 #include "test/common/tensor_op_test_utils.h"
@@ -15,6 +15,7 @@
 #include "test/util/include/inference_session_wrapper.h"
 #include "test/util/include/test_environment.h"
 #include "test/util/include/test_utils.h"
+#include "core/graph/onnx_protobuf.h"
 
 #if !defined(ORT_MINIMAL_BUILD)
 // if this is a full build we need the provider test utils
@@ -30,11 +31,12 @@ using namespace ::onnxruntime::logging;
 namespace onnxruntime {
 namespace test {
 
-// We want to run UT on CPU only to get output value without losing precision to pass the verification
-static constexpr uint32_t s_coreml_flags = COREML_FLAG_USE_CPU_ONLY;
-
-static std::unique_ptr<IExecutionProvider> MakeCoreMLExecutionProvider(uint32_t flags = s_coreml_flags) {
-  return std::make_unique<CoreMLExecutionProvider>(flags);
+static std::unique_ptr<IExecutionProvider> MakeCoreMLExecutionProvider(
+    std::string ModelFormat = "NeuralNetwork", std::string ComputeUnits = "CPUOnly", std::string ModelCacheDirectory = "") {
+  std::unordered_map<std::string, std::string> provider_options = {{kCoremlProviderOption_MLComputeUnits, ComputeUnits},
+                                                                   {kCoremlProviderOption_ModelFormat, ModelFormat},
+                                                                   {kCoremlProviderOption_ModelCacheDirectory, ModelCacheDirectory}};
+  return CoreMLProviderFactoryCreator::Create(provider_options)->CreateProvider();
 }
 
 #if !defined(ORT_MINIMAL_BUILD)
@@ -127,6 +129,10 @@ TEST(CoreMLExecutionProviderTest, ArgMaxCastTest) {
                             MakeCoreMLExecutionProvider(),
                             feeds,
                             verification_params);
+  RunAndVerifyOutputsWithEP(model_file_name, CurrentTestName(),
+                            MakeCoreMLExecutionProvider("MLProgram"),
+                            feeds,
+                            verification_params);
 #else
   TestModelLoad(model_file_name, MakeCoreMLExecutionProvider(), ExpectedEPNodeAssignment::All);
 #endif
@@ -164,6 +170,11 @@ TEST(CoreMLExecutionProviderTest, ArgMaxUnsupportedCastTest) {
                             MakeCoreMLExecutionProvider(),
                             feeds,
                             verification_params);
+
+  RunAndVerifyOutputsWithEP(model_file_name, CurrentTestName(),
+                            MakeCoreMLExecutionProvider("MLProgram"),
+                            feeds,
+                            verification_params);
 #else
   TestModelLoad(model_file_name, MakeCoreMLExecutionProvider(), ExpectedEPNodeAssignment::Some);
 #endif
@@ -259,5 +270,112 @@ TEST(CoreMLExecutionProviderTest, TestNameSanitization) {
 }
 #endif
 
+TEST(CoreMLExecutionProviderTest, TestModelCache) {
+  const ORTCHAR_T* model_file_name = ORT_TSTR("testdata/coreml_argmax_cast_test.onnx");
+
+  onnx::ModelProto model;
+  {
+    std::ifstream in(model_file_name, std::ios_base::binary);
+    model.ParseFromIstream(&in);
+    in.close();
+  }
+
+  std::string out_string;
+#if defined(__APPLE__)
+  std::vector<int64_t> dims_mul_x = {3, 2, 2};
+  std::vector<float> values_mul_x = {1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f, 10.0f, 11.0f, 12.0f};
+  OrtValue ml_value_x;
+  AllocatorPtr allocator = std::make_shared<CPUAllocator>();
+  CreateMLValue<float>(allocator, dims_mul_x, values_mul_x, &ml_value_x);
+
+  NameMLValMap feeds;
+  feeds.insert(std::make_pair("X", ml_value_x));
+  std::string subgraph_name;
+  const std::function<void(const Graph&)> graph_verifier = [&subgraph_name](const Graph& graph) {
+    GraphViewer graph_viewer{graph};
+    const auto& node_indices_in_order = graph_viewer.GetNodesInTopologicalOrder();
+    const auto* node = graph.GetNode(node_indices_in_order[0]);
+    auto _first = node->Name().find('_') + 1;
+    auto _second = node->Name().find('_', _first);
+    subgraph_name = node->Name().substr(_first, _second - _first);
+  };
+  EPVerificationParams verification_params{.graph_verifier = &graph_verifier};
+
+  auto* metadata_props = model.add_metadata_props();
+  metadata_props->set_key(kCOREML_CACHE_KEY);
+  {  // test with valid model cache directory
+    metadata_props->set_value("legalhash123");
+    model.SerializeToString(&out_string);
+    gsl::span<const std::byte> model_data{reinterpret_cast<const std::byte*>(out_string.data()), out_string.size()};
+    RunAndVerifyOutputsWithEP(model_data, CurrentTestName(),
+                              MakeCoreMLExecutionProvider("MLProgram", "CPUOnly", ORT_TSTR("./tmp/")),
+                              feeds,
+                              verification_params);
+    ASSERT_EQ(std::filesystem::exists("./tmp/legalhash123"), true);
+  }
+  {
+    // test with invalid model cache directory, only alphanumeric characters are allowed
+    out_string.clear();
+    metadata_props->set_key(kCOREML_CACHE_KEY);
+    metadata_props->set_value("illegalhash__123");
+    model.SerializeToString(&out_string);
+    gsl::span<const std::byte> model_data{reinterpret_cast<const std::byte*>(out_string.data()), out_string.size()};
+    RunAndVerifyOutputsWithEP(model_data, CurrentTestName(),
+                              MakeCoreMLExecutionProvider("MLProgram", "CPUOnly", ORT_TSTR("./tmp")),
+                              feeds,
+                              verification_params);
+    ASSERT_EQ(std::filesystem::exists("./tmp/illegalhash__123"), false);
+    // the cache folder name should be the first part of the subgraph name
+    ASSERT_EQ(std::filesystem::exists("./tmp/" + subgraph_name), true);
+  }
+  {
+    // test with invalid model cache directory,  more than 64 characters
+    out_string.clear();
+    metadata_props->set_key(kCOREML_CACHE_KEY);
+    metadata_props->set_value("modelhashwithmorethan64charactersmodelhashwithmorethan64charactersmodelhashwithmorethan64characters");
+    model.SerializeToString(&out_string);
+    gsl::span<const std::byte> model_data{reinterpret_cast<const std::byte*>(out_string.data()), out_string.size()};
+    RunAndVerifyOutputsWithEP(model_data, CurrentTestName(),
+                              MakeCoreMLExecutionProvider("MLProgram", "CPUOnly", ORT_TSTR("./tmp")),
+                              feeds,
+                              verification_params);
+    ASSERT_EQ(std::filesystem::exists("./tmp/modelhashwithmorethan64charactersmodelhashwithmorethan64charactersmodelhashwithmorethan64characters"), false);
+    // the cache folder name should be the first part of the subgraph name
+    ASSERT_EQ(std::filesystem::exists("./tmp/" + subgraph_name), true);
+  }
+  {
+    // test with invalid model cache directory,  empty
+    out_string.clear();
+    metadata_props->set_key(kCOREML_CACHE_KEY);
+    metadata_props->set_value("");
+    model.SerializeToString(&out_string);
+    gsl::span<const std::byte> model_data{reinterpret_cast<const std::byte*>(out_string.data()), out_string.size()};
+    RunAndVerifyOutputsWithEP(model_data, CurrentTestName(),
+                              MakeCoreMLExecutionProvider("MLProgram", "CPUOnly", ORT_TSTR("./tmp")),
+                              feeds,
+                              verification_params);
+    // the cache folder name should be the first part of the subgraph name
+    ASSERT_EQ(std::filesystem::exists("./tmp/" + subgraph_name), true);
+  }
+  {
+    // test with invalid model cache directory, caching shall be disabled
+    out_string.clear();
+    metadata_props->set_key(kCOREML_CACHE_KEY);
+    metadata_props->set_value("");
+    model.SerializeToString(&out_string);
+    gsl::span<const std::byte> model_data{reinterpret_cast<const std::byte*>(out_string.data()), out_string.size()};
+    RunAndVerifyOutputsWithEP(model_data, CurrentTestName(),
+                              MakeCoreMLExecutionProvider("MLProgram", "CPUOnly", ORT_TSTR("/")),
+                              feeds,
+                              verification_params);
+    // this folder can't be created
+    ASSERT_EQ(std::filesystem::exists("/" + subgraph_name), false);
+  }
+#else
+  model.SerializeToString(&out_string);
+  gsl::span<const std::byte> model_data{reinterpret_cast<const std::byte*>(out_string.data()), out_string.size()};
+  TestModelLoad(model_data, MakeCoreMLExecutionProvider(), ExpectedEPNodeAssignment::All);
+#endif
+}
 }  // namespace test
 }  // namespace onnxruntime
