@@ -120,19 +120,62 @@ TEST(MathOpTest, Clip_Default_uint64) {
 }
 
 TEST(MathOpTest, Clip_MLFloat16) {
+  auto run_test = [](bool min_max_are_initializer) {
+    OpTester test("Clip", 12);
+
+    std::vector<int64_t> dims{3, 3};
+    test.AddInput<MLFloat16>("X", dims,
+                             {MLFloat16(-1.0f), MLFloat16(-2.0f), MLFloat16(-3.0f),
+                              MLFloat16(-4.0f), MLFloat16(0.0f), MLFloat16(2.0f),
+                              MLFloat16(4.0f), MLFloat16(6.0f), MLFloat16(8.0f)});
+    test.AddInput<MLFloat16>("min", {}, {MLFloat16(0.0f)}, min_max_are_initializer);
+    test.AddInput<MLFloat16>("max", {}, {MLFloat16(6.0f)}, min_max_are_initializer);
+    test.AddOutput<MLFloat16>("Y", dims,
+                              {MLFloat16(0.0f), MLFloat16(0.0f), MLFloat16(0.0f),
+                               MLFloat16(0.0f), MLFloat16(0.0f), MLFloat16(2.0f),
+                               MLFloat16(4.0f), MLFloat16(6.0f), MLFloat16(6.0f)});
+
+    test.Run();
+  };
+  run_test(true);  // coreml requires constant max/min
+  run_test(false);
+}
+
+TEST(MathOpTest, Clip_MLFloat16_NoMin_NoMax) {
   OpTester test("Clip", 12);
 
-  std::vector<int64_t> dims{3, 3};
+  std::vector<int64_t> dims{3};
   test.AddInput<MLFloat16>("X", dims,
-                           {MLFloat16(-1.0f), MLFloat16(-2.0f), MLFloat16(-3.0f),
-                            MLFloat16(-4.0f), MLFloat16(0.0f), MLFloat16(2.0f),
-                            MLFloat16(4.0f), MLFloat16(6.0f), MLFloat16(8.0f)});
-  test.AddInput<MLFloat16>("min", {}, {MLFloat16(0.0f)});
-  test.AddInput<MLFloat16>("max", {}, {MLFloat16(6.0f)});
+                           {MLFloat16(-1.0f), MLFloat16(-2.0f), MLFloat16(3.0f)});
   test.AddOutput<MLFloat16>("Y", dims,
-                            {MLFloat16(0.0f), MLFloat16(0.0f), MLFloat16(0.0f),
-                             MLFloat16(0.0f), MLFloat16(0.0f), MLFloat16(2.0f),
-                             MLFloat16(4.0f), MLFloat16(6.0f), MLFloat16(6.0f)});
+                            {MLFloat16(-1.0f), MLFloat16(-2.0f), MLFloat16(3.0f)});
+
+  test.Run();
+}
+
+TEST(MathOpTest, Clip_MLFloat16_NoMax) {
+  OpTester test("Clip", 12);
+
+  std::vector<int64_t> dims{3};
+  test.AddInput<MLFloat16>("X", dims,
+                           {MLFloat16(-1.0f), MLFloat16(-2.0f), MLFloat16(3.0f)});
+  test.AddInput<MLFloat16>("min", {}, {MLFloat16(0.0f)});
+  test.AddOutput<MLFloat16>("Y", dims,
+                            {MLFloat16(0.0f), MLFloat16(0.0f), MLFloat16(3.0f)});
+
+  test.Run();
+}
+
+TEST(MathOpTest, Clip_MLFloat16_NoMin) {
+  OpTester test("Clip", 12);
+
+  std::vector<int64_t> dims{3};
+  test.AddInput<MLFloat16>("X", dims,
+                           {MLFloat16(-1.0f), MLFloat16(-2.0f), MLFloat16(3.0f)});
+  test.AddOptionalInputEdge<MLFloat16>();  // no min
+  test.AddInput<MLFloat16>("max", {}, {MLFloat16(0.0f)});
+  test.AddOutput<MLFloat16>("Y", dims,
+                            {MLFloat16(-1.0f), MLFloat16(-2.0f), MLFloat16(0.0f)});
 
   test.Run();
 }
