@@ -63,13 +63,21 @@ TEST(NonMaxSuppressionOpTest, TwoClasses) {
   test.AddInput<int64_t>("max_output_boxes_per_class", {}, {6L});
   test.AddInput<float>("iou_threshold", {}, {0.5f});
   test.AddInput<float>("score_threshold", {}, {0.0f});
+  // The selected_indices in ORT is sorted by class, whereas in TRT the selected_indices is sorted by score,
+// So it needs to sort the output to pass the output check when there is more than one class using TRT EP.
+#ifdef USE_TENSORRT
+  bool sort_output = true;
+#else
+  bool sort_output = false;  // default
+#endif
   test.AddOutput<int64_t>("selected_indices", {6, 3},
                           {0L, 0L, 3L,
                            0L, 0L, 0L,
                            0L, 0L, 5L,
                            0L, 1L, 3L,
                            0L, 1L, 0L,
-                           0L, 1L, 5L});
+                           0L, 1L, 5L},
+                          sort_output);
   test.Run();
 }
 
@@ -125,6 +133,13 @@ TEST(NonMaxSuppressionOpTest, TwoBatches_TwoClasses) {
                         0.1f, 0.2f, 0.6f, 0.3f, 0.9f});
   test.AddInput<int64_t>("max_output_boxes_per_class", {}, {2L});
   test.AddInput<float>("iou_threshold", {}, {0.8f});
+  // The selected_indices in ORT is sorted by class, whereas in TRT the selected_indices is sorted by score,
+// So it needs to sort the output to pass the output check when there is more than one class using TRT EP.
+#ifdef USE_TENSORRT
+  bool sort_output = true;
+#else
+  bool sort_output = false;  // default
+#endif
   test.AddOutput<int64_t>("selected_indices", {8, 3},
                           {0L, 0L, 4L,
                            0L, 0L, 2L,
@@ -134,7 +149,8 @@ TEST(NonMaxSuppressionOpTest, TwoBatches_TwoClasses) {
                            1L, 0L, 4L,
                            1L, 0L, 1L,
                            1L, 1L, 4L,
-                           1L, 1L, 1L});
+                           1L, 1L, 1L},
+                          sort_output);
   test.Run();
 }
 
@@ -302,7 +318,11 @@ TEST(NonMaxSuppressionOpTest, InconsistentBoxAndScoreShapes) {
   test.AddInput<float>("iou_threshold", {}, {0.5f});
   test.AddInput<float>("score_threshold", {}, {0.0f});
   test.AddOutput<int64_t>("selected_indices", {0, 3}, {});
+#ifdef USE_TENSORRT
+  test.Run(OpTester::ExpectResult::kExpectFailure, "");  // TensorRT EP will output different failure message, providing empty string simply skips checking the error message.
+#else
   test.Run(OpTester::ExpectResult::kExpectFailure, "boxes and scores should have same spatial_dimension.");
+#endif
 }
 
 TEST(NonMaxSuppressionOpTest, InvalidIOUThreshold) {
@@ -313,7 +333,8 @@ TEST(NonMaxSuppressionOpTest, InvalidIOUThreshold) {
   test.AddInput<float>("iou_threshold", {}, {1.2f});
   test.AddInput<float>("score_threshold", {}, {0.0f});
   test.AddOutput<int64_t>("selected_indices", {0, 3}, {});
-  test.Run(OpTester::ExpectResult::kExpectFailure, "iou_threshold must be in range [0, 1]");
+  // TRT is missing a runtime check validating IouThreshold value. Once the bug is fixed, we will add back this unit test for TRT.
+  test.Run(OpTester::ExpectResult::kExpectFailure, "iou_threshold must be in range [0, 1]", {kTensorrtExecutionProvider});
 }
 
 TEST(NonMaxSuppressionOpTest, EmptyInput) {
