@@ -2,13 +2,7 @@
 # Licensed under the MIT License.
 
 
-from typing import Any, Tuple  # noqa: F401
-
-import numpy as np  # type: ignore
-
-# import onnx
-# from ..base import Base
-# from . import expect
+import numpy as np
 
 DebugOutput = True
 np.set_printoptions(suppress=True)  # , precision=16, floatmode='maxprec')
