@@ -358,11 +358,11 @@ class TestCalibrateMinMaxCalibrator(unittest.TestCase):
             rmin = np.minimum(rmin, np.amin(output, axis=1))
             rmax = np.maximum(rmax, np.amax(output, axis=1))
 
-        min_max_pairs = list(zip(rmin, rmax))
+        min_max_pairs = list(zip(rmin, rmax, strict=False))
         output_names = [infer_session.get_outputs()[i].name for i in range(len(infer_session.get_outputs()))]
-        output_min_max_dict = dict(zip(output_names, min_max_pairs))
-        for output_name in output_min_max_dict:
-            self.assertEqual(output_min_max_dict[output_name], tensors_range[output_name].range_value)
+        output_min_max_dict = dict(zip(output_names, min_max_pairs, strict=False))
+        for output_name, min_max in output_min_max_dict.items():
+            self.assertEqual(min_max, tensors_range[output_name].range_value)
 
     def test_histogram_calibrators_run(self):
         """
@@ -521,11 +521,11 @@ class TestCalibrateMinMaxCalibrator(unittest.TestCase):
             rmin = np.minimum(rmin, np.amin(output, axis=-1))
             rmax = np.maximum(rmax, np.amax(output, axis=-1))
 
-        min_max_pairs = list(zip(rmin, rmax))
+        min_max_pairs = list(zip(rmin, rmax, strict=False))
         output_names = [infer_session.get_outputs()[i].name for i in range(len(infer_session.get_outputs()))]
-        output_min_max_dict = dict(zip(output_names, min_max_pairs))
-        for output_name in output_min_max_dict:
-            np.testing.assert_equal(output_min_max_dict[output_name], tensors_range[output_name].range_value)
+        output_min_max_dict = dict(zip(output_names, min_max_pairs, strict=False))
+        for output_name, min_max in output_min_max_dict.items():
+            np.testing.assert_equal(min_max, tensors_range[output_name].range_value)
 
 
 if __name__ == "__main__":
