@@ -10,9 +10,13 @@ import unittest
 import numpy as np
 import onnx
 from onnx import TensorProto, helper, numpy_helper
-from op_test_utils import TestDataFeeds  # noqa: F401
-from op_test_utils import check_op_type_order  # noqa: F401
-from op_test_utils import check_model_correctness, check_op_type_count, check_qtype_by_node_type
+from op_test_utils import (
+    TestDataFeeds,  # noqa: F401
+    check_model_correctness,
+    check_op_type_count,
+    check_op_type_order,  # noqa: F401
+    check_qtype_by_node_type,
+)
 
 from onnxruntime.quantization import DynamicQuantConfig, QuantType, quantize, quantize_dynamic
 
