@@ -550,7 +550,17 @@ class TestOpGemm(unittest.TestCase):
         C = np.array([[0, 0], [0, 0]], dtype=np.int32)
         scale = np.array([1], dtype=np.float32)
         zp = np.array([0], dtype=np.int8)
-        feeds = dict(A=A, scaleA=scaleA, zpA=zpA, B=B, scaleB=scaleB, zpB=zpB, C=C, scale=scale, zp=zp)
+        feeds = {
+            "A": A,
+            "scaleA": scaleA,
+            "zpA": zpA,
+            "B": B,
+            "scaleB": scaleB,
+            "zpB": zpB,
+            "C": C,
+            "scale": scale,
+            "zp": zp,
+        }
         expected = sess.run(None, feeds)[0]
         got = ref.run(None, feeds)[0]
         assert_allclose(expected, got)
@@ -628,7 +638,7 @@ class TestOpGemm(unittest.TestCase):
         A = np.array([[2, 1], [1, 0]], dtype=np.float32)
         scaleA = np.array([1], dtype=np.float32)
         zpA = np.array([0], dtype=np.uint8)
-        feeds = dict(A=A, scaleA=scaleA, zpA=zpA)
+        feeds = {"A": A, "scaleA": scaleA, "zpA": zpA}
         expected = sess.run(None, feeds)[0]
         got = ref.run(None, feeds)[0]
         assert_allclose(expected, got)
@@ -694,7 +704,7 @@ class TestOpGemm(unittest.TestCase):
         A = np.array([[2, 1], [1, 0]], dtype=np.float32)
         scaleA = np.array([1], dtype=np.float32)
         zpA = np.array([0], dtype=np.int8)
-        feeds = dict(A=A, scaleA=scaleA, zpA=zpA)
+        feeds = {"A": A, "scaleA": scaleA, "zpA": zpA}
         expected = sess.run(None, feeds)[0]
         got = ref.run(None, feeds)[0]
         assert_allclose(expected, got)
@@ -828,7 +838,7 @@ class TestOpGemm(unittest.TestCase):
         qpath = "test_dynamic_quantization.quantized.onnx"
         quantize_dynamic(model_input=model_path, model_output=qpath, use_external_data_format=True, **run_config)
         onx = onnx.load(qpath)
-        self.assertIn("DynamicQuantizeLinear", set(n.op_type for n in onx.graph.node))
+        self.assertIn("DynamicQuantizeLinear", {n.op_type for n in onx.graph.node})
 
 
 if __name__ == "__main__":
