@@ -9,7 +9,6 @@ import tempfile
 import unittest
 from importlib.util import find_spec
 from pathlib import Path
-from typing import Dict, Tuple, Union
 
 import numpy as np
 import onnx
@@ -28,7 +27,7 @@ class TestOpMatMul4Bits(unittest.TestCase):
     def tearDownClass(cls):
         cls._tmp_model_dir.cleanup()
 
-    def fill_int4_data(self, shape: Union[int, Tuple[int, ...]], symmetric: bool) -> np.ndarray:
+    def fill_int4_data(self, shape: int | tuple[int, ...], symmetric: bool) -> np.ndarray:
         line = np.zeros(shape)
         line = line.reshape(-1)
 
@@ -51,12 +50,19 @@ class TestOpMatMul4Bits(unittest.TestCase):
 
         return line.reshape(shape)
 
-    def input_feeds(self, n: int, name2shape: Dict[str, Union[int, Tuple[int, ...]]]) -> TestDataFeeds:
+    def input_feeds(
+        self,
+        n: int,
+        name2shape: dict[str, int | tuple[int, ...]],
+        low: int = -1,
+        high: int = 2,
+        dtype: type = np.float32,
+    ) -> TestDataFeeds:
         input_data_list = []
         for _i in range(n):
             inputs = {}
             for name, shape in name2shape.items():
-                inputs.update({name: np.random.randint(-1, 2, shape).astype(np.float32)})
+                inputs.update({name: np.random.randint(low, high, shape).astype(dtype)})
             input_data_list.extend([inputs])
         dr = TestDataFeeds(input_data_list)
         return dr
@@ -72,7 +78,7 @@ class TestOpMatMul4Bits(unittest.TestCase):
         initializers = []
 
         def make_matmul(
-            input_name, weight_shape: Union[int, Tuple[int, ...]], weight_name: str, output_name: str, node_name: str
+            input_name, weight_shape: int | tuple[int, ...], weight_name: str, output_name: str, node_name: str
         ):
             weight_data = self.fill_int4_data(weight_shape, symmetric).astype(np.float32)
             initializers.append(onnx.numpy_helper.from_array(weight_data, name=weight_name))
@@ -111,6 +117,65 @@ class TestOpMatMul4Bits(unittest.TestCase):
 
         onnx.save(model, output_model_path)
 
+    def construct_model_gather(
+        self,
+        output_model_path: str,
+        symmetric: bool,
+        tdata: TensorProto.DataType,
+        tind: TensorProto.DataType,
+        vocab_size: int = 545,
+        embedding_len: int = 228,
+    ) -> None:
+        #      (input)
+        #         |
+        #       Gather
+        #         |
+        #      (output)
+        indices_name = "input"
+        output_name = "output"
+        initializers = []
+
+        def make_gather(
+            indices_name, data_shape: int | tuple[int, ...], data_name: str, output_name: str, node_name: str
+        ):
+            weight_data = self.fill_int4_data(data_shape, symmetric).astype(
+                np.float32 if tdata == TensorProto.FLOAT else np.float16
+            )
+            initializers.append(onnx.numpy_helper.from_array(weight_data, name=data_name))
+            kwargs = {"axis": 0}
+            return onnx.helper.make_node(
+                "Gather",
+                [data_name, indices_name],
+                [output_name],
+                node_name,
+                **kwargs,
+            )
+
+        gather_node = make_gather(
+            indices_name,
+            (vocab_size, embedding_len),
+            "linear1.weight",
+            output_name,
+            "Gather_0",
+        )
+
+        # make graph
+        input_tensor = helper.make_tensor_value_info(indices_name, tind, [-1, 1000])
+        output_tensor = helper.make_tensor_value_info(output_name, tdata, [-1, 1000, embedding_len])
+        graph_name = "gather_4bits_test"
+        graph = helper.make_graph(
+            [gather_node],
+            graph_name,
+            [input_tensor],
+            [output_tensor],
+            initializer=initializers,
+        )
+        # QDQ and gather requires op set >= 21. The tool should automatically update the opset.
+        model = helper.make_model(graph, opset_imports=[helper.make_opsetid("", 19)])
+        model.ir_version = 9  # use stable onnx ir version
+
+        onnx.save(model, output_model_path)
+
     def quant_test(
         self,
         model_fp32_path: str,
@@ -118,9 +183,13 @@ class TestOpMatMul4Bits(unittest.TestCase):
         block_size: int,
         is_symmetric: bool,
         quant_format: quant_utils.QuantFormat = quant_utils.QuantFormat.QOperator,
+        op_types_to_quantize: tuple[str, ...] = ("MatMul",),
+        quant_axes: tuple[tuple[str, int], ...] = (("MatMul", 0), ("Gather", 1)),
+        rtol: float = 0.01,
+        atol: float = 0.05,
     ):
         use_qdq = quant_format == quant_utils.QuantFormat.QDQ
-        name_prefix = "DQ_MatMul" if use_qdq else "MatMulNBits"
+        name_prefix = "QDQ" if use_qdq else "QOperator"
         model_int4_path = str(
             Path(self._tmp_model_dir.name).joinpath(f"{name_prefix}_{block_size}_{is_symmetric}.onnx").absolute()
         )
@@ -130,13 +199,20 @@ class TestOpMatMul4Bits(unittest.TestCase):
 
         model = quant_utils.load_model_with_shape_infer(Path(model_fp32_path))
         quant_config = matmul_4bits_quantizer.DefaultWeightOnlyQuantConfig(
-            block_size=block_size, is_symmetric=is_symmetric, quant_format=quant_format
+            block_size=block_size,
+            is_symmetric=is_symmetric,
+            quant_format=quant_format,
+            op_types_to_quantize=op_types_to_quantize,
+            quant_axes=quant_axes,
         )
         quant = matmul_4bits_quantizer.MatMul4BitsQuantizer(model, algo_config=quant_config)
         quant.process()
         quant.model.save_model_to_file(model_int4_path, False)
 
-        quant_nodes = {"DequantizeLinear": 1, "MatMul": 1} if use_qdq else {"MatMulNBits": 1}
+        if "Gather" in op_types_to_quantize:
+            quant_nodes = {"GatherBlockQuantized": 1}
+        else:
+            quant_nodes = {"DequantizeLinear": 1, "MatMul": 1} if use_qdq else {"MatMulNBits": 1}
         check_op_type_count(self, model_int4_path, **quant_nodes)
 
         if use_qdq:
@@ -163,7 +239,7 @@ class TestOpMatMul4Bits(unittest.TestCase):
         data_reader.rewind()
 
         try:
-            check_model_correctness(self, model_fp32_path, model_int4_path, data_reader.get_next())
+            check_model_correctness(self, model_fp32_path, model_int4_path, data_reader.get_next(), rtol, atol)
         except Exception as exception:
             if "4b quantization not yet supported on this hardware platform!" in exception.args[0]:
                 # Currently we don't have int4 quantization support on all platforms, has to tolerate this exception
@@ -224,7 +300,7 @@ class TestOpMatMul4Bits(unittest.TestCase):
 
         model_fp32_path = str(Path(self._tmp_model_dir.name).joinpath("matmul_fp32_symmetric.onnx").absolute())
         self.construct_model_matmul(model_fp32_path, symmetric=True)
-        data_reader = self.input_feeds(1, {"input": [100, 52]})
+        data_reader = self.input_feeds(1, {"input": (100, 52)})
         self.quant_test(model_fp32_path, data_reader, 32, True)
 
     @unittest.skipIf(
@@ -233,9 +309,31 @@ class TestOpMatMul4Bits(unittest.TestCase):
     def test_quantize_matmul_int4_offsets(self):
         model_fp32_path = str(Path(self._tmp_model_dir.name).joinpath("matmul_fp32_offset.onnx").absolute())
         self.construct_model_matmul(model_fp32_path, symmetric=False)
-        data_reader = self.input_feeds(1, {"input": [100, 52]})
+        data_reader = self.input_feeds(1, {"input": (100, 52)})
         self.quant_test(model_fp32_path, data_reader, 32, False)
 
+    @unittest.skipIf(
+        find_spec("onnxruntime.training"), "Skip because training package doesn't has quantize_matmul_4bits"
+    )
+    def test_quantize_gather_int4_symmetric(self):
+        np.random.seed(13)
+
+        model_fp32_path = str(Path(self._tmp_model_dir.name).joinpath("gather_fp32_symmetric.onnx").absolute())
+        self.construct_model_gather(model_fp32_path, True, TensorProto.FLOAT, TensorProto.INT32)
+        data_reader = self.input_feeds(1, {"input": (100, 1000)}, -545, 535, np.int32)
+        # cover rounding error
+        self.quant_test(model_fp32_path, data_reader, 32, True, op_types_to_quantize=("Gather",), rtol=0.2, atol=0.5)
+
+    @unittest.skipIf(
+        find_spec("onnxruntime.training"), "Skip because training package doesn't has quantize_matmul_4bits"
+    )
+    def test_quantize_gather_int4_offsets(self):
+        model_fp32_path = str(Path(self._tmp_model_dir.name).joinpath("gather_fp32_offset.onnx").absolute())
+        self.construct_model_gather(model_fp32_path, False, TensorProto.FLOAT16, TensorProto.INT64)
+        data_reader = self.input_feeds(1, {"input": (100, 1000)}, -545, 535, np.int64)
+        # cover rounding error
+        self.quant_test(model_fp32_path, data_reader, 32, False, op_types_to_quantize=("Gather",), rtol=0.2, atol=0.5)
+
     @unittest.skipIf(
         find_spec("onnxruntime.training"), "Skip because training package doesn't has quantize_matmul_4bits"
     )
@@ -244,7 +342,7 @@ class TestOpMatMul4Bits(unittest.TestCase):
 
         model_fp32_path = str(Path(self._tmp_model_dir.name).joinpath("matmul_fp32_symmetric.onnx").absolute())
         self.construct_model_matmul(model_fp32_path, symmetric=True)
-        data_reader = self.input_feeds(1, {"input": [100, 52]})
+        data_reader = self.input_feeds(1, {"input": (100, 52)})
         self.quant_test(model_fp32_path, data_reader, 32, True, quant_utils.QuantFormat.QDQ)
 
     @unittest.skipIf(
@@ -253,7 +351,7 @@ class TestOpMatMul4Bits(unittest.TestCase):
     def test_quantize_matmul_int4_offsets_qdq(self):
         model_fp32_path = str(Path(self._tmp_model_dir.name).joinpath("matmul_fp32_offset.onnx").absolute())
         self.construct_model_matmul(model_fp32_path, symmetric=False)
-        data_reader = self.input_feeds(1, {"input": [100, 52]})
+        data_reader = self.input_feeds(1, {"input": (100, 52)})
         self.quant_test(model_fp32_path, data_reader, 32, False, quant_utils.QuantFormat.QDQ)
 
     @unittest.skipIf(
@@ -264,7 +362,7 @@ class TestOpMatMul4Bits(unittest.TestCase):
             self.skipTest("skip test_smooth_quant since neural_compressor is not installed")
         model_fp32_path = str(Path(self._tmp_model_dir.name).joinpath("matmul_fp32_offset.onnx").absolute())
         self.construct_model_matmul(model_fp32_path, symmetric=False)
-        data_reader = self.input_feeds(1, {"input": [100, 52]})
+        data_reader = self.input_feeds(1, {"input": (100, 52)})
         self.quant_test_with_algo("RTN", model_fp32_path, data_reader, 32, False)
 
     @unittest.skipIf(
@@ -275,7 +373,7 @@ class TestOpMatMul4Bits(unittest.TestCase):
             self.skipTest("skip test_smooth_quant since neural_compressor is not installed")
         model_fp32_path = str(Path(self._tmp_model_dir.name).joinpath("matmul_fp32_offset.onnx").absolute())
         self.construct_model_matmul(model_fp32_path, symmetric=False)
-        data_reader = self.input_feeds(1, {"input": [100, 52]})
+        data_reader = self.input_feeds(1, {"input": (100, 52)})
         self.quant_test_with_algo("GPTQ", model_fp32_path, data_reader, 32, False)
 
     @unittest.skipIf(
@@ -286,7 +384,7 @@ class TestOpMatMul4Bits(unittest.TestCase):
             self.skipTest("skip test_hqq_quant since torch is not installed")
         model_fp32_path = str(Path(self._tmp_model_dir.name).joinpath("matmul_fp32_offset.onnx").absolute())
         self.construct_model_matmul(model_fp32_path, symmetric=False)
-        data_reader = self.input_feeds(1, {"input": [100, 52]})
+        data_reader = self.input_feeds(1, {"input": (100, 52)})
         self.quant_test_with_algo("HQQ", model_fp32_path, data_reader, 32, False)
 
 
