@@ -4,7 +4,6 @@
 # license information.
 # --------------------------------------------------------------------------
 
-from typing import List
 
 import numpy as np
 import onnx
@@ -13,7 +12,7 @@ from onnx import TensorProto, helper, numpy_helper
 
 
 # Adapted from bert_model_generator.py
-def get_tensor_and_weight(name: str, shape: List[int], random=False, zeros=False):
+def get_tensor_and_weight(name: str, shape: list[int], random=False, zeros=False):
     low = 0.0
     high = 1.0
     total_elements = 1
@@ -22,7 +21,9 @@ def get_tensor_and_weight(name: str, shape: List[int], random=False, zeros=False
     weights = (
         [np.random.uniform(low, high) for _ in range(total_elements)]
         if random
-        else [0.0] * total_elements if zeros else [1.0] * total_elements
+        else [0.0] * total_elements
+        if zeros
+        else [1.0] * total_elements
     )
     return helper.make_tensor(name, TensorProto.FLOAT, shape, weights), weights
 
