@@ -215,6 +215,52 @@ TEST(CApiTest, TestLoadModelFromArrayWithExternalInitializersFromFileArrayPathRo
 #endif
 }
 
+// The model has external data, Test loading model from array
+// Extra API required to set the external data path
+TEST(CApiTest, TestLoadModelFromArrayWithExternalInitializersViaSetExternalDataPath) {
+  std::string model_file_name = "conv_qdq_external_ini.onnx";
+  std::string external_bin_name = "conv_qdq_external_ini.bin";
+  std::string test_folder = "testdata/";
+  std::string model_path = test_folder + model_file_name;
+  std::vector<char> buffer;
+  ReadFileToBuffer(model_path.c_str(), buffer);
+
+  std::vector<char> external_bin_buffer;
+  std::string external_bin_path = test_folder + external_bin_name;
+  ReadFileToBuffer(external_bin_path.c_str(), external_bin_buffer);
+
+  Ort::SessionOptions so;
+  std::string optimized_model_file_name(model_file_name);
+  auto length = optimized_model_file_name.length();
+  optimized_model_file_name.insert(length - 5, "_opt");
+  std::string optimized_file_path(test_folder + optimized_model_file_name);
+  PathString optimized_file_path_t(optimized_file_path.begin(), optimized_file_path.end());
+
+  //  Dump the optimized model with external data so that it will unpack the external data from the loaded model
+  so.SetOptimizedModelFilePath(optimized_file_path_t.c_str());
+
+  // set the model external file folder path
+  so.AddConfigEntry(kOrtSessionOptionsModelExternalInitializersFileFolderPath, test_folder.c_str());
+
+  std::string opt_bin_file_name(optimized_model_file_name);
+  opt_bin_file_name.replace(optimized_model_file_name.length() - 4, 4, "bin");
+  so.SetGraphOptimizationLevel(GraphOptimizationLevel::ORT_DISABLE_ALL);
+  so.AddConfigEntry(kOrtSessionOptionsOptimizedModelExternalInitializersFileName, opt_bin_file_name.c_str());
+  so.AddConfigEntry(kOrtSessionOptionsOptimizedModelExternalInitializersMinSizeInBytes, "10");
+
+  Ort::Session session(*ort_env.get(), buffer.data(), buffer.size(), so);
+
+  std::string generated_bin_path = test_folder + opt_bin_file_name;
+  std::vector<char> generated_bin_buffer;
+  ReadFileToBuffer(generated_bin_path.c_str(), generated_bin_buffer);
+
+  ASSERT_EQ(external_bin_buffer, generated_bin_buffer);
+
+  // Cleanup.
+  ASSERT_EQ(std::remove(optimized_file_path.c_str()), 0);
+  ASSERT_EQ(std::remove(generated_bin_path.c_str()), 0);
+}
+
 #ifndef _WIN32
 struct FileDescriptorTraits {
   using Handle = int;
