@@ -4,7 +4,6 @@
 #include "core/common/common.h"
 #include "core/session/onnxruntime_cxx_api.h"
 #include "core/session/onnxruntime_session_options_config_keys.h"
-#include "core/optimizer/graph_transformer_level.h"
 #include "gmock/gmock.h"
 
 using namespace onnxruntime;
@@ -15,6 +14,12 @@ TEST(CApiTest, session_options_graph_optimization_level) {
   options.SetGraphOptimizationLevel(ORT_ENABLE_EXTENDED);
 }
 
+TEST(CApiTest, session_options_deterministic_compute) {
+  // Manual validation currently. Check that SetDeterministicCompute in abi_session_options.cc is hit.
+  Ort::SessionOptions options;
+  options.SetDeterministicCompute(true);
+}
+
 #if !defined(ORT_MINIMAL_BUILD) && !defined(ORT_EXTENDED_MINIMAL_BUILD) && !defined(ORT_NO_EXCEPTIONS)
 
 TEST(CApiTest, session_options_oversized_affinity_string) {
