@@ -3,10 +3,7 @@
 
 #include "orttraining/core/framework/torch/dlpack_python.h"
 
-namespace onnxruntime {
-namespace training {
-namespace framework {
-namespace torch {
+namespace onnxruntime::training::framework::torch {
 
 static void DlpackCapsuleDestructor(PyObject* data) {
   DLManagedTensor* dlmanged_tensor = reinterpret_cast<DLManagedTensor*>(
@@ -35,7 +32,4 @@ OrtValue FromDlpack(PyObject* dlpack_tensor, const bool is_bool_tensor) {
   return ort_value;
 }
 
-}  // namespace torch
-}  // namespace framework
-}  // namespace training
-}  // namespace onnxruntime
+}  // namespace onnxruntime::training::framework::torch
