@@ -40,7 +40,6 @@ than CUDA forward + backward.
 """
 
 import math
-from typing import List, Tuple
 
 import torch
 import triton
@@ -793,7 +792,7 @@ def flash_attn_forward(q, k, v, bias=None, **kwargs):
         elif bias.shape[2:] == (seqlen_q, seqlen_k):
             bias_type = "matrix"
         else:
-            raise RuntimeError("Last 2 dimensions of bias must be (1, seqlen_k)" " or (seqlen_q, seqlen_k)")
+            raise RuntimeError("Last 2 dimensions of bias must be (1, seqlen_k) or (seqlen_q, seqlen_k)")
         bias = bias.expand(batch, nheads, seqlen_q, seqlen_k)
     bias_strides = (bias.stride(0), bias.stride(1), bias.stride(2)) if has_bias else (0, 0, 0)
 
@@ -903,7 +902,7 @@ def flash_attn_backward(do, q, k, v, o, lse, bias=None, **kwargs):
         elif bias.shape[2:] == (seqlen_q, seqlen_k):
             bias_type = "matrix"
         else:
-            raise RuntimeError("Last 2 dimensions of bias must be (1, seqlen_k)" " or (seqlen_q, seqlen_k)")
+            raise RuntimeError("Last 2 dimensions of bias must be (1, seqlen_k) or (seqlen_q, seqlen_k)")
         bias = bias.expand(batch, nheads, seqlen_q, seqlen_k)
     bias_strides = (bias.stride(0), bias.stride(1), bias.stride(2)) if has_bias else (0, 0, 0)
 
@@ -1009,7 +1008,7 @@ def _make_flash_attention_nodes(
 
 
 # Without causal mask, without Dropout. For example, BERT model in HuggingFace.
-_PATTERN_0: List[Tuple[str, bool, List[Tuple[int, int, int]]]] = [
+_PATTERN_0: list[tuple[str, bool, list[tuple[int, int, int]]]] = [
     ("MatMul", False, []),  # 0
     ("Transpose", True, [(0, 0, 0)]),  # 1
     ("Transpose", True, [(0, 0, 1)]),  # 2
@@ -1034,7 +1033,7 @@ _PATTERN_0: List[Tuple[str, bool, List[Tuple[int, int, int]]]] = [
 ]
 
 
-def _optimize_for_pattern_0(matcher: GraphMatcher, idx: int, nodes: List[NodeProto]):
+def _optimize_for_pattern_0(matcher: GraphMatcher, idx: int, nodes: list[NodeProto]):
     # Check forward only as the backward is expected to be consistent if it's built correctly.
     scale_value = matcher.get_constant_value(nodes[3].input[1])
     if not (
@@ -1063,7 +1062,7 @@ def _optimize_for_pattern_0(matcher: GraphMatcher, idx: int, nodes: List[NodePro
 
 
 # llama2+peft, k doesn't require grad.
-_PATTERN_1: List[Tuple[str, bool, List[Tuple[int, int, int]]]] = [
+_PATTERN_1: list[tuple[str, bool, list[tuple[int, int, int]]]] = [
     ("MatMul", False, []),  # 0
     ("Transpose", True, [(0, 0, 1)]),  # 1
     ("Div", False, [(0, 0, 0)]),  # 2
@@ -1087,7 +1086,7 @@ _PATTERN_1: List[Tuple[str, bool, List[Tuple[int, int, int]]]] = [
 ]
 
 
-def _optimize_for_pattern_1(matcher: GraphProto, idx: int, nodes: List[NodeProto]):
+def _optimize_for_pattern_1(matcher: GraphProto, idx: int, nodes: list[NodeProto]):
     # Check forward only as the backward is expected to be consistent if it's built correctly.
     scale_value = matcher.get_constant_value(nodes[2].input[1])
     if not (
@@ -1138,7 +1137,7 @@ def _optimize_for_pattern_1(matcher: GraphProto, idx: int, nodes: List[NodeProto
 
 
 # llama2+peft, k requires grad.
-_PATTERN_2: List[Tuple[str, bool, List[Tuple[int, int, int]]]] = [
+_PATTERN_2: list[tuple[str, bool, list[tuple[int, int, int]]]] = [
     ("MatMul", False, []),  # 0
     ("Transpose", True, [(0, 0, 1)]),  # 1
     ("Div", False, [(0, 0, 0)]),  # 2
@@ -1164,7 +1163,7 @@ _PATTERN_2: List[Tuple[str, bool, List[Tuple[int, int, int]]]] = [
 ]
 
 
-def _aptimize_for_pattern_2(matcher: GraphProto, idx: int, nodes: List[NodeProto]):
+def _aptimize_for_pattern_2(matcher: GraphProto, idx: int, nodes: list[NodeProto]):
     # Check forward only as the backward is expected to be consistent if it's built correctly.
     scale_value = matcher.get_constant_value(nodes[2].input[1])
     if not (
