@@ -5,7 +5,6 @@
 
 
 import sys
-from typing import Optional, Tuple
 
 import torch
 
@@ -52,7 +51,7 @@ class SubscriberBase:
     With this, the overall flow can be traced as a data flow graph (DAG).
     """
 
-    def __init__(self, start_step: Optional[int], end_step: Optional[int]):
+    def __init__(self, start_step: int | None, end_step: int | None):
         """
         Steps in [start_step, end_step) will run the subscriber's actions, and other steps will skip.
         If start_step is None, 0 is given; if end_step is None, sys.maxsize is given.
@@ -66,7 +65,7 @@ class SubscriberBase:
         module: torch.nn.Module,
         args: ORTModelInputOutputType,
         kwargs: ORTModelInputOutputType,
-    ) -> Tuple[ORTModelInputOutputType, ORTModelInputOutputType]:
+    ) -> tuple[ORTModelInputOutputType, ORTModelInputOutputType]:
         """This function is called inside the nn.Module's pre-forward hook.
 
         Args:
@@ -91,7 +90,7 @@ class SubscriberBase:
         module: torch.nn.Module,
         args: ORTModelInputOutputType,
         kwargs: ORTModelInputOutputType,
-    ) -> Tuple[ORTModelInputOutputType, ORTModelInputOutputType]:
+    ) -> tuple[ORTModelInputOutputType, ORTModelInputOutputType]:
         return args, kwargs
 
     def pre_forward_tensor_apply(
@@ -121,7 +120,7 @@ class SubscriberBase:
         module: torch.nn.Module,
         args: ORTModelInputOutputType,
         outputs: ORTModelInputOutputType,
-    ) -> Tuple[ORTModelInputOutputType, ORTModelInputOutputType]:
+    ) -> tuple[ORTModelInputOutputType, ORTModelInputOutputType]:
         """This function is called inside the nn.Module's post-forward hook.
 
         Args:
@@ -146,7 +145,7 @@ class SubscriberBase:
         module: torch.nn.Module,
         args: ORTModelInputOutputType,
         outputs: ORTModelInputOutputType,
-    ) -> Tuple[ORTModelInputOutputType, ORTModelInputOutputType]:
+    ) -> tuple[ORTModelInputOutputType, ORTModelInputOutputType]:
         return args, outputs
 
     def post_forward_tensor_apply(
@@ -179,7 +178,7 @@ class SubscriberBase:
         module: torch.nn.Module,
         args: ORTModelInputOutputType,
         outputs: ORTModelInputOutputType,
-    ) -> Tuple[ORTModelInputOutputType, ORTModelInputOutputType]:
+    ) -> tuple[ORTModelInputOutputType, ORTModelInputOutputType]:
         """This function is called inside the outmost nn.Module's post-forward hook.
 
         Args:
@@ -204,7 +203,7 @@ class SubscriberBase:
         module: torch.nn.Module,
         args: ORTModelInputOutputType,
         outputs: ORTModelInputOutputType,
-    ) -> Tuple[ORTModelInputOutputType, ORTModelInputOutputType]:
+    ) -> tuple[ORTModelInputOutputType, ORTModelInputOutputType]:
         return args, outputs
 
     def _need_skip_step(self, current_step: int) -> bool:
