@@ -256,7 +256,10 @@ def test_data_flatten_and_unflatten(input_output_map, flag: int):
 
     if flag == 0:
         out, schema = extract_data_and_schema(raw_data)
-        assert all([torch.allclose(o, d) if isinstance(o, torch.Tensor) else o == d for o, d in zip(out, flatten_data)])
+        assert all(
+            torch.allclose(o, d) if isinstance(o, torch.Tensor) else o == d
+            for o, d in zip(out, flatten_data, strict=False)
+        )
         if not isinstance(raw_data, torch.Tensor):
             assert type(schema) is type(raw_data)
 
@@ -274,10 +277,8 @@ def test_data_flatten_and_unflatten(input_output_map, flag: int):
             assert raw_data == schema
         else:
             assert all(
-                [
-                    torch.allclose(o, d) if isinstance(o, torch.Tensor) else o == d
-                    for o, d in zip(out, flatten_data_constant_as_tensor)
-                ]
+                torch.allclose(o, d) if isinstance(o, torch.Tensor) else o == d
+                for o, d in zip(out, flatten_data_constant_as_tensor, strict=False)
             )
 
     elif flag == 1:
