@@ -5,6 +5,12 @@ from .get_azcopy import get_azcopy  # noqa: F401
 from .logger import get_logger
 from .platform_helpers import is_linux, is_macOS, is_windows  # noqa: F401
 from .run import run  # noqa: F401
+from .vcpkg_helpers import (  # noqa: F401
+    generate_android_triplets,
+    generate_posix_triplets,
+    generate_vcpkg_triplets_for_emscripten,
+    generate_windows_triplets,
+)
 
 try:
     import flatbuffers  # noqa: F401
