@@ -170,8 +170,8 @@ void TensorToVideoFrameConverter::DX12TensorToVideoFrame(
           D3D12_RESOURCE_DESC cachedTextureDesc = output_resource_->GetDesc();
 
           if (cachedTextureDesc.Width != videoFrameTextureDesc.Width ||
-                        cachedTextureDesc.Height != videoFrameTextureDesc.Height ||
-                        cachedTextureDesc.Format != videoFrameTextureDesc.Format) {
+              cachedTextureDesc.Height != videoFrameTextureDesc.Height ||
+              cachedTextureDesc.Format != videoFrameTextureDesc.Format) {
             // The dimensions or format don't match, so we need to re-create our texture
             output_resource_ = CreateShareableD3D12Texture(videoFrameTextureDesc, pDeviceCache->GetD3D12Device());
             D3D11_cached_texture_ = ShareD3D12Texture(output_resource_.Get(), pDeviceCache->GetD3D11Device());
@@ -196,12 +196,12 @@ void TensorToVideoFrameConverter::DX12TensorToVideoFrame(
         UINT comPtrSize = static_cast<UINT>(sizeof(spSharedD3D11Texture.GetAddressOf()));
         UINT handleSize = static_cast<UINT>(sizeof(sharedHandle));
 
-        if ((FAILED(spVideoFrameTexture->GetPrivateData(
-                         _d3d11TextureGUID, &comPtrSize, spSharedD3D11Texture.GetAddressOf()
-                     )) ||
-                     !spSharedD3D11Texture.Get()) ||
-                    (FAILED(spVideoFrameTexture->GetPrivateData(_handleGUID, &handleSize, &sharedHandle)) ||
-                     sharedHandle != shared_handle_)) {
+        if ((FAILED(
+               spVideoFrameTexture->GetPrivateData(_d3d11TextureGUID, &comPtrSize, spSharedD3D11Texture.GetAddressOf())
+             ) ||
+             !spSharedD3D11Texture.Get()) ||
+            (FAILED(spVideoFrameTexture->GetPrivateData(_handleGUID, &handleSize, &sharedHandle)) ||
+             sharedHandle != shared_handle_)) {
           // Create a new shared texture that we cache on the video frame texture
           output_resource_ = CreateShareableD3D12Texture(videoFrameTextureDesc, pDeviceCache->GetD3D12Device());
           spSharedD3D11Texture = ShareD3D12Texture(output_resource_.Get(), spTextureDevice.Get());
@@ -366,7 +366,8 @@ void TensorToVideoFrameConverter::SoftwareTensorToVideoFrame(
   wgdx::Direct3D11::IDirect3DSurface spOutputSurface = pDestVideoFrame.Direct3DSurface();
 
   // only one of softwarebitmap or direct3Dsurface should be non-null
-  if ((spOutputSoftwareBitmap == nullptr && spOutputSurface == nullptr) || (spOutputSoftwareBitmap != nullptr && spOutputSurface != nullptr)) {
+  if ((spOutputSoftwareBitmap == nullptr && spOutputSurface == nullptr) ||
+      (spOutputSoftwareBitmap != nullptr && spOutputSurface != nullptr)) {
     WINML_THROW_HR(E_INVALIDARG);
   }
   if (spOutputSoftwareBitmap) {
@@ -382,7 +383,10 @@ void TensorToVideoFrameConverter::SoftwareTensorToVideoFrame(
   if (_winmli::NeedsVideoFrameConversion(
         pDestVideoFrame, {}, {0, 0, (UINT32)tensorWidth, (UINT32)tensorHeight}, tensorWidth, tensorHeight
       )) {
-    if (converted_video_frame_ == nullptr || _winmli::NeedsVideoFrameConversion(converted_video_frame_, {}, {0, 0, (UINT32)tensorWidth, (UINT32)tensorHeight}, tensorWidth, tensorHeight)) {
+    if (converted_video_frame_ == nullptr ||
+        _winmli::NeedsVideoFrameConversion(
+          converted_video_frame_, {}, {0, 0, (UINT32)tensorWidth, (UINT32)tensorHeight}, tensorWidth, tensorHeight
+        )) {
       converted_video_frame_ = wm::VideoFrame::CreateWithSoftwareBitmap(
         wgi::SoftwareBitmap(wgi::BitmapPixelFormat::Bgra8, tensorWidth, tensorHeight)
       );
@@ -499,9 +503,9 @@ void TensorToVideoFrameConverter::ConvertGPUTensorToDX12Texture(
   outputResourceDesc.Flags = D3D12_RESOURCE_FLAG_ALLOW_UNORDERED_ACCESS;
 
   if (!UAV_resource_ || outputDesc.Format != UAV_resource_->GetDesc().Format ||
-        outputDesc.Width != UAV_resource_->GetDesc().Width || outputDesc.Height != UAV_resource_->GetDesc().Height) {
+      outputDesc.Width != UAV_resource_->GetDesc().Width || outputDesc.Height != UAV_resource_->GetDesc().Height) {
     WINML_THROW_IF_FAILED(device_cache.GetD3D12Device()->CreateCommittedResource(
-      &CD3DX12_HEAP_PROPERTIES(D3D12_HEAP_TYPE_DEFAULT),
+      unmove_ptr(CD3DX12_HEAP_PROPERTIES(D3D12_HEAP_TYPE_DEFAULT)),
       D3D12_HEAP_FLAG_NONE,
       &outputResourceDesc,
       D3D12_RESOURCE_STATE_UNORDERED_ACCESS,
@@ -599,31 +603,31 @@ void TensorToVideoFrameConverter::ConvertGPUTensorToDX12Texture(
 
     command_list_->ResourceBarrier(
       1,
-      &CD3DX12_RESOURCE_BARRIER::Transition(
+      unmove_ptr(CD3DX12_RESOURCE_BARRIER::Transition(
         pInputResource, D3D12_RESOURCE_STATE_UNORDERED_ACCESS, D3D12_RESOURCE_STATE_NON_PIXEL_SHADER_RESOURCE
-      )
+      ))
     );
     command_list_->Dispatch(dispatchWidth, dispatchHeight, 1);
     command_list_->ResourceBarrier(
       1,
-      &CD3DX12_RESOURCE_BARRIER::Transition(
+      unmove_ptr(CD3DX12_RESOURCE_BARRIER::Transition(
         pInputResource, D3D12_RESOURCE_STATE_NON_PIXEL_SHADER_RESOURCE, D3D12_RESOURCE_STATE_UNORDERED_ACCESS
-      )
+      ))
     );
 
     // Copy the UAV data to the output resource after detensorization
     command_list_->ResourceBarrier(
       1,
-      &CD3DX12_RESOURCE_BARRIER::Transition(
+      unmove_ptr(CD3DX12_RESOURCE_BARRIER::Transition(
         UAV_resource_.Get(), D3D12_RESOURCE_STATE_UNORDERED_ACCESS, D3D12_RESOURCE_STATE_COPY_SOURCE
-      )
+      ))
     );
     command_list_->CopyResource(pOutputResource, UAV_resource_.Get());
     command_list_->ResourceBarrier(
       1,
-      &CD3DX12_RESOURCE_BARRIER::Transition(
+      unmove_ptr(CD3DX12_RESOURCE_BARRIER::Transition(
         UAV_resource_.Get(), D3D12_RESOURCE_STATE_COPY_SOURCE, D3D12_RESOURCE_STATE_UNORDERED_ACCESS
-      )
+      ))
     );
 
     WINML_THROW_IF_FAILED(command_list_->Close());
@@ -657,9 +661,9 @@ void TensorToVideoFrameConverter::ConvertGPUTensorToSoftwareBitmap(
   // TODO: Make an allocator for readback heaps
   if (!readback_heap_ || readback_heap_->GetDesc().Width < singleVideoFramebufferSize) {
     WINML_THROW_IF_FAILED(device_cache.GetD3D12Device()->CreateCommittedResource(
-      &CD3DX12_HEAP_PROPERTIES(D3D12_HEAP_TYPE_READBACK),
+      unmove_ptr(CD3DX12_HEAP_PROPERTIES(D3D12_HEAP_TYPE_READBACK)),
       D3D12_HEAP_FLAG_NONE,
-      &CD3DX12_RESOURCE_DESC::Buffer(singleVideoFramebufferSize),
+      unmove_ptr(CD3DX12_RESOURCE_DESC::Buffer(singleVideoFramebufferSize)),
       D3D12_RESOURCE_STATE_COPY_DEST,
       nullptr,
       IID_PPV_ARGS(&readback_heap_)
@@ -689,12 +693,14 @@ void TensorToVideoFrameConverter::ConvertGPUTensorToSoftwareBitmap(
   device_cache.SyncD3D12ToCPU();
 
   void* pCPUTensorBuffer = nullptr;
-  WINML_THROW_IF_FAILED(readback_heap_->Map(0, &CD3DX12_RANGE(0, singleVideoFramebufferSize), &pCPUTensorBuffer));
+  WINML_THROW_IF_FAILED(
+    readback_heap_->Map(0, unmove_ptr(CD3DX12_RANGE(0, singleVideoFramebufferSize)), &pCPUTensorBuffer)
+  );
 
   // We avoid the Video Frame pipeline by manually downloading the GPU data to the CPU and detensorize while we are filling the readback heap
   ConvertCPUTensorToSoftwareBitmap(pCPUTensorBuffer, tensorDesc, softwareBitmap);
 
-  readback_heap_->Unmap(0, &CD3DX12_RANGE(0, 0));
+  readback_heap_->Unmap(0, unmove_ptr(CD3DX12_RANGE(0, 0)));
 }
 
 void TensorToVideoFrameConverter::ConvertBatchedDX12TensorToBuffers(
@@ -708,9 +714,9 @@ void TensorToVideoFrameConverter::ConvertBatchedDX12TensorToBuffers(
   // TODO: Make an allocator for readback heaps
   if (!readback_heap_ || readback_heap_->GetDesc().Width < buffer_size_in_bytes) {
     WINML_THROW_IF_FAILED(device_cache.GetD3D12Device()->CreateCommittedResource(
-      &CD3DX12_HEAP_PROPERTIES(D3D12_HEAP_TYPE_READBACK),
+      unmove_ptr(CD3DX12_HEAP_PROPERTIES(D3D12_HEAP_TYPE_READBACK)),
       D3D12_HEAP_FLAG_NONE,
-      &CD3DX12_RESOURCE_DESC::Buffer(buffer_size_in_bytes),
+      unmove_ptr(CD3DX12_RESOURCE_DESC::Buffer(buffer_size_in_bytes)),
       D3D12_RESOURCE_STATE_COPY_DEST,
       nullptr,
       IID_PPV_ARGS(&readback_heap_)
@@ -733,9 +739,9 @@ void TensorToVideoFrameConverter::ConvertBatchedDX12TensorToBuffers(
   device_cache.SyncD3D12ToCPU();
 
   byte* readback_buffer = nullptr;
-  WINML_THROW_IF_FAILED(
-    readback_heap_->Map(0, &CD3DX12_RANGE(0, buffer_size_in_bytes), reinterpret_cast<void**>(&readback_buffer))
-  );
+  WINML_THROW_IF_FAILED(readback_heap_->Map(
+    0, unmove_ptr(CD3DX12_RANGE(0, buffer_size_in_bytes)), reinterpret_cast<void**>(&readback_buffer)
+  ));
   auto readback_buffer_span = gsl::span<byte>(readback_buffer, buffer_size_in_bytes);
   _winml::StoreSpanIntoDisjointBuffers(
     buffers.size(),
@@ -748,7 +754,7 @@ void TensorToVideoFrameConverter::ConvertBatchedDX12TensorToBuffers(
     readback_buffer_span
   );
 
-  readback_heap_->Unmap(0, &CD3DX12_RANGE(0, 0));
+  readback_heap_->Unmap(0, unmove_ptr(CD3DX12_RANGE(0, 0)));
 }
 
 D3D12_SHADER_RESOURCE_VIEW_DESC TensorToVideoFrameConverter::CreateSRVDescriptor(
