@@ -114,14 +114,14 @@ std::unordered_map<std::string, std::string> disabledTests({
   {             "coreml_DecisionTreeClassifier_OpenML_1464_blood_transfusion_opset7",                                                                                           disabledTestDefaultReason},
   {                                                  "coreml_AgeNet_ImageNet_opset7",                                                                                           disabledTestDefaultReason},
 
- // GPU specific cases:
+  // GPU specific cases:
 
   // ONNX zoo models
   {                                                          "mask_rcnn_opset10_GPU",
    "Bug 31005388: mask_rcnn opset 10 onnx zoo model fails to evaluate on DirectML https://microsoft.visualstudio.com/OS/_workitems/edit/31005388"                                                        },
   {                                                        "faster_rcnn_opset10_GPU",
    "Bug 31005511: Failed to extract tensor data from evaluate result of faster_rcnn opset 10 model in DirectML https://microsoft.visualstudio.com/OS/_workitems/edit/31005511"                           },
- // ONNX model zoo's int8/qdq models generally do not work on CPUs that lack 8-bit instructions.
+  // ONNX model zoo's int8/qdq models generally do not work on CPUs that lack 8-bit instructions.
   {                                                         "YOLOv3_12_int8_opset12",                                                                                           disabledTestDefaultReason},
   {                                                            "VGG_16_int8_opset12",                                                                                           disabledTestDefaultReason},
   {                                                               "SSD_int8_opset12",                                                                                           disabledTestDefaultReason},
@@ -137,7 +137,7 @@ std::unordered_map<std::string, std::string> disabledTests({
   {                                                 "EfficientNet_Lite4_qdq_opset11",                                                                                           disabledTestDefaultReason},
   {                                                "EfficientNet_Lite4_int8_opset11",                                                                                           disabledTestDefaultReason},
 
- // Tier 2 models
+  // Tier 2 models
   {                                               "fp16_test_tiny_yolov2_opset7_GPU",
    "Bug 31005780: Result of fp16_test_tiny_yolov2_opset7 and fp16_coreml_FNS_Candy_opset7 models on DirectML aren't as accurate as on CPU https://microsoft.visualstudio.com/OS/_workitems/edit/31005780"},
   {                                                    "fp16_tiny_yolov2_opset8_GPU",
@@ -146,8 +146,6 @@ std::unordered_map<std::string, std::string> disabledTests({
    "Bug 31005780: Result of fp16_test_tiny_yolov2_opset7 and fp16_coreml_FNS_Candy_opset7 models on DirectML aren't as accurate as on CPU https://microsoft.visualstudio.com/OS/_workitems/edit/31005780"},
   {                                           "mlperf_ssd_mobilenet_300_opset10_GPU",
    "Bug 31005624: mlperf_ssd_mobilenet_300 opset 10 model fails to evaluate in DirectML https://microsoft.visualstudio.com/OS/_workitems/edit/31005624"                                                  },
-  {                                           "mlperf_ssd_resnet34_1200_opset10_GPU",
-   "Bug 31005624: mlperf_ssd_resnet34_1200_opset10_GPU opset 10 model fails to evaluate in DirectML https://microsoft.visualstudio.com/OS/_workitems/edit/31005624"                                      },
 });
 
 /*
